(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('rxjs'), require('@angular/common/http'), require('@angular/animations'), require('@angular/common'), require('@angular/forms'), require('@angular/material/slide-toggle'), require('@angular/material/chips'), require('@angular/material/button'), require('@angular/material/icon'), require('@angular/material/checkbox'), require('@angular/material/select'), require('@angular/material/core'), require('@angular/material/input'), require('@angular/platform-browser/animations'), require('rxjs/operators'), require('lodash'), require('@angular/cdk/drag-drop'), require('@angular/material/progress-bar'), require('@angular/router')) :
    typeof define === 'function' && define.amd ? define('ngx-dynamic-dashboard', ['exports', '@angular/core', 'rxjs', '@angular/common/http', '@angular/animations', '@angular/common', '@angular/forms', '@angular/material/slide-toggle', '@angular/material/chips', '@angular/material/button', '@angular/material/icon', '@angular/material/checkbox', '@angular/material/select', '@angular/material/core', '@angular/material/input', '@angular/platform-browser/animations', 'rxjs/operators', 'lodash', '@angular/cdk/drag-drop', '@angular/material/progress-bar', '@angular/router'], factory) :
    (global = global || self, factory(global['ngx-dynamic-dashboard'] = {}, global.ng.core, global.rxjs, global.ng.common.http, global.ng.animations, global.ng.common, global.ng.forms, global.ng.material.slideToggle, global.ng.material.chips, global.ng.material.button, global.ng.material.icon, global.ng.material.checkbox, global.ng.material.select, global.ng.material.core, global.ng.material.input, global.ng.platformBrowser.animations, global.rxjs.operators, global.lodash, global.ng.cdk.dragDrop, global.ng.material.progressBar, global.ng.router));
}(this, (function (exports, i0, rxjs, i1, animations, common, forms, slideToggle, chips, button, icon, checkbox, select, core, input, animations$1, operators, lodash, dragDrop, progressBar, router) { 'use strict';

    var OptionsService = /** @class */ (function () {
        function OptionsService() {
            this.optionsCollectionName = 'dashboardOptions';
            this.defaultOptions = {
                'enableHover': false,
                'displayGadgetOptionsInSideBar': false
            };
            this.globalOptionsChangeEventSubject = new rxjs.Subject();
        }
        OptionsService.prototype.getBoardOptions = function () {
            var databaseOptions = JSON.parse(localStorage.getItem(this.optionsCollectionName));
            if (databaseOptions == null) {
                this.persistDefautBoardOptions();
                return this.defaultOptions;
            }
            else {
                return databaseOptions;
            }
        };
        OptionsService.prototype.setBoardOptions = function (options) {
            /**
             * Todo this will need to change to support the update to individual options. Currently there is only one
             * option but once there is more than one this method must change to take the input and update just that
             * property of the options object.
             */
            localStorage.removeItem(this.optionsCollectionName);
            /**
             *  Raise an event to listeners, primarily the gadgets, when the global options change. The listeners can use
             * the event to change their behavior
             */
            this.globalOptionsChangeEventSubject.next(options);
            return localStorage.setItem(this.optionsCollectionName, JSON.stringify(options));
        };
        /**
         * The gadget-base can use this method to subscribe to events that are created when the global options change.
         */
        OptionsService.prototype.listenForGlobalOptionsChanges = function () {
            return this.globalOptionsChangeEventSubject.asObservable();
        };
        OptionsService.prototype.persistDefautBoardOptions = function () {
            localStorage.setItem(this.optionsCollectionName, JSON.stringify(this.defaultOptions));
        };
        return OptionsService;
    }());
    OptionsService.decorators = [
        { type: i0.Injectable }
    ];
    OptionsService.ctorParameters = function () { return []; };

    var Message = /** @class */ (function () {
        function Message(content, id, style) {
            this.dismissed = false;
            this.content = content;
            this.style = style || 'info';
            this.id = id;
        }
        return Message;
    }());

    var ToastService = /** @class */ (function () {
        function ToastService() {
            this.messages = [];
        }
        ToastService.prototype.getMessages = function () {
            return this.messages;
        };
        ToastService.prototype.sendMessage = function (content, style) {
            var message = new Message(content, this.messages.length + 1, style);
            this.messages.push(message);
        };
        ToastService.prototype.dismissMessage = function (messageKey) {
            this.purgeDismissedMessages();
            this.messages.forEach(function (message) {
                if (message.id === messageKey) {
                    message.dismissed = true;
                }
            });
        };
        ToastService.prototype.purgeDismissedMessages = function () {
            this.messages.forEach(function (message, index, object) {
                if (message.dismissed) {
                    object.splice(index, 1);
                }
            });
        };
        return ToastService;
    }());
    ToastService.decorators = [
        { type: i0.Injectable }
    ];
    ToastService.ctorParameters = function () { return []; };

    /**
     * Created by jayhamilton on 1/24/17.
     */
    var OptionsConfigurationTabComponent = /** @class */ (function () {
        function OptionsConfigurationTabComponent(_optionsService, _toastService) {
            this._optionsService = _optionsService;
            this._toastService = _toastService;
            this.enableHover = this._optionsService.getBoardOptions()['enableHover'];
            this.displayGadgetOptionsInSideBar = this._optionsService.getBoardOptions()['displayGadgetOptionsInSideBar'];
        }
        OptionsConfigurationTabComponent.prototype.onHooverOptionChange = function (value) {
            this._optionsService.setBoardOptions({
                'enableHover': value['checked'],
                'displayGadgetOptionsInSideBar': this.displayGadgetOptionsInSideBar
            });
            this._toastService.sendMessage('The board configuration has changed!', null);
        };
        OptionsConfigurationTabComponent.prototype.onDisplayGadgetOptionsInSideBarChange = function (value) {
            this._optionsService.setBoardOptions({
                'enableHover': this.enableHover,
                'displayGadgetOptionsInSideBar': value['checked']
            });
            this._toastService.sendMessage('The board configuration has changed!', null);
        };
        return OptionsConfigurationTabComponent;
    }());
    OptionsConfigurationTabComponent.decorators = [
        { type: i0.Component, args: [{
                    selector: 'dashboard-options-config-tab',
                    moduleId: module.id,
                    template: "<br>\nThe following options control various aspects of the board's and gadget's behavior. Changes take effect immediately.\n<br>\n<br>\n\n<mat-slide-toggle\n    (change)=\"onHooverOptionChange($event)\"\n    [checked]=\"enableHover\">\n    Always show the gadget buttons in the title. By default buttons show/hide when you hover over the title.\n</mat-slide-toggle>\n<br>\n<br>\n<mat-slide-toggle\n    (change)=\"onDisplayGadgetOptionsInSideBarChange($event)\"\n    [checked]=\"displayGadgetOptionsInSideBar\">\n    (Experimental) When set the options are displayed in the side bar. Otherwise it is displayed within the gadget.\n</mat-slide-toggle>\n",
                    styles: [""]
                },] }
    ];
    OptionsConfigurationTabComponent.ctorParameters = function () { return [
        { type: OptionsService },
        { type: ToastService }
    ]; };

    var defaultBoard = {
        title: 'Dashboard',
        structure: '6-6',
        id: 5,
        boardInstanceId: 0,
        rows: [
            {
                columns: [
                    {
                        styleClass: 'six wide',
                        gadgets: []
                    },
                    {
                        styleClass: 'six wide',
                        gadgets: []
                    }
                ]
            }
        ]
    };

    var sampleBoardCollection = {
        'board': [{
                'title': 'Board Sample 1',
                'structure': '3-6-3',
                'id': 9,
                'boardInstanceId': 1,
                'rows': [
                    {
                        'columns': [
                            {
                                'styleClass': 'three wide',
                                'gadgets': [
                                    {
                                        'componentType': 'NewsGadgetComponent',
                                        'name': 'News',
                                        'description': 'What\'s new',
                                        'icon': 'images/news.png',
                                        'instanceId': 1500253814523,
                                        'tags': [
                                            {
                                                'facet': 'Informational',
                                                'name': 'news'
                                            },
                                            {
                                                'facet': 'List',
                                                'name': 'news'
                                            }
                                        ],
                                        'config': {
                                            'propertyPages': [
                                                {
                                                    'displayName': 'Run',
                                                    'groupId': 'run',
                                                    'position': 10,
                                                    'properties': [
                                                        {
                                                            'value': 'news',
                                                            'key': 'endpoint',
                                                            'label': 'News URL',
                                                            'required': false,
                                                            'order': 3,
                                                            'controlType': 'dynamicdropdown'
                                                        },
                                                        {
                                                            'value': 'News',
                                                            'key': 'title',
                                                            'label': 'Title',
                                                            'required': false,
                                                            'order': 1,
                                                            'controlType': 'textbox'
                                                        },
                                                        {
                                                            'value': 2,
                                                            'key': 'instanceId',
                                                            'required': false,
                                                            'order': -1,
                                                            'controlType': 'hidden'
                                                        }
                                                    ]
                                                }
                                            ]
                                        }
                                    }
                                ]
                            },
                            {
                                'styleClass': 'six wide',
                                'gadgets': [
                                    {
                                        'componentType': 'CPUGadgetComponent',
                                        'name': 'CPU Chart',
                                        'description': 'Monitors CPU utilization for CDM.',
                                        'icon': 'images/cpu.png',
                                        'instanceId': 1499912922910,
                                        'tags': [
                                            {
                                                'facet': 'Performance',
                                                'name': 'real-time'
                                            },
                                            {
                                                'facet': 'Chart',
                                                'name': 'bar'
                                            }
                                        ],
                                        'config': {
                                            'propertyPages': [
                                                {
                                                    'displayName': 'Run',
                                                    'groupId': 'run',
                                                    'position': 10,
                                                    'properties': [
                                                        {
                                                            'value': 'CPU Utilization',
                                                            'key': 'title',
                                                            'label': 'Title',
                                                            'required': false,
                                                            'order': 1,
                                                            'controlType': 'textbox'
                                                        },
                                                        {
                                                            'value': 'Carlosappliance - Process Monitor',
                                                            'key': 'endpoint',
                                                            'label': 'API Endpoints',
                                                            'required': false,
                                                            'order': 3,
                                                            'controlType': 'dynamicdropdown'
                                                        },
                                                        {
                                                            'value': 999,
                                                            'key': 'instanceId',
                                                            'required': false,
                                                            'order': -1,
                                                            'controlType': 'hidden'
                                                        }
                                                    ]
                                                },
                                                {
                                                    'displayName': 'Chart',
                                                    'groupId': 'chart',
                                                    'position': 11,
                                                    'properties': [
                                                        {
                                                            'value': true,
                                                            'key': 'chart_properties',
                                                            'label': 'Show chart details',
                                                            'required': false,
                                                            'order': 3,
                                                            'controlType': 'checkbox'
                                                        }
                                                    ]
                                                }
                                            ]
                                        }
                                    },
                                    {
                                        'componentType': 'TrendGadgetComponent',
                                        'name': 'Trend',
                                        'description': 'General trends.',
                                        'icon': 'images/trend.png',
                                        'instanceId': 1499912901569,
                                        'tags': [
                                            {
                                                'facet': 'Performance',
                                                'name': 'trend'
                                            },
                                            {
                                                'facet': 'Chart',
                                                'name': 'area'
                                            }
                                        ],
                                        'config': {
                                            'propertyPages': [
                                                {
                                                    'displayName': 'Run',
                                                    'groupId': 'run',
                                                    'position': 10,
                                                    'properties': [
                                                        {
                                                            'value': 'Devappliance - ECX',
                                                            'key': 'endpoint',
                                                            'label': 'API Endpoints',
                                                            'required': false,
                                                            'order': 2,
                                                            'controlType': 'dynamicdropdown'
                                                        },
                                                        {
                                                            'value': 'Trend',
                                                            'key': 'title',
                                                            'label': 'Title',
                                                            'required': false,
                                                            'order': 1,
                                                            'controlType': 'textbox'
                                                        },
                                                        {
                                                            'value': 2,
                                                            'key': 'instanceId',
                                                            'required': false,
                                                            'order': -1,
                                                            'controlType': 'hidden'
                                                        }
                                                    ]
                                                },
                                                {
                                                    'displayName': 'Chart',
                                                    'groupId': 'chart',
                                                    'position': 11,
                                                    'properties': [
                                                        {
                                                            'value': true,
                                                            'key': 'chart_properties',
                                                            'label': 'Show chart details',
                                                            'required': false,
                                                            'order': 3,
                                                            'controlType': 'checkbox'
                                                        }
                                                    ]
                                                }
                                            ]
                                        }
                                    }
                                ]
                            },
                            {
                                'styleClass': 'three wide',
                                'gadgets': []
                            }
                        ]
                    }
                ]
            }, {
                'title': 'Board Sample 2',
                'structure': '4-4-4-4/8-4-4',
                'id': 8,
                'boardInstanceId': 2,
                'rows': [
                    {
                        'columns': [
                            {
                                'styleClass': 'four wide',
                                'gadgets': [
                                    {
                                        'componentType': 'StatisticGadgetComponent',
                                        'name': 'Statistic',
                                        'description': 'General statistic.',
                                        'icon': 'images/statistic.png',
                                        'instanceId': 1499912861630,
                                        'tags': [
                                            {
                                                'facet': 'Statistic',
                                                'name': 'counter'
                                            }
                                        ],
                                        'config': {
                                            'propertyPages': [
                                                {
                                                    'displayName': 'Run',
                                                    'groupId': 'run',
                                                    'position': 10,
                                                    'properties': [
                                                        {
                                                            'value': 'database',
                                                            'key': 'resource',
                                                            'label': 'Resource',
                                                            'required': false,
                                                            'order': 2,
                                                            'controlType': 'dropdown',
                                                            'options': [
                                                                {
                                                                    'key': 'vm',
                                                                    'value': 'vm'
                                                                },
                                                                {
                                                                    'key': 'database',
                                                                    'value': 'database'
                                                                },
                                                                {
                                                                    'key': 'volume',
                                                                    'value': 'volume'
                                                                },
                                                                {
                                                                    'key': 'job',
                                                                    'value': 'job'
                                                                }
                                                            ]
                                                        },
                                                        {
                                                            'value': 'Devappliance - ECX',
                                                            'key': 'endpoint',
                                                            'label': 'API Endpoints',
                                                            'required': false,
                                                            'order': 3,
                                                            'controlType': 'dynamicdropdown'
                                                        },
                                                        {
                                                            'value': 'Statistic',
                                                            'key': 'title',
                                                            'label': 'Title',
                                                            'required': false,
                                                            'order': 1,
                                                            'controlType': 'textbox'
                                                        },
                                                        {
                                                            'value': 2,
                                                            'key': 'instanceId',
                                                            'required': true,
                                                            'order': -1,
                                                            'controlType': 'hidden'
                                                        }
                                                    ]
                                                }
                                            ]
                                        }
                                    }
                                ]
                            },
                            {
                                'styleClass': 'four wide',
                                'gadgets': [
                                    {
                                        'componentType': 'StatisticGadgetComponent',
                                        'name': 'Statistic',
                                        'description': 'General statistic.',
                                        'icon': 'images/statistic.png',
                                        'instanceId': 1499912845375,
                                        'tags': [
                                            {
                                                'facet': 'Statistic',
                                                'name': 'counter'
                                            }
                                        ],
                                        'config': {
                                            'propertyPages': [
                                                {
                                                    'displayName': 'Run',
                                                    'groupId': 'run',
                                                    'position': 10,
                                                    'properties': [
                                                        {
                                                            'value': 'vm',
                                                            'key': 'resource',
                                                            'label': 'Resource',
                                                            'required': false,
                                                            'order': 2,
                                                            'controlType': 'dropdown',
                                                            'options': [
                                                                {
                                                                    'key': 'vm',
                                                                    'value': 'vm'
                                                                },
                                                                {
                                                                    'key': 'database',
                                                                    'value': 'database'
                                                                },
                                                                {
                                                                    'key': 'volume',
                                                                    'value': 'volume'
                                                                },
                                                                {
                                                                    'key': 'job',
                                                                    'value': 'job'
                                                                }
                                                            ]
                                                        },
                                                        {
                                                            'value': 'Carlosappliance - Process Monitor',
                                                            'key': 'endpoint',
                                                            'label': 'API Endpoints',
                                                            'required': false,
                                                            'order': 3,
                                                            'controlType': 'dynamicdropdown'
                                                        },
                                                        {
                                                            'value': 'Statistic',
                                                            'key': 'title',
                                                            'label': 'Title',
                                                            'required': false,
                                                            'order': 1,
                                                            'controlType': 'textbox'
                                                        },
                                                        {
                                                            'value': 2,
                                                            'key': 'instanceId',
                                                            'required': true,
                                                            'order': -1,
                                                            'controlType': 'hidden'
                                                        }
                                                    ]
                                                }
                                            ]
                                        }
                                    }
                                ]
                            },
                            {
                                'styleClass': 'four wide',
                                'gadgets': [
                                    {
                                        'componentType': 'StatisticGadgetComponent',
                                        'name': 'Statistic',
                                        'description': 'General statistic.',
                                        'icon': 'images/statistic.png',
                                        'instanceId': 1500251223835,
                                        'tags': [
                                            {
                                                'facet': 'Statistic',
                                                'name': 'counter'
                                            }
                                        ],
                                        'config': {
                                            'propertyPages': [
                                                {
                                                    'displayName': 'Run',
                                                    'groupId': 'run',
                                                    'position': 10,
                                                    'properties': [
                                                        {
                                                            'value': 'volume',
                                                            'key': 'resource',
                                                            'label': 'Resource',
                                                            'required': false,
                                                            'order': 2,
                                                            'controlType': 'dropdown',
                                                            'options': [
                                                                {
                                                                    'key': 'vm',
                                                                    'value': 'vm'
                                                                },
                                                                {
                                                                    'key': 'database',
                                                                    'value': 'database'
                                                                },
                                                                {
                                                                    'key': 'volume',
                                                                    'value': 'volume'
                                                                },
                                                                {
                                                                    'key': 'job',
                                                                    'value': 'job'
                                                                }
                                                            ]
                                                        },
                                                        {
                                                            'value': 'Carlosappliance - Process Monitor',
                                                            'key': 'endpoint',
                                                            'label': 'API Endpoints',
                                                            'required': false,
                                                            'order': 3,
                                                            'controlType': 'dynamicdropdown'
                                                        },
                                                        {
                                                            'value': 'Statistic',
                                                            'key': 'title',
                                                            'label': 'Title',
                                                            'required': false,
                                                            'order': 1,
                                                            'controlType': 'textbox'
                                                        },
                                                        {
                                                            'value': 2,
                                                            'key': 'instanceId',
                                                            'required': true,
                                                            'order': -1,
                                                            'controlType': 'hidden'
                                                        }
                                                    ]
                                                }
                                            ]
                                        }
                                    }
                                ]
                            },
                            {
                                'styleClass': 'four wide',
                                'gadgets': [
                                    {
                                        'componentType': 'StatisticGadgetComponent',
                                        'name': 'Statistic',
                                        'description': 'General statistic.',
                                        'icon': 'images/statistic.png',
                                        'instanceId': 1500251241829,
                                        'tags': [
                                            {
                                                'facet': 'Statistic',
                                                'name': 'counter'
                                            }
                                        ],
                                        'config': {
                                            'propertyPages': [
                                                {
                                                    'displayName': 'Run',
                                                    'groupId': 'run',
                                                    'position': 10,
                                                    'properties': [
                                                        {
                                                            'value': 'job',
                                                            'key': 'resource',
                                                            'label': 'Resource',
                                                            'required': false,
                                                            'order': 2,
                                                            'controlType': 'dropdown',
                                                            'options': [
                                                                {
                                                                    'key': 'vm',
                                                                    'value': 'vm'
                                                                },
                                                                {
                                                                    'key': 'database',
                                                                    'value': 'database'
                                                                },
                                                                {
                                                                    'key': 'volume',
                                                                    'value': 'volume'
                                                                },
                                                                {
                                                                    'key': 'job',
                                                                    'value': 'job'
                                                                }
                                                            ]
                                                        },
                                                        {
                                                            'value': 'Carlosappliance - Process Monitor',
                                                            'key': 'endpoint',
                                                            'label': 'API Endpoints',
                                                            'required': false,
                                                            'order': 3,
                                                            'controlType': 'dynamicdropdown'
                                                        },
                                                        {
                                                            'value': 'Statistic',
                                                            'key': 'title',
                                                            'label': 'Title',
                                                            'required': false,
                                                            'order': 1,
                                                            'controlType': 'textbox'
                                                        },
                                                        {
                                                            'value': 2,
                                                            'key': 'instanceId',
                                                            'required': true,
                                                            'order': -1,
                                                            'controlType': 'hidden'
                                                        }
                                                    ]
                                                }
                                            ]
                                        }
                                    }
                                ]
                            }
                        ]
                    },
                    {
                        'columns': [
                            {
                                'styleClass': 'eight wide',
                                'gadgets': [
                                    {
                                        'componentType': 'TrendLineGadgetComponent',
                                        'name': 'Realtime Performance',
                                        'description': 'IOPs and Network Bandwidth.',
                                        'icon': 'images/trend-line.png',
                                        'instanceId': 1500251340757,
                                        'tags': [
                                            {
                                                'facet': 'Performance',
                                                'name': 'trend'
                                            },
                                            {
                                                'facet': 'Chart',
                                                'name': 'line'
                                            }
                                        ],
                                        'config': {
                                            'propertyPages': [
                                                {
                                                    'displayName': 'Run',
                                                    'groupId': 'run',
                                                    'position': 10,
                                                    'properties': [
                                                        {
                                                            'value': 'Devappliance - ECX',
                                                            'key': 'endpoint',
                                                            'label': 'API Endpoints',
                                                            'required': false,
                                                            'order': 2,
                                                            'controlType': 'dynamicdropdown'
                                                        },
                                                        {
                                                            'value': 'Performance',
                                                            'key': 'title',
                                                            'label': 'Title',
                                                            'required': false,
                                                            'order': 1,
                                                            'controlType': 'textbox'
                                                        },
                                                        {
                                                            'value': 2,
                                                            'key': 'instanceId',
                                                            'required': false,
                                                            'order': -1,
                                                            'controlType': 'hidden'
                                                        }
                                                    ]
                                                },
                                                {
                                                    'displayName': 'Chart',
                                                    'groupId': 'chart',
                                                    'position': 11,
                                                    'properties': [
                                                        {
                                                            'value': true,
                                                            'key': 'chart_properties',
                                                            'label': 'Show chart details',
                                                            'required': false,
                                                            'order': 3,
                                                            'controlType': 'checkbox'
                                                        }
                                                    ]
                                                }
                                            ]
                                        }
                                    },
                                    {
                                        'componentType': 'ServiceListGadgetComponent',
                                        'name': 'Service Status',
                                        'description': 'Monitors Service Status',
                                        'icon': 'images/service.png',
                                        'instanceId': 1500343727872,
                                        'tags': [
                                            {
                                                'facet': 'List',
                                                'name': 'health'
                                            }
                                        ],
                                        'config': {
                                            'propertyPages': [
                                                {
                                                    'displayName': 'Run',
                                                    'groupId': 'run',
                                                    'position': 10,
                                                    'properties': [
                                                        {
                                                            'value': 'Devappliance - Process Monitor',
                                                            'key': 'endpoint',
                                                            'label': 'API Endpoints',
                                                            'required': false,
                                                            'order': 2,
                                                            'controlType': 'dynamicdropdown'
                                                        },
                                                        {
                                                            'value': 'Service Status',
                                                            'key': 'title',
                                                            'label': 'Title',
                                                            'required': false,
                                                            'order': 1,
                                                            'controlType': 'textbox'
                                                        },
                                                        {
                                                            'value': 999,
                                                            'key': 'instanceId',
                                                            'required': false,
                                                            'order': -1,
                                                            'controlType': 'hidden'
                                                        }
                                                    ]
                                                },
                                                {
                                                    'displayName': 'List',
                                                    'groupId': 'chart',
                                                    'position': 11,
                                                    'properties': [
                                                        {
                                                            'value': true,
                                                            'key': 'chart_properties',
                                                            'label': 'Show chart details',
                                                            'required': false,
                                                            'order': 3,
                                                            'controlType': 'checkbox'
                                                        }
                                                    ]
                                                }
                                            ]
                                        }
                                    }
                                ]
                            },
                            {
                                'styleClass': 'four wide',
                                'gadgets': [
                                    {
                                        'componentType': 'DiskGadgetComponent',
                                        'name': 'Disk Utilization',
                                        'description': 'Disk consumption information for ECX.',
                                        'icon': 'images/donut.png',
                                        'instanceId': 1500253741130,
                                        'tags': [
                                            {
                                                'facet': 'Chart',
                                                'name': 'pie'
                                            },
                                            {
                                                'facet': 'Performance',
                                                'name': 'storage'
                                            }
                                        ],
                                        'config': {
                                            'propertyPages': [
                                                {
                                                    'displayName': 'Run',
                                                    'groupId': 'run',
                                                    'position': 10,
                                                    'properties': [
                                                        {
                                                            'value': 'Carlosappliance - Process Monitor',
                                                            'key': 'endpoint',
                                                            'label': 'API Endpoints',
                                                            'required': false,
                                                            'order': 4,
                                                            'controlType': 'dynamicdropdown'
                                                        },
                                                        {
                                                            'value': 'Disk Consumption',
                                                            'key': 'title',
                                                            'label': 'Title',
                                                            'required': false,
                                                            'order': 1,
                                                            'controlType': 'textbox'
                                                        },
                                                        {
                                                            'value': 50,
                                                            'key': 'threshold',
                                                            'label': 'Consumed Threshold',
                                                            'required': false,
                                                            'order': 3,
                                                            'controlType': 'number'
                                                        },
                                                        {
                                                            'value': 'g3g3egegre',
                                                            'key': 'diskid',
                                                            'label': 'Disk Id',
                                                            'required': false,
                                                            'order': 2,
                                                            'controlType': 'dropdown',
                                                            'options': [
                                                                {
                                                                    'key': '2344112gdfgdfg',
                                                                    'value': '2344112gdfgdfg'
                                                                },
                                                                {
                                                                    'key': 'g3g3egegre',
                                                                    'value': 'g3g3egegre'
                                                                },
                                                                {
                                                                    'key': '8435f34',
                                                                    'value': '8435f34'
                                                                }
                                                            ]
                                                        },
                                                        {
                                                            'value': 2,
                                                            'key': 'instanceId',
                                                            'required': false,
                                                            'order': -1,
                                                            'controlType': 'hidden'
                                                        }
                                                    ]
                                                },
                                                {
                                                    'displayName': 'Thresholds',
                                                    'groupId': 'threshold',
                                                    'position': 10,
                                                    'properties': [
                                                        {
                                                            'value': 30,
                                                            'key': 'frequency',
                                                            'label': 'Frequency',
                                                            'required': false,
                                                            'order': 3,
                                                            'controlType': 'number'
                                                        },
                                                        {
                                                            'value': 30,
                                                            'key': 'retentionH',
                                                            'label': 'Retention High',
                                                            'required': false,
                                                            'order': 4,
                                                            'controlType': 'number'
                                                        },
                                                        {
                                                            'value': 180,
                                                            'key': 'retentionL',
                                                            'label': 'Retention Low',
                                                            'required': false,
                                                            'order': 4,
                                                            'controlType': 'number'
                                                        }
                                                    ]
                                                },
                                                {
                                                    'displayName': 'Chart',
                                                    'groupId': 'chart',
                                                    'position': 11,
                                                    'properties': [
                                                        {
                                                            'value': true,
                                                            'key': 'chart_properties',
                                                            'label': 'Show chart details',
                                                            'required': false,
                                                            'order': 3,
                                                            'controlType': 'checkbox'
                                                        }
                                                    ]
                                                }
                                            ]
                                        }
                                    }
                                ]
                            },
                            {
                                'styleClass': 'four wide',
                                'gadgets': [
                                    {
                                        'componentType': 'DiskGadgetComponent',
                                        'name': 'Disk Utilization',
                                        'description': 'Disk consumption information for ECX.',
                                        'icon': 'images/donut.png',
                                        'instanceId': 1500252483066,
                                        'tags': [
                                            {
                                                'facet': 'Chart',
                                                'name': 'pie'
                                            },
                                            {
                                                'facet': 'Performance',
                                                'name': 'storage'
                                            }
                                        ],
                                        'config': {
                                            'propertyPages': [
                                                {
                                                    'displayName': 'Run',
                                                    'groupId': 'run',
                                                    'position': 10,
                                                    'properties': [
                                                        {
                                                            'value': 'Carlosappliance - Process Monitor',
                                                            'key': 'endpoint',
                                                            'label': 'API Endpoints',
                                                            'required': false,
                                                            'order': 4,
                                                            'controlType': 'dynamicdropdown'
                                                        },
                                                        {
                                                            'value': 'Disk Consumption',
                                                            'key': 'title',
                                                            'label': 'Title',
                                                            'required': false,
                                                            'order': 1,
                                                            'controlType': 'textbox'
                                                        },
                                                        {
                                                            'value': 50,
                                                            'key': 'threshold',
                                                            'label': 'Consumed Threshold',
                                                            'required': false,
                                                            'order': 3,
                                                            'controlType': 'number'
                                                        },
                                                        {
                                                            'value': 'g3g3egegre',
                                                            'key': 'diskid',
                                                            'label': 'Disk Id',
                                                            'required': false,
                                                            'order': 2,
                                                            'controlType': 'dropdown',
                                                            'options': [
                                                                {
                                                                    'key': '2344112gdfgdfg',
                                                                    'value': '2344112gdfgdfg'
                                                                },
                                                                {
                                                                    'key': 'g3g3egegre',
                                                                    'value': 'g3g3egegre'
                                                                },
                                                                {
                                                                    'key': '8435f34',
                                                                    'value': '8435f34'
                                                                }
                                                            ]
                                                        },
                                                        {
                                                            'value': 2,
                                                            'key': 'instanceId',
                                                            'required': false,
                                                            'order': -1,
                                                            'controlType': 'hidden'
                                                        }
                                                    ]
                                                },
                                                {
                                                    'displayName': 'Thresholds',
                                                    'groupId': 'threshold',
                                                    'position': 10,
                                                    'properties': [
                                                        {
                                                            'value': 30,
                                                            'key': 'frequency',
                                                            'label': 'Frequency',
                                                            'required': false,
                                                            'order': 3,
                                                            'controlType': 'number'
                                                        },
                                                        {
                                                            'value': 30,
                                                            'key': 'retentionH',
                                                            'label': 'Retention High',
                                                            'required': false,
                                                            'order': 4,
                                                            'controlType': 'number'
                                                        },
                                                        {
                                                            'value': 180,
                                                            'key': 'retentionL',
                                                            'label': 'Retention Low',
                                                            'required': false,
                                                            'order': 4,
                                                            'controlType': 'number'
                                                        }
                                                    ]
                                                },
                                                {
                                                    'displayName': 'Chart',
                                                    'groupId': 'chart',
                                                    'position': 11,
                                                    'properties': [
                                                        {
                                                            'value': true,
                                                            'key': 'chart_properties',
                                                            'label': 'Show chart details',
                                                            'required': false,
                                                            'order': 3,
                                                            'controlType': 'checkbox'
                                                        }
                                                    ]
                                                }
                                            ]
                                        }
                                    },
                                    {
                                        'componentType': 'TrendGadgetComponent',
                                        'name': 'Trend',
                                        'description': 'General trends.',
                                        'icon': 'images/trend.png',
                                        'instanceId': 1500252536414,
                                        'tags': [
                                            {
                                                'facet': 'Performance',
                                                'name': 'trend'
                                            },
                                            {
                                                'facet': 'Chart',
                                                'name': 'area'
                                            }
                                        ],
                                        'config': {
                                            'propertyPages': [
                                                {
                                                    'displayName': 'Run',
                                                    'groupId': 'run',
                                                    'position': 10,
                                                    'properties': [
                                                        {
                                                            'value': 'Carlosappliance - Process Monitor',
                                                            'key': 'endpoint',
                                                            'label': 'API Endpoints',
                                                            'required': false,
                                                            'order': 2,
                                                            'controlType': 'dynamicdropdown'
                                                        },
                                                        {
                                                            'value': 'Trend',
                                                            'key': 'title',
                                                            'label': 'Title',
                                                            'required': false,
                                                            'order': 1,
                                                            'controlType': 'textbox'
                                                        },
                                                        {
                                                            'value': 2,
                                                            'key': 'instanceId',
                                                            'required': false,
                                                            'order': -1,
                                                            'controlType': 'hidden'
                                                        }
                                                    ]
                                                },
                                                {
                                                    'displayName': 'Chart',
                                                    'groupId': 'chart',
                                                    'position': 11,
                                                    'properties': [
                                                        {
                                                            'value': true,
                                                            'key': 'chart_properties',
                                                            'label': 'Show chart details',
                                                            'required': false,
                                                            'order': 3,
                                                            'controlType': 'checkbox'
                                                        }
                                                    ]
                                                }
                                            ]
                                        }
                                    }
                                ]
                            }
                        ]
                    }
                ]
            }
        ]
    };

    // The file contents for the current environment will overwrite these during build.
    // The build system defaults to the dev environment which uses `environment.ts`, but if you do
    // `ng build --env=prod` then `environment.prod.ts` will be used instead.
    // The list of which env maps to which file can be found in `.angular-cli.json`.
    var environment = {
        production: false,
        productName: 'Dynamic Dashboard Framework',
        productVersion: '0.0.5',
        menu: {
            documentation: true,
            aiSearch: false,
            notification: true
        },
        boardConfiguration: {
            board: true,
            ai: false,
            endpoint: true
        }
    };

    /**
     * Created by jayhamilton on 2/7/17.
     */
    var ConfigurationService = /** @class */ (function () {
        function ConfigurationService(_http) {
            this._http = _http;
            this.demo = true;
            /**
             * todo - fix this hard coded store
             * @type {string}
             */
            this.remoteConfigurationRepository = '';
            this.defaultBoard = Object.assign({}, defaultBoard);
            this.sampleBoardCollection = Object.assign({}, sampleBoardCollection);
            this.env = environment;
            this.seedLocalStorageWithSampleBoardCollection();
        }
        ConfigurationService.prototype.getBoards = function () {
            if (this.demo) {
                return new rxjs.Observable(function (observer) {
                    var data = JSON.parse(localStorage.getItem('board'));
                    if (!data) {
                        data = { board: [] };
                    }
                    observer.next(data.board);
                    return function () {
                    };
                });
            }
            else {
                /**
                 * todo - this call is based on an internal representation (admin console) of something called a store.
                 * That concept requires refactoring.
                 */
                return this._http.get(this.remoteConfigurationRepository);
            }
        };
        ConfigurationService.prototype.saveBoard = function (board) {
            var _this = this;
            this.model = board;
            if (Object.keys(board).length === 0 && board.constructor === Object) {
                return rxjs.EMPTY;
            }
            if (this.demo) {
                return new rxjs.Observable(function (observer) {
                    var board_collection;
                    // find and remove board from storage
                    _this.deleteBoardFromLocalStore(board.title);
                    // get a collection object and add board to it
                    if ((board_collection = JSON.parse(localStorage.getItem('board'))) == null) {
                        board_collection = {
                            board: []
                        };
                    }
                    board_collection['board'].push(board);
                    // save
                    localStorage.setItem('board', JSON.stringify(board_collection));
                    observer.next({});
                    return function () {
                    };
                });
            }
            else {
                /**
                 * todo - a delete must happen here
                 *
                 */
                var httpOptions = {
                    headers: new i1.HttpHeaders({
                        'Content-Type': 'application/json'
                    })
                };
                return this._http.post(this.remoteConfigurationRepository + '?id=' + board.title, JSON.stringify(board), httpOptions);
            }
        };
        ConfigurationService.prototype.deleteBoard = function (boardTitle) {
            var _this = this;
            if (this.demo) {
                return new rxjs.Observable(function (observer) {
                    _this.deleteBoardFromLocalStore(boardTitle);
                    observer.next({});
                    return function () {
                    };
                });
            }
            else {
                return this._http.delete(this.remoteConfigurationRepository + '/' + boardTitle);
            }
        };
        ConfigurationService.prototype.getDefaultBoard = function () {
            var _this = this;
            return new rxjs.Observable(function (observer) {
                observer.next(_this.defaultBoard);
                return function () {
                };
            });
        };
        /*
         when a gadget instance's property page is updated and saved, the change gets communicated to all
         gadgets. The gadget instance id that caused the change will update their current instance. todo - this might be able to be
         improved. For now the utility of this approach allows the configuration service to capture the property page change in a way
         that allows us to update the persisted board model.
         */
        ConfigurationService.prototype.notifyGadgetOnPropertyChange = function (gadgetConfig, instanceId) {
            this.savePropertyPageConfigurationToStore(gadgetConfig, instanceId);
        };
        ConfigurationService.prototype.setCurrentModel = function (_currentModel) {
            this.currentModel = _currentModel;
        };
        ConfigurationService.prototype.savePropertyPageConfigurationToStore = function (gadgetConfig, instanceId) {
            var _this = this;
            this.currentModel.rows.forEach(function (row) {
                row.columns.forEach(function (column) {
                    if (column.gadgets) {
                        column.gadgets.forEach(function (gadget) {
                            _this.updateProperties(gadgetConfig, gadget, instanceId);
                        });
                    }
                });
            });
            this.saveBoard(this.currentModel).subscribe(function (result) {
                /**
                 * todo - create popup/toast to show configuration saved message
                 */
                console.debug('The following configuration model was saved!');
            }, function (error) { return console.error('Error' + error); }, function () { return console.debug('Saving configuration to store!'); });
        };
        ConfigurationService.prototype.updateProperties = function (updatedProperties, gadget, instanceId) {
            var updatedPropsObject = JSON.parse(updatedProperties);
            if (gadget.instanceId === instanceId) {
                gadget.config.propertyPages.forEach(function (propertyPage) {
                    for (var x = 0; x < propertyPage.properties.length; x++) {
                        for (var prop in updatedPropsObject) {
                            if (updatedPropsObject.hasOwnProperty(prop)) {
                                if (prop === propertyPage.properties[x].key) {
                                    propertyPage.properties[x].value = updatedPropsObject[prop];
                                }
                            }
                        }
                    }
                });
            }
        };
        ConfigurationService.prototype.seedLocalStorageWithSampleBoardCollection = function () {
            if (localStorage.getItem('board') === null) {
                if (!this.env.production) {
                    localStorage.setItem('board', JSON.stringify(this.sampleBoardCollection));
                }
            }
        };
        ConfigurationService.prototype.delete = function (board_collection) {
            localStorage.removeItem('board');
            localStorage.setItem('board', JSON.stringify(board_collection));
        };
        ConfigurationService.prototype.deleteBoardFromLocalStore = function (boardTitle) {
            var board_collection = JSON.parse(localStorage.getItem('board'));
            var index;
            if (board_collection && (index = board_collection['board'].findIndex(function (item) {
                return item.title === boardTitle;
            })) >= 0) {
                board_collection['board'].splice(index, 1);
                this.delete(board_collection);
            }
        };
        return ConfigurationService;
    }());
    ConfigurationService.decorators = [
        { type: i0.Injectable }
    ];
    ConfigurationService.ctorParameters = function () { return [
        { type: i1.HttpClient }
    ]; };

    /**
     * Created by jayhamilton on 1/24/17.
     */
    /**
     * Message Modal - clasable modal with message
     *
     * Selector message-modal
     *
     * Methods
     *      popMessageModal - display a message modal for a sepcified duration
     *      showMessageModal - show the message modal
     *      hideMessageModal - hide the message modal
     */
    var BoardsConfigurationTabComponent = /** @class */ (function () {
        function BoardsConfigurationTabComponent(_configurationService) {
            this._configurationService = _configurationService;
            this.dashboardCreateEvent = new i0.EventEmitter();
            this.dashboardEditEvent = new i0.EventEmitter();
            this.dashboardDeleteEvent = new i0.EventEmitter();
            this.newDashboardItem = '';
            this.env = environment;
        }
        BoardsConfigurationTabComponent.prototype.createBoard = function (name) {
            if (name !== '') {
                this.dashboardCreateEvent.emit(name);
                this.newDashboardItem = '';
            }
        };
        BoardsConfigurationTabComponent.prototype.editBoard = function (name) {
            this.dashboardEditEvent.emit(name);
        };
        BoardsConfigurationTabComponent.prototype.deleteBoard = function (name) {
            this.dashboardDeleteEvent.emit(name);
        };
        return BoardsConfigurationTabComponent;
    }());
    BoardsConfigurationTabComponent.decorators = [
        { type: i0.Component, args: [{
                    selector: 'dashboard-config-tab',
                    moduleId: module.id,
                    template: "<table class=\"ui padded table\">\n    <thead>\n    <th class=\"index-column\">#</th>\n    <th>name</th>\n    <th></th>\n    </thead>\n    <tr *ngFor=\"let board of dashboardList; let i = index\" class=\"ui basic segment\">\n        <td class=\"index-column\">{{i + 1}}</td>\n        <td>{{board}}</td>\n        <td style=\"text-align:right\">\n            <a (click)=\"deleteBoard(board)\" mat-mini-fab routerLink=\".\">\n                <mat-icon>clear</mat-icon>\n            </a>\n        </td>\n    </tr>\n</table>\n\n<div class=\"ui basic segment\">\n\n    <div class=\"grid-row\">\n        <mat-form-field class=\"example-full-width\">\n            <input [(ngModel)]=\"newDashboardItem\" matInput placeholder=\"Enter Dashboard Name\">\n        </mat-form-field>\n\n        <a (click)=\"createBoard(newDashboardItem)\" color='primary' mat-mini-fab routerLink=\".\">\n            <mat-icon>add</mat-icon>\n        </a>\n    </div>\n\n</div>\n",
                    styles: ["input{font-family:Roboto,sans-serif!important;font-size:1.1em;font-weight:300!important;outline:none!important;padding:5px}label{color:#4a4d50!important;font-family:Roboto,sans-serif;font-size:1.3em!important}label.endpoint{font-size:1em!important;line-height:1.5em!important}td.detail{background-color:#fff!important}.index-column{background-color:#ddebf5!important;border:none!important;width:5%!important}.index-column,.ui.table thead th td{color:#788ca0!important}.ui.table{color:rgba(97,97,97,.68)!important}.inLine{display:inline-block;padding-left:4px;padding-right:4px}.example-full-width{width:100%}.grid-row{align-items:center;display:flex;justify-content:space-around}"]
                },] }
    ];
    BoardsConfigurationTabComponent.ctorParameters = function () { return [
        { type: ConfigurationService }
    ]; };
    BoardsConfigurationTabComponent.propDecorators = {
        dashboardCreateEvent: [{ type: i0.Output }],
        dashboardEditEvent: [{ type: i0.Output }],
        dashboardDeleteEvent: [{ type: i0.Output }],
        dashboardList: [{ type: i0.Input }]
    };

    var tabsModel = [
        {
            groupId: 'boards',
            displayName: 'Boards'
        },
        {
            groupId: 'options',
            displayName: 'Options'
        }
    ];

    /**
     * Created by jayhamilton on 1/24/17.
     */
    /**
     * Message Modal - clasable modal with message
     *
     * Selector message-modal
     *
     * Methods
     *      popMessageModal - display a message modal for a sepcified duration
     *      showMessageModal - show the message modal
     *      hideMessageModal - hide the message modal
     */
    var ConfigurationComponent = /** @class */ (function () {
        function ConfigurationComponent() {
            this.dashboardCreateEvent = new i0.EventEmitter();
            this.dashboardEditEvent = new i0.EventEmitter();
            this.dashboardDeleteEvent = new i0.EventEmitter();
            this.newDashboardItem = '';
            Object.assign(this, { tabsModel: tabsModel });
            this.setCurrentTab(0);
            this.env = environment;
        }
        ConfigurationComponent.prototype.showConfigurationModal = function (header) {
            this.modalheader = header;
            this.configModal.modal('show');
        };
        ConfigurationComponent.prototype.hideMessageModal = function () {
            this.modalicon = '';
            this.modalheader = '';
            this.modalconfig = '';
            this.configModal.modal('hide');
        };
        ConfigurationComponent.prototype.createBoard = function (name) {
            if (name !== '') {
                this.dashboardCreateEvent.emit(name);
                this.newDashboardItem = '';
            }
            console.log('Creating new board event from configuration component: ' + name);
        };
        ConfigurationComponent.prototype.editBoard = function (name) {
            this.dashboardEditEvent.emit(name);
        };
        ConfigurationComponent.prototype.deleteBoard = function (name) {
            this.dashboardDeleteEvent.emit(name);
        };
        ConfigurationComponent.prototype.ngAfterViewInit = function () {
            this.configModal = jQuery(this.boardconfigmodalaRef.nativeElement);
            this.configModal.modal('hide');
        };
        ConfigurationComponent.prototype.setCurrentTab = function (tab_index) {
            this.currentTab = this.tabsModel[tab_index].displayName;
        };
        return ConfigurationComponent;
    }());
    ConfigurationComponent.decorators = [
        { type: i0.Component, args: [{
                    selector: 'dashboard-configuration-modal',
                    moduleId: module.id,
                    template: "<div #boardconfigmodal_tag class=\"ui long modal\">\n    <div class=\"header\">\n        <h2>{{modalheader}}</h2>\n    </div>\n\n\n    <div class=\"ui top attached tabular menu\">\n\n        <a (click)=\"setCurrentTab(i)\" *ngFor=\"let tab of tabsModel; let i = index\"\n           [ngClass]=\"{'active': tab.displayName == currentTab }\"\n           attr.data-tab=\"{{tab.groupId}}\" class=\"item\">{{tab.displayName}}</a>\n\n    </div>\n\n    <div *ngFor=\"let tab of tabsModel; let i = index\" [@contentSwitch]=\"tab.displayName == currentTab ? 'active':'inactive'\"\n         [ngClass]=\"{'active': tab.displayName == currentTab }\"\n         attr.data-tab=\"{{tab.groupId}}\" class=\"ui bottom attached tab segment\">\n\n\n        <div *ngIf=\"tab.groupId == 'boards'\">\n\n            <dashboard-config-tab (dashboardCreateEvent)=\"createBoard($event)\"\n                                  (dashboardDeleteEvent)=\"deleteBoard($event)\"\n                                  (dashboardEditEvent)=\"editBoard($event)\"\n                                  [dashboardList]=\"dashboardList\"></dashboard-config-tab>\n        </div>\n\n        <div *ngIf=\"tab.groupId == 'options'\">\n\n            <dashboard-options-config-tab></dashboard-options-config-tab>\n\n        </div>\n    </div>\n\n\n    <div class=\"actions\">\n        <button (click)=\"hideMessageModal()\" class=\"ui button\">Close</button>\n    </div>\n</div>\n",
                    animations: [
                        animations.trigger('contentSwitch', [
                            animations.state('inactive', animations.style({
                                opacity: 0
                            })),
                            animations.state('active', animations.style({
                                opacity: 1
                            })),
                            animations.transition('inactive => active', animations.animate('750ms ease-in')),
                            animations.transition('active => inactive', animations.animate('750ms ease-out'))
                        ])
                    ],
                    styles: [".ui.tabular.menu .active.item{border-top:3px solid #3f51b5!important}.ui.attached.segment{margin:0;width:100%}"]
                },] }
    ];
    ConfigurationComponent.ctorParameters = function () { return []; };
    ConfigurationComponent.propDecorators = {
        dashboardCreateEvent: [{ type: i0.Output }],
        dashboardEditEvent: [{ type: i0.Output }],
        dashboardDeleteEvent: [{ type: i0.Output }],
        dashboardList: [{ type: i0.Input }],
        boardconfigmodalaRef: [{ type: i0.ViewChild, args: ['boardconfigmodal_tag', { static: true },] }]
    };

    var ConfigurationModule = /** @class */ (function () {
        function ConfigurationModule() {
        }
        return ConfigurationModule;
    }());
    ConfigurationModule.decorators = [
        { type: i0.NgModule, args: [{
                    imports: [
                        common.CommonModule,
                        button.MatButtonModule,
                        icon.MatIconModule,
                        checkbox.MatCheckboxModule,
                        input.MatInputModule,
                        select.MatSelectModule,
                        core.MatOptionModule,
                        forms.FormsModule,
                        forms.ReactiveFormsModule,
                        slideToggle.MatSlideToggleModule,
                        chips.MatChipsModule
                    ],
                    declarations: [
                        BoardsConfigurationTabComponent,
                        OptionsConfigurationTabComponent,
                        ConfigurationComponent
                    ],
                    providers: [
                        OptionsService
                    ],
                    exports: [
                        BoardsConfigurationTabComponent,
                        OptionsConfigurationTabComponent,
                        ConfigurationComponent
                    ]
                },] }
    ];

    /**
     The grid is primarily controlled by MenuService Events. You can change the behavior by
     changing the MenuComponent and have it call methods within the Grid component via Output Events
     within the Menu component or by using the @ViewChild approach.

     In the current code, the MenuService is shared by the GridComponent and MenuComponent.
     The MenuComponent relays events from the various components that make up the MenuComponent.
     The GridComponent listens for those events via an Observable. The GridComponent will also
     raise/emit an event that will be picked up by the MenuComponent via the MenuService through an Observable as well.
     */
    var MenuEventService = /** @class */ (function () {
        function MenuEventService() {
            this.menuSubject = new rxjs.Subject();
            this.gridSubject = new rxjs.Subject();
            this.subscribers = [];
        }
        MenuEventService.prototype.raiseMenuEvent = function (event) {
            this.menuSubject.next(event);
        };
        MenuEventService.prototype.listenForMenuEvents = function () {
            return this.menuSubject.asObservable();
        };
        MenuEventService.prototype.raiseGridEvent = function (event) {
            this.menuSubject.next(event);
        };
        MenuEventService.prototype.listenForGridEvents = function () {
            return this.gridSubject.asObservable();
        };
        MenuEventService.prototype.addSubscriber = function (subscriber) {
            this.subscribers.push(subscriber);
        };
        MenuEventService.prototype.unSubscribeAll = function () {
            this.subscribers.forEach(function (subscription) {
                subscription.unsubscribe();
            });
            this.subscribers.length = 0;
        };
        return MenuEventService;
    }());
    MenuEventService.decorators = [
        { type: i0.Injectable }
    ];
    MenuEventService.ctorParameters = function () { return []; };

    /**a
     * Menu component
     *
     */
    var MenuComponent = /** @class */ (function () {
        function MenuComponent(_configurationService, _menuEventService) {
            this._configurationService = _configurationService;
            this._menuEventService = _menuEventService;
            this.host = window.location.host;
            this.dashboardList = [];
            this.selectedBoard = '';
            this.placeHolderText = 'Ask the board to do something!';
            this.searchList = [];
            this.show = false;
            this.typeAheadIsInMenu = true;
            this.layoutId = 0;
            this._menuEventService.unSubscribeAll();
            this.setupEventListeners();
            this.env = environment;
        }
        MenuComponent.prototype.setupEventListeners = function () {
            var _this = this;
            var gridEventSubscription = this._menuEventService.listenForGridEvents().subscribe(function (event) {
                var edata = event['data'];
                switch (event['name']) {
                    case 'boardUpdateEvent':
                        _this.updateDashboardMenu(edata);
                        break;
                }
            });
            this._menuEventService.addSubscriber(gridEventSubscription);
        };
        MenuComponent.prototype.ngOnInit = function () {
            this.updateDashboardMenu('');
        };
        MenuComponent.prototype.ngAfterViewInit = function () {
            this.stickyMenu = jQuery(this.stickyMenuRef.nativeElement);
            this.stickyMenu.sticky();
        };
        MenuComponent.prototype.emitBoardChangeLayoutEvent = function (event) {
            this._menuEventService.raiseMenuEvent({ name: 'boardChangeLayoutEvent', data: event });
        };
        MenuComponent.prototype.emitBoardSelectEvent = function (event) {
            this.boardSelect(event);
            this._menuEventService.raiseMenuEvent({ name: 'boardSelectEvent', data: event });
        };
        MenuComponent.prototype.emitBoardCreateEvent = function (event) {
            this._menuEventService.raiseMenuEvent({ name: 'boardCreateEvent', data: event });
            this.updateDashboardMenu(event);
        };
        MenuComponent.prototype.emitBoardEditEvent = function (event) {
            this._menuEventService.raiseMenuEvent({ name: 'boardEditEvent', data: event });
        };
        MenuComponent.prototype.emitBoardDeleteEvent = function (event) {
            this._menuEventService.raiseMenuEvent({ name: 'boardDeleteEvent', data: event });
            this.updateDashboardMenu('');
        };
        MenuComponent.prototype.emitBoardAddGadgetEvent = function (event) {
            this._menuEventService.raiseMenuEvent({ name: 'boardAddGadgetEvent', data: event });
        };
        MenuComponent.prototype.emitBoardAIAddGadgetEvent = function (event) {
            this._menuEventService.raiseMenuEvent({ name: 'boardAIAddGadgetEvent', data: event });
        };
        MenuComponent.prototype.updateDashboardMenu = function (selectedBoard) {
            var _this = this;
            this._configurationService.getBoards().subscribe(function (data) {
                var me = _this;
                if (data && data instanceof Array && data.length) {
                    _this.dashboardList.length = 0;
                    // sort boards
                    data.sort(function (a, b) { return a.boardInstanceId - b.boardInstanceId; });
                    data.forEach(function (board) {
                        me.dashboardList.push(board.title);
                    });
                    if (selectedBoard === '') {
                        _this.boardSelect(_this.dashboardList[0]);
                    }
                    else {
                        _this.boardSelect(selectedBoard);
                    }
                }
            });
        };
        MenuComponent.prototype.boardSelect = function (selectedBoard) {
            this.selectedBoard = selectedBoard;
        };
        MenuComponent.prototype.toggleLayoutSideBar = function () {
            this.layoutSideBar = jQuery(this.layoutSideBarRef.nativeElement);
            this.layoutSideBar.sidebar('setting', 'transition', 'overlay');
            this.layoutSideBar.sidebar('toggle');
            this.layoutId = this._configurationService.currentModel.id;
        };
        MenuComponent.prototype.toggleNotificationSideBar = function () {
            this.notificationSideBar = jQuery(this.notificationSideBarRef.nativeElement);
            this.notificationSideBar.sidebar('setting', 'transition', 'overlay');
            this.notificationSideBar.sidebar('toggle');
        };
        MenuComponent.prototype.toggleAboutSideBar = function () {
            this.aboutSideBar = jQuery(this.aboutSideBarRef.nativeElement);
            this.aboutSideBar.sidebar('setting', 'transition', 'overlay');
            this.aboutSideBar.sidebar('toggle');
        };
        return MenuComponent;
    }());
    MenuComponent.decorators = [
        { type: i0.Component, args: [{
                    moduleId: module.id,
                    selector: 'dashboard-menu',
                    template: "<div #stickymenu_tag class=\"ui inverted menu sticky\" ng-show=\"show\" style=\"height: 66px !important\">\n\n\n    <dashboard-typeahead-input (ArtificialIntelligenceEventEmitter)=\"emitBoardAIAddGadgetEvent($event)\" *ngIf=\"env.menu.aiSearch\" [placeHolderText]=\"placeHolderText\"\n                               [searchList]=\"searchList\"\n                               [typeAheadIsInMenu]=\"typeAheadIsInMenu\"\n                               class=\"item\"\n                               style=\"width:25%; padding-bottom:0\">\n    </dashboard-typeahead-input>\n\n\n    <div class=\"ui secondary inverted pointing menu board\">\n        <a (click)=\"emitBoardSelectEvent(board)\" *ngFor=\"let board of dashboardList; let i = index\"\n           [ngClass]=\"{'active':board == selectedBoard}\"\n           class=\"item\">\n            {{board}}\n        </a>\n    </div>\n\n\n    <div class=\"right inverted menu\">\n        <a (click)=\"toggleNotificationSideBar()\" *ngIf=\"env.menu.notification\"\n           class=\"item\">\n            <i class=\"large inverted alarm icon\"></i>\n            <div class=\"floating ui red label\">\n                <span style=\"font-size: 1.2em; font-weight: normal\">4</span>\n            </div>\n        </a>\n\n\n        <a (click)=\"addGadgetModal.showComponentLibraryModal('Add Gadget')\"\n           class=\"item\">\n            <i class=\"large inverted plus icon\"></i>\n        </a>\n\n\n        <a (click)=\"toggleLayoutSideBar()\"\n           class=\"item\">\n            <i class=\"large inverted grid layout icon\"></i>\n        </a>\n\n\n        <a (click)=\"boardConfigModal.showConfigurationModal('Configuration')\"\n           class=\"item\">\n            <i class=\"large inverted setting icon\"></i>\n        </a>\n\n\n        <a (click)=\"toggleAboutSideBar()\"\n           class=\"item\">\n            <i class=\"large inverted info icon\"></i>\n        </a>\n\n    </div>\n\n</div>\n<br>\n\n<!-- the modal will emit an event and call the grid component to add the requested cell to the board -->\n<dashboard-add-gadget-modal #addGadgetModal\n                            (addGadgetEvent)=\"emitBoardAddGadgetEvent($event)\">\n</dashboard-add-gadget-modal>\n\n<dashboard-configuration-modal #boardConfigModal\n                               (dashboardCreateEvent)=\"emitBoardCreateEvent($event)\"\n                               (dashboardDeleteEvent)=\"emitBoardDeleteEvent($event)\"\n                               (dashboardEditEvent)=\"emitBoardEditEvent($event)\"\n                               [dashboardList]=\"dashboardList\">\n\n\n</dashboard-configuration-modal>\n\n\n<!-- side bar content -->\n<div #notificationSideBar_tag class=\"ui sidebar inverted right vertical menu\">\n    <dashboard-notification-modal></dashboard-notification-modal>\n</div>\n\n<!-- side bar content -->\n<div #aboutSideBar_tag class=\"ui sidebar inverted right vertical menu\">\n    <dashboard-about-modal></dashboard-about-modal>\n</div>\n\n\n<div #layoutSideBar_tag class=\"ui sidebar inverted right vertical menu\">\n    <dashboard-layout-manager-modal (boardLayoutChangeEvent)=\"emitBoardChangeLayoutEvent($event)\"\n                                    [layoutId]=\"layoutId\">\n    </dashboard-layout-manager-modal>\n</div>\n\n",
                    styles: [".ui.secondary.pointing.menu .active.item{border-color:#0f0!important;border-width:medium!important;color:#f9f9f9!important}.ui.secondary.pointing.menu .active.item,.ui.secondary.pointing.menu .item{font-family:Roboto,sans-serif!important;font-size:1.2em;font-weight:500!important}.ui.secondary.pointing.menu .item{color:hsla(0,0%,50.2%,.7)!important}.ui.secondary.pointing.menu{border-bottom:none!important}input{font-family:Roboto,sans-serif!important;font-size:1.1em;font-weight:300!important;outline:none!important;padding:5px}.ui.floating.label{margin:0;top:.5em}.ui.menu .item>.floating.label{padding:.3em .6em}.ui.menu{border-radius:0}.ui.inverted.menu{background-color:#1c2b36}.ui.menu .item>i.icon{margin:0}"]
                },] }
    ];
    MenuComponent.ctorParameters = function () { return [
        { type: ConfigurationService },
        { type: MenuEventService }
    ]; };
    MenuComponent.propDecorators = {
        show: [{ type: i0.Input }],
        notificationSideBarRef: [{ type: i0.ViewChild, args: ['notificationSideBar_tag', { static: false },] }],
        layoutSideBarRef: [{ type: i0.ViewChild, args: ['layoutSideBar_tag', { static: false },] }],
        aboutSideBarRef: [{ type: i0.ViewChild, args: ['aboutSideBar_tag', { static: false },] }],
        stickyMenuRef: [{ type: i0.ViewChild, args: ['stickymenu_tag', { static: false },] }]
    };

    /**
     * Created by jayhamilton on 1/31/17.
     */
    var boardLayouts = [
        {
            id: 0,
            boardInstanceId: 0,
            title: 'one-narrow',
            checked: false,
            structure: '8',
            rows: [{
                    columns: [
                        {
                            styleClass: 'eight wide',
                        }
                    ]
                }]
        },
        {
            id: 1,
            boardInstanceId: 1,
            title: 'single',
            checked: false,
            structure: '14',
            rows: [{
                    columns: [
                        {
                            styleClass: 'fourteen wide',
                        }
                    ]
                }]
        },
        {
            id: 2,
            boardId: 2,
            title: 'narrow-right',
            checked: false,
            structure: '10-6',
            rows: [{
                    columns: [
                        {
                            styleClass: 'ten wide',
                        },
                        {
                            styleClass: 'six wide',
                        }
                    ]
                }]
        },
        {
            id: 3,
            boardInstanceId: 3,
            title: 'wide-center',
            checked: false,
            structure: '4-8-4',
            rows: [{
                    columns: [
                        {
                            styleClass: 'four wide',
                        },
                        {
                            styleClass: 'eight wide',
                        },
                        {
                            styleClass: 'four wide',
                        }
                    ]
                }]
        },
        {
            id: 4,
            boardInstanceId: 4,
            title: 'narrow-left',
            checked: false,
            structure: '4-12',
            rows: [{
                    columns: [
                        {
                            styleClass: 'four wide',
                        },
                        {
                            styleClass: 'twelve wide',
                        }
                    ]
                }]
        },
        {
            id: 5,
            boardInstanceId: 5,
            title: 'two-even',
            checked: true,
            structure: '6-6',
            rows: [{
                    columns: [
                        {
                            styleClass: 'six wide',
                        },
                        {
                            styleClass: 'six wide',
                        }
                    ]
                }]
        },
        {
            id: 6,
            boardInstanceId: 6,
            title: 'three-even',
            checked: false,
            structure: '5-5-5',
            rows: [{
                    columns: [
                        {
                            styleClass: 'four wide',
                        },
                        {
                            styleClass: 'four wide',
                        },
                        {
                            styleClass: 'four wide',
                        }
                    ]
                }]
        },
        {
            id: 7,
            boardInstanceId: 7,
            title: 'wide-top',
            checked: false,
            structure: '16/8-8',
            rows: [
                {
                    columns: [
                        {
                            styleClass: 'twelve wide'
                        }
                    ]
                },
                {
                    columns: [
                        {
                            styleClass: 'six wide'
                        },
                        {
                            styleClass: 'six wide'
                        }
                    ]
                }
            ]
        },
        {
            id: 8,
            boardInstanceId: 8,
            title: 'ngadmin',
            checked: false,
            structure: '4-4-4-4/8-4-4',
            rows: [
                {
                    columns: [
                        {
                            styleClass: 'four wide'
                        },
                        {
                            styleClass: 'four wide'
                        },
                        {
                            styleClass: 'four wide'
                        },
                        {
                            styleClass: 'four wide'
                        }
                    ]
                },
                {
                    columns: [
                        {
                            styleClass: 'eight wide'
                        },
                        {
                            styleClass: 'four wide'
                        },
                        {
                            styleClass: 'four wide'
                        }
                    ]
                }
            ]
        },
        {
            id: 9,
            boardInstanceId: 9,
            title: 'google-layout',
            checked: false,
            structure: '3-6-3',
            rows: [
                {
                    columns: [
                        {
                            styleClass: 'three wide'
                        },
                        {
                            styleClass: 'six wide'
                        },
                        {
                            styleClass: 'three wide'
                        }
                    ]
                }
            ]
        }
    ];

    /**
     * Created by jayhamilton on 1/24/17.
     */
    /**
     * Message Modal - clasable modal with message
     *
     * Selector message-modal
     *
     * Methods
     *      popMessageModal - display a message modal for a sepcified duration
     *      showMessageModal - show the message modal
     *      hideMessageModal - hide the message modal
     */
    var BoardLayoutManagerComponent = /** @class */ (function () {
        function BoardLayoutManagerComponent() {
            this.boardLayoutChangeEvent = new i0.EventEmitter();
            this.modalHeader = 'Layout';
            this.initializeLayouts();
        }
        BoardLayoutManagerComponent.prototype.selectBoardLayout = function (layoutId) {
            for (var x = 0; x < this.boardLayouts.length; x++) {
                if (this.boardLayouts[x].id === layoutId) {
                    this.boardLayoutChangeEvent.emit(this.boardLayouts[x]);
                    this.layoutId = layoutId;
                    break;
                }
            }
        };
        BoardLayoutManagerComponent.prototype.initializeLayouts = function () {
            Object.assign(this, { boardLayouts: boardLayouts });
        };
        return BoardLayoutManagerComponent;
    }());
    BoardLayoutManagerComponent.decorators = [
        { type: i0.Component, args: [{
                    selector: 'dashboard-layout-manager-modal',
                    moduleId: 'layout',
                    template: "<br>\n\n<div style=\"text-align: center\">\n    <h2>{{modalHeader}}</h2>\n</div>\n<hr>\n\n<br>\n<br>\n<ul>\n    <li *ngFor=\"let layout of boardLayouts; let i = index\"\n        style='list-style: none'>\n        <img (click)=\"selectBoardLayout(layout.id)\" [ngClass]=\"{'layout-selected': layoutId == layout.id }\"\n             src=\"/assets/images/{{layout.title}}.png\">\n\n    </li>\n</ul>\n\n\n\n",
                    styles: [".layout-selected{border-top:solid #7cfc00;padding-top:5px}ul{text-align:center}h2{color:#fff!important}dir,menu,ul{-webkit-padding-start:5px}"]
                },] }
    ];
    BoardLayoutManagerComponent.ctorParameters = function () { return []; };
    BoardLayoutManagerComponent.propDecorators = {
        layoutId: [{ type: i0.Input }],
        boardLayoutChangeEvent: [{ type: i0.Output }]
    };

    var LayoutModule = /** @class */ (function () {
        function LayoutModule() {
        }
        return LayoutModule;
    }());
    LayoutModule.decorators = [
        { type: i0.NgModule, args: [{
                    imports: [
                        common.CommonModule
                    ],
                    declarations: [BoardLayoutManagerComponent],
                    exports: [BoardLayoutManagerComponent]
                },] }
    ];

    /**
     * Created by jayhamilton on 2/7/17.
     */
    var AddGadgetService = /** @class */ (function () {
        function AddGadgetService(_http) {
            this._http = _http;
        }
        AddGadgetService.prototype.getGadgetLibrary = function () {
            // TODO: make this configurable. the application need to define this.
            var gadgetLibraryJson = 'gadget-library-model.json';
            return this._http.get('/assets/api/' + gadgetLibraryJson);
        };
        return AddGadgetService;
    }());
    AddGadgetService.ɵprov = i0.ɵɵdefineInjectable({ factory: function AddGadgetService_Factory() { return new AddGadgetService(i0.ɵɵinject(i1.HttpClient)); }, token: AddGadgetService, providedIn: "root" });
    AddGadgetService.decorators = [
        { type: i0.Injectable, args: [{ providedIn: 'root' },] }
    ];
    AddGadgetService.ctorParameters = function () { return [
        { type: i1.HttpClient }
    ]; };

    var Facet = /** @class */ (function () {
        function Facet(name, tags) {
            this.name = name;
            this.tags = tags;
        }
        return Facet;
    }());
    var Tag = /** @class */ (function () {
        function Tag(name) {
            this.name = name;
            this.count = 1;
        }
        return Tag;
    }());

    var FacetTagProcessor = /** @class */ (function () {
        function FacetTagProcessor(objectList) {
            this.facet_tags = [];
            this.objectList = objectList;
        }
        FacetTagProcessor.prototype.getFacetTags = function () {
            var me = this;
            this.objectList.forEach(function (item) {
                me.formatAndUpdateTagList(item.tags);
            });
            return this.facet_tags;
        };
        FacetTagProcessor.prototype.formatAndUpdateTagList = function (items) {
            var _this = this;
            items.forEach(function (tag) {
                // add the first facet and tag to the facet_tag array
                if (!_this.facet_tags.length) {
                    _this.createFacetAndAddItToTheFacetTagArray(tag);
                }
                else {
                    var facetExists_1 = false;
                    _this.facet_tags.forEach(function (facet) {
                        if (facet.name.toLowerCase() === tag.facet.toLowerCase()) {
                            facetExists_1 = true;
                        }
                    });
                    if (facetExists_1) {
                        _this.updateFacetWithTag(tag);
                    }
                    else {
                        _this.createFacetAndAddItToTheFacetTagArray(tag);
                    }
                }
            });
        };
        FacetTagProcessor.prototype.createFacetAndAddItToTheFacetTagArray = function (tag) {
            var _tags = [];
            var _tag = this.createTag(this.capitalize(tag.name));
            _tags.push(_tag);
            var facet = new Facet(tag.facet, _tags);
            this.facet_tags.push(facet);
        };
        FacetTagProcessor.prototype.createTag = function (tag) {
            return new Tag(tag);
        };
        FacetTagProcessor.prototype.updateFacetWithTag = function (tag) {
            var _this = this;
            // find the facet and then add the tag or update the count
            this.facet_tags.forEach(function (facet) {
                if (facet.name.toLowerCase() === tag.facet.toLowerCase()) {
                    var tagExists_1 = false;
                    facet.tags.forEach(function (_tag) {
                        if (_tag.name.toLowerCase() === tag.name.toLowerCase()) {
                            tagExists_1 = true;
                            _tag.count = _tag.count + 1;
                        }
                    });
                    if (!tagExists_1) {
                        facet.tags.push(_this.createTag(_this.capitalize(tag.name)));
                    }
                }
            });
        };
        FacetTagProcessor.prototype.capitalize = function (value) {
            return value.charAt(0).toUpperCase() + value.slice(1).toLowerCase();
        };
        return FacetTagProcessor;
    }());

    /**
     * Created by jayhamilton on 1/24/17.
     */
    /**
     * Message Modal - clasable modal with message
     *
     * Selector message-modal
     *
     * Methods
     *      popMessageModal - display a message modal for a sepcified duration
     *      showMessageModal - show the message modal
     *      hideMessageModal - hide the message modal
     */
    var AddGadgetComponent = /** @class */ (function () {
        function AddGadgetComponent(_addGadgetService) {
            this._addGadgetService = _addGadgetService;
            this.addGadgetEvent = new i0.EventEmitter();
            this.gadgetObjectList = [];
            this.gadgetObjectTitleList = [];
            this.placeHolderText = 'Begin typing gadget name';
            this.layoutColumnOneWidth = 'six';
            this.layoutColumnTwoWidth = 'ten';
            this.listHeader = 'Gadgets';
            this.color = 'white';
            this.typeAheadIsInMenu = false;
            this.getObjectList();
        }
        AddGadgetComponent.prototype.actionHandler = function (actionItem, actionName) {
            this.addGadgetEvent.emit(actionItem);
            this.hideMessageModal();
        };
        AddGadgetComponent.prototype.showMessageModal = function (icon, header, message) {
            this.modalicon = icon;
            this.modalheader = header;
            this.modalmessage = message;
            this.messageModal.modal('show');
        };
        AddGadgetComponent.prototype.showComponentLibraryModal = function (header) {
            this.modalheader = header;
            this.messageModal.modal('show');
        };
        AddGadgetComponent.prototype.hideMessageModal = function () {
            this.modalicon = '';
            this.modalheader = '';
            this.modalmessage = '';
            this.messageModal.modal('hide');
        };
        AddGadgetComponent.prototype.ngAfterViewInit = function () {
            this.messageModal = jQuery(this.messagemodalRef.nativeElement);
        };
        AddGadgetComponent.prototype.getObjectList = function () {
            var _this = this;
            this._addGadgetService.getGadgetLibrary().subscribe(function (data) {
                _this.gadgetObjectList.length = 0;
                var me = _this;
                data.library.forEach(function (item) {
                    me.gadgetObjectList.push(item);
                    me.gadgetObjectTitleList.push(item.name);
                });
                var facetTagProcess = new FacetTagProcessor(_this.gadgetObjectList);
                _this.facetTags = facetTagProcess.getFacetTags();
            });
        };
        return AddGadgetComponent;
    }());
    AddGadgetComponent.decorators = [
        { type: i0.Component, args: [{
                    selector: 'dashboard-add-gadget-modal',
                    moduleId: module.id,
                    template: "<div #messagemodal_tag class=\"ui long modal\">\n    <div class=\"header\">\n        <h2>{{modalheader}}</h2>\n    </div>\n\n    <dashboard-data-list\n\n        [facetTags]=\"facetTags\"\n        [layoutColumnOneWidth]=\"layoutColumnOneWidth\"\n        [layoutColumnTwoWidth]=\"layoutColumnTwoWidth\"\n        [listHeader]=\"listHeader\"\n        [objectList]=\"gadgetObjectList\"\n        [objectTitleList]=\"gadgetObjectTitleList\"\n        [placeHolderText]=\"placeHolderText\"\n        [typeAheadIsInMenu]=\"typeAheadIsInMenu\">\n\n        <ng-template let-item=\"$implicit\">\n\n            <div [@showHideAnimation] class=\"ui segment\" style=\"background-color:white;\">\n                <div class=\"ui large middle aligned divided list\">\n                    <div class=\"item\">\n                        <div class=\"right floated content\">\n                            <!-- todo : if there are more than one action display a button with\n                            vertical ellipse and implement onHover to show all action buttons\n                            -->\n                            <a (click)=\"actionHandler(item, item.actions[0].name)\"\n                               *ngIf=\"item.actions.length == 1\"\n                               color=\"primary\"\n                               mat-raised-button routerLink=\".\">\n                                {{item.actions[0].name}}\n                            </a>\n                        </div>\n                        <img class=\"ui image\" src=\"{{item.icon}}\">\n                        <div class=\"content\">\n                            <div class=\"header\">{{item.name}}</div>\n                            {{item.description}}\n                        </div>\n                    </div>\n                </div>\n                <div class=\"ui bottom attached\">\n                    <button *ngFor=\"let tag of item.tags\" class=\"ui basic gray compact button\">{{tag.name}}\n                    </button>\n                </div>\n            </div>\n\n        </ng-template>\n\n    </dashboard-data-list>\n\n    <div class=\"actions\">\n        <div class=\"ui approve button\">Close</div>\n    </div>\n</div>\n",
                    animations: [
                        animations.trigger('showHideAnimation', [
                            animations.transition(':enter', [
                                animations.style({ opacity: 0 }),
                                animations.animate(750, animations.style({ opacity: 1 }))
                            ]),
                            animations.transition(':leave', [
                                animations.animate(750, animations.style({ opacity: 0 }))
                            ])
                        ])
                    ],
                    styles: [".modal{background-color:#f7f7f7}"]
                },] }
    ];
    AddGadgetComponent.ctorParameters = function () { return [
        { type: AddGadgetService }
    ]; };
    AddGadgetComponent.propDecorators = {
        addGadgetEvent: [{ type: i0.Output }],
        messagemodalRef: [{ type: i0.ViewChild, args: ['messagemodal_tag', { static: true },] }]
    };

    /**
     * Created by jayhamilton on 7/11/17.
     */
    var FacetComponent = /** @class */ (function () {
        function FacetComponent() {
            this.tagSelectEvent = new i0.EventEmitter();
        }
        FacetComponent.prototype.ngOnInit = function () {
            if (this.openFacet) {
                this.facetOpen = 'in';
            }
            else {
                this.facetOpen = 'out';
            }
        };
        FacetComponent.prototype.toggleAccordion = function () {
            this.facetOpen = this.facetOpen === 'out' ? 'in' : 'out';
        };
        FacetComponent.prototype.tagSelect = function (tagName) {
            this.tagSelectEvent.emit(tagName);
        };
        return FacetComponent;
    }());
    FacetComponent.decorators = [
        { type: i0.Component, args: [{
                    moduleId: module.id,
                    selector: 'dashboard-facet',
                    template: "\n\n        <hr style='max-width: 100%; margin-left:0;'>\n        <br>\n        <div class='ui container grid'>\n            <div class='eight wide column' style=\"margin-left: 0 !important;padding-left: 0 !important;\">\n                <h4>{{facet.name}}</h4>\n            </div>\n            <div class='eight wide column'>\n                <div class='ui top right attached label' style='background-color: whitesmoke'>\n                    <i\n                        class='chevron icon' [ngClass]= \"{'up': facetOpen == 'in', 'down': facetOpen=='out'}\"\n                        (click)='toggleAccordion()'\n                        style='color:grey'>\n                    </i>\n                </div>\n            </div>\n        </div>\n        <div [@accordion]='facetOpen'>\n            <table class='ui very basic table' [@accordion2]='facetOpen'>\n                <tbody>\n                <tr *ngFor='let tag of facet.tags'>\n                    <td style=\"width: 35px !important\">\n                        <mat-checkbox (click)='tagSelect(tag.name)'></mat-checkbox>\n                        <!--<input type='checkbox' (click)='tagSelect(tag.name)'>-->\n                    </td>\n                    <td style='color:grey'>{{tag.name}} &nbsp;( {{tag.count}} )</td>\n                </tr>\n                </tbody>\n            </table>\n        </div>\n        <br>\n\n    ",
                    animations: [
                        animations.trigger('accordion', [
                            animations.state('in', animations.style({
                                height: '*'
                            })),
                            animations.state('out', animations.style({
                                opacity: '0',
                                height: '0px'
                            })),
                            animations.transition('in => out', animations.animate('700ms ease-in-out')),
                            animations.transition('out => in', animations.animate('300ms ease-in-out'))
                        ]),
                        animations.trigger('accordion2', [
                            animations.state('in', animations.style({
                                height: '*'
                            })),
                            animations.state('out', animations.style({
                                opacity: '0',
                                height: '0px'
                            })),
                            animations.transition('in => out', animations.animate('300ms ease-in-out')),
                            animations.transition('out => in', animations.animate('800ms ease-in-out'))
                        ])
                    ],
                    styles: [".ui.avatar.image{height:4em!important;width:4.4em!important}hr{background:hsla(0,0%,43.1%,.1)!important;border:0;color:hsla(0,0%,43.1%,.1)!important;font-size:0;height:2px!important}.ui.input.focus input,.ui.input input:focus{border-color:#e9e9e9!important}h3,h4{color:rgba(66,66,66,.74)!important;font-size:.85em;font-weight:400!important}.segment,.ui.basic.segment{border-radius:5px!important}.ui.table tr td{border-top:none!important}.ui.table td{font-size:.8em;padding:.3em!important}"]
                },] }
    ];
    FacetComponent.ctorParameters = function () { return []; };
    FacetComponent.propDecorators = {
        tagSelectEvent: [{ type: i0.Output }],
        facet: [{ type: i0.Input }],
        openFacet: [{ type: i0.Input }]
    };

    /**
     * Created by jayhamilton on 6/27/17.
     */
    var FilterListComponent = /** @class */ (function () {
        function FilterListComponent() {
            this.tagSelectEvent = new i0.EventEmitter();
        }
        FilterListComponent.prototype.tagSelect = function (tagName) {
            this.tagSelectEvent.emit(tagName);
        };
        return FilterListComponent;
    }());
    FilterListComponent.decorators = [
        { type: i0.Component, args: [{
                    moduleId: module.id,
                    selector: 'dashboard-filter-list',
                    template: "\n        <br>\n        <div *ngFor='let facet of facet_tags ;let i = index'>\n\n            <dashboard-facet [facet]='facet' (tagSelectEvent)='tagSelect($event)' [openFacet]='i < 2'></dashboard-facet>\n\n        </div>\n    ",
                    styles: [".ui.avatar.image{height:4em!important;width:4.4em!important}hr{background:hsla(0,0%,43.1%,.1)!important;border:0;color:hsla(0,0%,43.1%,.1)!important;font-size:0;height:2px!important}.ui.input.focus input,.ui.input input:focus{border-color:#e9e9e9!important}h3,h4{color:rgba(66,66,66,.74)!important;font-size:.85em;font-weight:400!important}.segment,.ui.basic.segment{border-radius:5px!important}.ui.table tr td{border-top:none!important}.ui.table td{font-size:.8em;padding:.3em!important}"]
                },] }
    ];
    FilterListComponent.propDecorators = {
        tagSelectEvent: [{ type: i0.Output }],
        facet_tags: [{ type: i0.Input }]
    };

    /**
     * Created by jayhamilton on 6/27/17.
     */
    var FilterTagComponent = /** @class */ (function () {
        function FilterTagComponent() {
            this.updateFilterListEvent = new i0.EventEmitter();
            this.filterList = [];
        }
        FilterTagComponent.prototype.updateFilterList = function (filter) {
            filter = filter.toLocaleLowerCase();
            var index = this.filterList.indexOf(filter);
            if (index !== -1) {
                this.filterList.splice(index, 1);
            }
            else {
                this.filterList.push(filter);
            }
            this.updateFilterListEvent.emit(this.filterList);
        };
        return FilterTagComponent;
    }());
    FilterTagComponent.decorators = [
        { type: i0.Component, args: [{
                    moduleId: module.id,
                    selector: 'dashboard-filter-tag',
                    template: "\n        <div class='ui basic segment' style='background-color:white; min-height: 4.5em; border-radius:5px'>\n            <div class='ui large circular labels'>\n               <a class='ui label' [@showHideAnimation] *ngFor='let tag of filterList'>\n                    {{tag}}\n                </a>\n            </div>\n        </div>\n    ",
                    animations: [
                        animations.trigger('showHideAnimation', [
                            animations.transition(':enter', [
                                animations.style({ opacity: 0 }),
                                animations.animate(750, animations.style({ opacity: 1 }))
                            ]),
                            animations.transition(':leave', [
                                animations.animate(750, animations.style({ opacity: 0 }))
                            ])
                        ])
                    ],
                    styles: [".gadget{color:grey;font-family:Helvetica Neue,Helvetica,Open Sans,Arial,Lucida Grande,sans-serif;margin-bottom:15px!important;min-height:inherit!important}:host /deep/ div:not(.proppages){display:block!important;margin-left:auto;margin-right:auto}.ui.form .field>label{color:grey;font-family:Helvetica Neue,Helvetica,Open Sans,Arial,Lucida Grande,sans-serif;font-size:1.3em!important;font-weight:300}#bt{text-align:center!important}input{border:none!important;font-size:1.3em!important;font-weight:300!important;outline:none!important;padding:5px;width:75%!important}.ct-title,input{font-family:Helvetica Neue,Helvetica,Open Sans,Arial,Lucida Grande,sans-serif}.ct-title{color:#585858!important;font-size:1.5em!important;font-weight:500!important;text-align:left!important}.ui.top.attached.label,.ui.top.left.floated.label{background-color:#fff}hr{border-style:solid;color:#000;font-weight:300!important;height:1px}.ui[class*=\"top attached\"].label:before{border-bottom:0 solid #d0d3d6;bottom:5px;content:\"\";height:1px;left:2%;position:absolute;width:95%}.spacer{height:2em}.ui.items>.item>.content>.description{color:#a9a9a9!important}.suggestions>li{background-color:#fff!important}td{color:grey}td,th{font-family:Helvetica Neue,Helvetica,Open Sans,Arial,Lucida Grande,sans-serif}th{color:#369}.example-full-width{width:100%}h4{font-weight:400}.ui.segment{border:none;box-shadow:none}"]
                },] }
    ];
    FilterTagComponent.ctorParameters = function () { return []; };
    FilterTagComponent.propDecorators = {
        updateFilterListEvent: [{ type: i0.Output }]
    };

    /*
     * Capitalize the first letter of the string
     * Takes a string as a value.
     * Usage:
     *  value | capitalizefirst
     * Example:
     *  // value.name = daniel
     *  {{ value.name | capitalizefirst  }}
     *  fromats to: Daniel
     */
    var CapitalizeFirstPipe = /** @class */ (function () {
        function CapitalizeFirstPipe() {
        }
        CapitalizeFirstPipe.prototype.transform = function (value, args) {
            if (value === null) {
                return 'Not assigned';
            }
            return value.charAt(0).toUpperCase() + value.slice(1);
        };
        return CapitalizeFirstPipe;
    }());
    CapitalizeFirstPipe.decorators = [
        { type: i0.Pipe, args: [{
                    name: 'capitalizeFirst'
                },] }
    ];

    var FacetModule = /** @class */ (function () {
        function FacetModule() {
        }
        return FacetModule;
    }());
    FacetModule.decorators = [
        { type: i0.NgModule, args: [{
                    imports: [
                        common.CommonModule,
                        forms.FormsModule,
                        checkbox.MatCheckboxModule
                    ],
                    declarations: [
                        FacetComponent,
                        FilterListComponent,
                        FilterTagComponent,
                        CapitalizeFirstPipe
                    ],
                    providers: [
                        AddGadgetService
                    ],
                    exports: [
                        FacetComponent,
                        FilterListComponent,
                        FilterTagComponent,
                        CapitalizeFirstPipe
                    ]
                },] }
    ];

    /**
     * Created by jayhamilton on 6/22/17.
     */
    var ErrorObject = /** @class */ (function () {
        function ErrorObject(detail, summary, solutions, resource) {
            this.detail = detail;
            this.summary = summary;
            this.solutions = solutions;
            this.resource = resource;
        }
        return ErrorObject;
    }());
    var SolutionObject = /** @class */ (function () {
        function SolutionObject(summary, articleId, link) {
            this.summary = summary;
            this.articleId = articleId;
            this.link = link;
        }
        return SolutionObject;
    }());

    /**
     * Created by jayhamilton on 7/5/17.
     */
    var ErrorHandler = /** @class */ (function () {
        function ErrorHandler() {
        }
        ErrorHandler.getErrorObject = function (errMsg) {
            return new ErrorObject(errMsg.statusText, 'Some description', ErrorHandler.getSolutionList(errMsg.status
                + ' '
                + errMsg.statusText), errMsg.resource);
        };
        /**
         * todo - fix this error handling logic. Move it to its own class.
         * @param errMsg
         * @returns {SolutionObject[]}
         */
        ErrorHandler.getSolutionList = function (errMsg) {
            console.log('ERROR CONDITION:  ' + errMsg);
            var solutionList = [];
            switch (ErrorHandler.getErrorType(errMsg.toLowerCase())) {
                case 'ERR_CERTIFICATE':
                    solutionList.push(new SolutionObject('Check to see if your browser has accepted the certificate', 0, 'http://link1'));
                    break;
                case 'ERR_CROSS_ORIGIN_RESOURCE_SHARING':
                    solutionList.push(new SolutionObject('Check to see if your browser has accepted the certificate', 0, 'http://link1'));
                    break;
                case 'ERR_CONNECTION_REFUSED':
                    solutionList.push(new SolutionObject('Check to see if the host/service you are attempting to connect to is up.', 0, 'http://link1'));
                    break;
                case 'ERR_NOT_FOUND':
                    solutionList.push(new SolutionObject('Resource not found.', 0, 'http://link1'));
                    break;
                case 'ERR_CONNECTION_TIMEOUT':
                    solutionList.push(new SolutionObject('A timeout occurred. The default timeout on a connection is 60 seconds. ' +
                        'Check the endpoint to see if you are able to access the ip and port using ping. ' +
                        'If 60 seconds is to short of a timeout go into configuration and increase it.', 0, 'http://link1'));
                    break;
                default: {
                    solutionList.push(new SolutionObject(errMsg, 0, 'http://link1'));
                }
            }
            return solutionList;
        };
        ErrorHandler.getErrorType = function (errMsg) {
            if (errMsg.indexOf('trust') > -1) {
                return 'ERR_CERTIFICATE';
            }
            if (errMsg.indexOf('cors') > -1) {
                return 'ERR_CROSS_ORIGIN_RESOURCE_SHARING';
            }
            if (errMsg.indexOf('refuse') > -1) {
                return 'ERR_CONNECTION_REFUSED';
            }
            if (errMsg.indexOf('timeout') > -1) {
                return 'ERR_CONNECTION_TIMEOUT';
            }
            if (errMsg.indexOf('404') > -1) {
                return 'ERR_NOT_FOUND';
            }
            return errMsg;
        };
        ErrorHandler.getWebSocketErrorReason = function (error) {
            var reason = 'There was probably a problem with an attempt to connect to the endpoint!';
            switch (error.code) {
                case 1000:
                    reason = 'Normal closure';
                    break;
                case 1001:
                    reason = 'An endpoint is going away';
                    break;
                case 1002:
                    reason = 'An endpoint is terminating the connection due to a protocol error.';
                    break;
                case 1003:
                    reason = 'An endpoint is terminating the connection because it has received a type of data it cannot accept';
                    break;
                case 1004:
                    reason = 'Reserved. The specific meaning might be defined in the future.';
                    break;
                case 1005:
                    reason = 'No status code was actually present';
                    break;
                case 1006:
                    reason = 'The connection was closed abnormally';
                    break;
                case 1007:
                    reason = 'The endpoint is terminating the connection because a message was received that contained inconsistent data';
                    break;
                case 1008:
                    reason = 'The endpoint is terminating the connection because it received a message that violates its policy';
                    break;
                case 1009:
                    reason = 'The endpoint is terminating the connection because a data frame was received that is too large';
                    break;
                case 1010:
                    reason = 'The client is terminating the connection because it expected the server to negotiate one or more extension, but the server didn\'t.';
                    break;
                case 1011:
                    reason = 'The server is terminating the connection because it encountered an unexpected condition that prevented it from fulfilling the request.';
                    break;
                case 1012:
                    reason = 'The server is terminating the connection because it is restarting';
                    break;
                case 1013:
                    reason = 'The server is terminating the connection due to a temporary condition';
                    break;
                case 1015:
                    reason = 'The connection was closed due to a failure to perform a TLS handshake';
                    break;
            }
            return reason;
        };
        return ErrorHandler;
    }());

    /**
     * Created by jayhamilton on 1/18/17.
     */
    var RuntimeService = /** @class */ (function () {
        function RuntimeService() {
        }
        RuntimeService.handleError = function (err) {
            var errMsg = {
                status: '-1',
                statusText: '',
                resource: ''
            };
            if (err.error instanceof Error) {
                errMsg.statusText = err.error.message;
                console.log('Client error');
            }
            else {
                errMsg.status = err.status;
                errMsg.statusText = 'A backend error occurred. In all likelihood the server/api service is not running.';
                errMsg.resource = err.url;
            }
            return rxjs.throwError(ErrorHandler.getErrorObject(errMsg));
        };
        return RuntimeService;
    }());
    RuntimeService.decorators = [
        { type: i0.Injectable }
    ];
    RuntimeService.ctorParameters = function () { return []; };

    /**
     * Created by jayhamilton on 2/26/17.
     */
    var TypeAheadInputComponent = /** @class */ (function () {
        function TypeAheadInputComponent(myElement, _runtimeService) {
            this._runtimeService = _runtimeService;
            this.selectionEvent = new i0.EventEmitter();
            this.ArtificialIntelligenceEventEmitter = new i0.EventEmitter();
            this.requestCounter = 0;
            this.maxAttempts = 5;
            this.query = '';
            this.filteredList = [];
            this.elementRef = myElement;
        }
        TypeAheadInputComponent.prototype.filter = function () {
            if (this.query !== '') {
                this.filteredList = this.searchList.filter(function (el) {
                    return el.toLowerCase().indexOf(this.query.toLowerCase()) > -1;
                }.bind(this));
            }
            else {
                this.filteredList = [];
            }
            this.selectionEvent.emit(this.query);
            this.requestCounter++;
        };
        TypeAheadInputComponent.prototype.select = function (item) {
            this.query = item;
            this.filteredList = [];
            this.selectionEvent.emit(item);
        };
        return TypeAheadInputComponent;
    }());
    TypeAheadInputComponent.decorators = [
        { type: i0.Component, args: [{
                    moduleId: module.id,
                    selector: 'dashboard-typeahead-input',
                    template: "<div [ngClass]=\"{inverted:typeAheadIsInMenu == true}\" class=\"ui fluid transparent icon input\">\n    <input (keyup)=filter() [(ngModel)]=query placeholder={{placeHolderText}} type=\"text\" width=\"100%\">\n    <i *ngIf=\"!query\" class=\"large grey search icon\"></i> <i (click)=\"query=''\" *ngIf=\"query\"\n                                                             class=\"large grey remove link icon\"></i>\n</div>\n\n<div *ngIf='filteredList.length > 0' class='suggestions'>\n\n    <ul style='list-style-type:none; background: white; border-radius: 7px;'>\n        <li *ngFor='let item of filteredList' style='padding:5px'>\n            <a (click)='select(item)'>{{item}}</a>\n        </li>\n    </ul>\n</div>\n",
                    styles: [".full-width{width:100%}/deep/ .ui.menu .item>.input input{font-size:1.1em!important}/deep/ .mat-form-field-underline{height:0!important}"]
                },] }
    ];
    TypeAheadInputComponent.ctorParameters = function () { return [
        { type: i0.ElementRef },
        { type: RuntimeService }
    ]; };
    TypeAheadInputComponent.propDecorators = {
        searchList: [{ type: i0.Input }],
        placeHolderText: [{ type: i0.Input }],
        typeAheadIsInMenu: [{ type: i0.Input }],
        selectionEvent: [{ type: i0.Output }],
        ArtificialIntelligenceEventEmitter: [{ type: i0.Output }]
    };

    var TypeAheadInputModule = /** @class */ (function () {
        function TypeAheadInputModule() {
        }
        return TypeAheadInputModule;
    }());
    TypeAheadInputModule.decorators = [
        { type: i0.NgModule, args: [{
                    imports: [
                        common.CommonModule,
                        forms.FormsModule
                    ],
                    declarations: [
                        TypeAheadInputComponent
                    ],
                    providers: [],
                    exports: [
                        TypeAheadInputComponent
                    ]
                },] }
    ];

    var DataListComponent = /** @class */ (function () {
        function DataListComponent() {
            this.color = 'white';
            this.objectListCopy = [];
        }
        DataListComponent.prototype.filterListByTags = function (filterList) {
            this.copyObjectList();
            this.objectList = this.objectListCopy.filter(function (object) {
                var tagFound = false;
                if (!filterList.length) {
                    return true;
                }
                else {
                    object.tags.forEach(function (tag) {
                        filterList.forEach(function (filter) {
                            if (tag.name.toLocaleLowerCase() === filter.toLocaleLowerCase()) {
                                tagFound = true;
                            }
                        });
                    });
                    return tagFound;
                }
            });
        };
        DataListComponent.prototype.filterListBySearchString = function (searchString) {
            this.copyObjectList();
            this.objectList = this.objectListCopy.filter(function (object) {
                if (searchString.localeCompare('') === 0) {
                    return true;
                }
                else {
                    if (object.name.toLowerCase().indexOf(searchString.toLowerCase()) !== -1) {
                        return true;
                    }
                }
            });
        };
        /**
         * todo - find a better way to manage the list that is displayed and filtered.
         */
        DataListComponent.prototype.copyObjectList = function () {
            var _this = this;
            if (this.objectListCopy.length === 0) {
                this.objectList.forEach(function (item) {
                    _this.objectListCopy.push(item);
                });
            }
        };
        return DataListComponent;
    }());
    DataListComponent.decorators = [
        { type: i0.Component, args: [{
                    selector: 'dashboard-data-list',
                    moduleId: module.id,
                    template: "<div class=\"content\">\n    <div class=\"ui grid\">\n        <div class=\"{{layoutColumnOneWidth}} wide column\">\n            <dashboard-typeahead-input\n                (selectionEvent)=\"filterListBySearchString($event)\"\n                [placeHolderText]=\"placeHolderText\"\n                [searchList]=\"objectTitleList\"\n                [typeAheadIsInMenu]=\"typeAheadIsInMenu\">\n            </dashboard-typeahead-input>\n            <dashboard-filter-list #filterComponent\n                                   (tagSelectEvent)=\"filterTag.updateFilterList($event)\"\n                                   [facet_tags]=\"facetTags\">\n\n            </dashboard-filter-list>\n        </div>\n        <div class=\"{{layoutColumnTwoWidth}} wide column\">\n            <br>\n            <dashboard-filter-tag #filterTag\n                                  (updateFilterListEvent)=\"filterListByTags($event)\">\n\n            </dashboard-filter-tag>\n\n            <div class=\"list-content\">\n                <ng-template [ngForOf]=\"objectList\" [ngForTemplate]=\"template\" ngFor></ng-template>\n            </div>\n        </div>\n    </div>\n</div>\n",
                    styles: [".content{background-color:#f7f7f7;margin-left:25px;margin-top:25px;padding:5px}hr{background:hsla(0,0%,43.1%,.1)!important;border:0;color:hsla(0,0%,43.1%,.1)!important;font-size:0;height:2px!important}.list-content{margin-top:20px;max-height:700px;overflow-y:scroll;padding-right:10px}"]
                },] }
    ];
    DataListComponent.propDecorators = {
        objectList: [{ type: i0.Input }],
        objectTitleList: [{ type: i0.Input }],
        placeHolderText: [{ type: i0.Input }],
        layoutColumnOneWidth: [{ type: i0.Input }],
        layoutColumnTwoWidth: [{ type: i0.Input }],
        listHeader: [{ type: i0.Input }],
        typeAheadIsInMenu: [{ type: i0.Input }],
        facetTags: [{ type: i0.Input }],
        template: [{ type: i0.ContentChild, args: [i0.TemplateRef, { static: true },] }]
    };

    var DataListModule = /** @class */ (function () {
        function DataListModule() {
        }
        return DataListModule;
    }());
    DataListModule.decorators = [
        { type: i0.NgModule, args: [{
                    imports: [
                        common.CommonModule,
                        FacetModule,
                        TypeAheadInputModule,
                        i1.HttpClientModule,
                        animations$1.BrowserAnimationsModule
                    ],
                    declarations: [
                        DataListComponent
                    ],
                    providers: [],
                    exports: [
                        DataListComponent
                    ]
                },] }
    ];

    var AddGadgetModule = /** @class */ (function () {
        function AddGadgetModule() {
        }
        return AddGadgetModule;
    }());
    AddGadgetModule.decorators = [
        { type: i0.NgModule, args: [{
                    imports: [
                        common.CommonModule,
                        DataListModule,
                        i1.HttpClientModule
                    ],
                    declarations: [
                        AddGadgetComponent
                    ],
                    providers: [
                        AddGadgetService
                    ],
                    exports: [
                        AddGadgetComponent
                    ]
                },] }
    ];

    var NotificationComponent = /** @class */ (function () {
        function NotificationComponent() {
            this.notificationFilters = [];
            this.currentNotificationFilter = {
                name: '',
                count: 0
            };
            this.modalHeader = 'Notifications';
            this.notificationFilters.push({ name: 'Unacknowledged', count: 4 });
            this.notificationFilters.push({ name: 'All Notifications', count: 4 });
            this.notificationFilters.push({ name: 'Trend Gadget', count: 4 });
            this.notificationFilters.push({ name: 'CPU Gadget', count: 0 });
            this.setSelectedNotificationFilter({ name: 'Unacknowledged', count: 4 });
        }
        NotificationComponent.prototype.ngOnInit = function () {
        };
        NotificationComponent.prototype.ngAfterViewInit = function () {
        };
        NotificationComponent.prototype.setSelectedNotificationFilter = function (item) {
            this.currentNotificationFilter = item;
        };
        return NotificationComponent;
    }());
    NotificationComponent.decorators = [
        { type: i0.Component, args: [{
                    moduleId: module.id,
                    selector: 'dashboard-notification-modal',
                    template: "<!-- <div class=\"ui long modal\" #notificationModalTag> -->\n<br>\n<div class=\"header\">\n    <h2><span style=\"padding: 30px; color:white\">{{modalHeader}}</span></h2>\n</div>\n\n\n<!--todo this should be two tables, one for the top filters and another for the component specific filters -->\n<table class=\"ui inverted selectable celled table\">\n    <tbody>\n    <tr (click)=\"setSelectedNotificationFilter(item)\"\n        *ngFor=\"let item of notificationFilters; let idx = index\">\n        <td [ngClass]=\"{'selected': item.name == currentNotificationFilter.name, 'components': idx==1}\">\n            <div style=\"float:left;text-align:left\">{{item.name}}</div>\n            <div style=\"float:right;text-align: right\">{{item.count}}</div>\n        </td>\n    </tr>\n    </tbody>\n</table>\n\n<div>\n\n    <div *ngIf=\"currentNotificationFilter\">\n\n        <dashboard-notification-detail></dashboard-notification-detail>\n    </div>\n\n</div>\n\n\n\n",
                    styles: [".components{border-bottom:solid!important;border-bottom-color:hsla(0,0%,50.2%,.69)!important;border-bottom-width:thin!important}td.selected{background-color:#4fa6d3!important;color:#fff;font-weight:lighter}td{line-height:1.7}.ui.table tr td{border:none}tr{cursor:pointer}"]
                },] }
    ];
    NotificationComponent.ctorParameters = function () { return []; };

    var NotificationService = /** @class */ (function () {
        function NotificationService() {
        }
        return NotificationService;
    }());
    NotificationService.decorators = [
        { type: i0.Injectable }
    ];
    NotificationService.ctorParameters = function () { return []; };

    var Notification = /** @class */ (function () {
        function Notification(name, description) {
            this.name = name;
            this.description = description;
        }
        Notification.prototype.setState = function (state) {
            this.state = state;
        };
        Notification.prototype.getState = function () {
            return this.state;
        };
        Notification.prototype.setIcon = function (icon) {
            this.icon = icon;
        };
        Notification.prototype.getIcon = function () {
            return this.icon;
        };
        Notification.prototype.setWhen = function (when) {
            this.when = when;
        };
        Notification.prototype.getWhen = function () {
            return this.when;
        };
        return Notification;
    }());

    /**
     * Created by jayhamilton on 5/16/17.
     */
    var NotificationDetailComponent = /** @class */ (function () {
        function NotificationDetailComponent() {
            this.notifications = [];
            var notification = new Notification('CPU Gadget', 'this is a cpu gadget error');
            notification.setWhen((new Date()).toString());
            this.notifications.push(notification);
            notification = new Notification('Trend Gadget', 'this is a trend gadget error2');
            notification.setWhen((new Date()).toString());
            this.notifications.push(notification);
            notification = new Notification('Disk Gadget', 'Disk 1 alert');
            notification.setWhen((new Date()).toString());
            this.notifications.push(notification);
            notification = new Notification('Disk Gadget', 'Disk 2 alert');
            notification.setWhen((new Date()).toString());
            this.notifications.push(notification);
        }
        return NotificationDetailComponent;
    }());
    NotificationDetailComponent.decorators = [
        { type: i0.Component, args: [{
                    selector: 'dashboard-notification-detail',
                    moduleId: module.id,
                    template: "<h3>&nbsp;&nbsp;Details</h3>\n<table class=\"ui inverted table\">\n    <tr *ngFor=\"let notification of notifications\">\n        <td style=\"font-weight:100 !important;\">\n            <div>\n                {{notification.description}}\n                <br>\n                {{notification.when}}\n            </div>\n        </td>\n    </tr>\n</table>\n\n\n\n\n\n"
                },] }
    ];
    NotificationDetailComponent.ctorParameters = function () { return []; };

    var NotificationModule = /** @class */ (function () {
        function NotificationModule() {
        }
        return NotificationModule;
    }());
    NotificationModule.decorators = [
        { type: i0.NgModule, args: [{
                    imports: [
                        common.CommonModule
                    ],
                    declarations: [NotificationComponent, NotificationDetailComponent],
                    providers: [NotificationService],
                    exports: [NotificationComponent]
                },] }
    ];

    var ObservableWebSocketService = /** @class */ (function () {
        function ObservableWebSocketService() {
        }
        ObservableWebSocketService.prototype.createObservableWebSocket = function (url) {
            var _this = this;
            this.ws = new WebSocket(url);
            return new rxjs.Observable(function (observer) {
                _this.ws.onmessage = function (event) { return observer.next(event.data); };
                _this.ws.onerror = function (event) { return observer.error(event); };
                _this.ws.onclose = function (event) { return observer.complete(); };
            });
        };
        ObservableWebSocketService.prototype.sendMessage = function (message) {
            this.ws.send(message);
        };
        return ObservableWebSocketService;
    }());
    ObservableWebSocketService.decorators = [
        { type: i0.Injectable }
    ];

    var AboutService = /** @class */ (function () {
        function AboutService(_http) {
            this._http = _http;
            this.env = environment;
        }
        AboutService.prototype.getAPIVersion = function () {
            var url = '/assets/api/version-model.json';
            return this._http.get(url)
                .pipe(operators.catchError(RuntimeService.handleError));
        };
        return AboutService;
    }());
    AboutService.ɵprov = i0.ɵɵdefineInjectable({ factory: function AboutService_Factory() { return new AboutService(i0.ɵɵinject(i1.HttpClient)); }, token: AboutService, providedIn: "root" });
    AboutService.decorators = [
        { type: i0.Injectable, args: [{ providedIn: 'root' },] }
    ];
    AboutService.ctorParameters = function () { return [
        { type: i1.HttpClient }
    ]; };

    /**
     * Created by jayhamilton on 1/24/17.
     */
    /**
     * Message Modal - clasable modal with message
     *
     * Selector message-modal
     *
     * Methods
     *      popMessageModal - display a message modal for a sepcified duration
     *      showMessageModal - show the message modal
     *      hideMessageModal - hide the message modal
     */
    var AboutComponent = /** @class */ (function () {
        function AboutComponent(_aboutService) {
            this._aboutService = _aboutService;
            this.modalHeader = 'About';
            this.env = environment;
        }
        AboutComponent.prototype.ngAfterViewInit = function () {
            this.getVersion();
        };
        AboutComponent.prototype.getVersion = function () {
            var _this = this;
            this._aboutService.getAPIVersion().subscribe(function (data) {
                _this.apiVersion = data['version'];
            });
        };
        return AboutComponent;
    }());
    AboutComponent.decorators = [
        { type: i0.Component, args: [{
                    selector: 'dashboard-about-modal',
                    moduleId: module.id,
                    template: "<br>\n\n<div style=\"text-align: center\">\n    <h2>{{modalHeader}}</h2>\n</div>\n\n<hr>\n\n<br>\n<br>\n\n<div style=\"text-align: center\">\n    <h4>{{env.productName}}</h4>\n</div>\n\n\n<div style=\"text-align: center\">\n    <h4>Version {{env.productVersion}}</h4>\n</div>\n\n<br>\n<br>\n\n<div style=\"text-align: center\">\n    <h4>API Version</h4>\n</div>\n\n<div style=\"text-align: center\">\n    <h4>{{apiVersion}}</h4>\n</div>\n\n\n\n\n\n      ",
                    styles: ["ul{text-align:center}h2,h3,h4{color:#fff!important;font-weight:400!important}"]
                },] }
    ];
    AboutComponent.ctorParameters = function () { return [
        { type: AboutService }
    ]; };

    var AboutModule = /** @class */ (function () {
        function AboutModule() {
        }
        return AboutModule;
    }());
    AboutModule.decorators = [
        { type: i0.NgModule, args: [{
                    imports: [
                        common.CommonModule
                    ],
                    declarations: [AboutComponent],
                    exports: [AboutComponent],
                    providers: [AboutService]
                },] }
    ];

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation.

    Permission to use, copy, modify, and/or distribute this software for any
    purpose with or without fee is hereby granted.

    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
    PERFORMANCE OF THIS SOFTWARE.
    ***************************************************************************** */
    /* global Reflect, Promise */
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b)
                if (Object.prototype.hasOwnProperty.call(b, p))
                    d[p] = b[p]; };
        return extendStatics(d, b);
    };
    function __extends(d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    }
    var __assign = function () {
        __assign = Object.assign || function __assign(t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s)
                    if (Object.prototype.hasOwnProperty.call(s, p))
                        t[p] = s[p];
            }
            return t;
        };
        return __assign.apply(this, arguments);
    };
    function __rest(s, e) {
        var t = {};
        for (var p in s)
            if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
                t[p] = s[p];
        if (s != null && typeof Object.getOwnPropertySymbols === "function")
            for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
                if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                    t[p[i]] = s[p[i]];
            }
        return t;
    }
    function __decorate(decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function")
            r = Reflect.decorate(decorators, target, key, desc);
        else
            for (var i = decorators.length - 1; i >= 0; i--)
                if (d = decorators[i])
                    r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    }
    function __param(paramIndex, decorator) {
        return function (target, key) { decorator(target, key, paramIndex); };
    }
    function __metadata(metadataKey, metadataValue) {
        if (typeof Reflect === "object" && typeof Reflect.metadata === "function")
            return Reflect.metadata(metadataKey, metadataValue);
    }
    function __awaiter(thisArg, _arguments, P, generator) {
        function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
        return new (P || (P = Promise))(function (resolve, reject) {
            function fulfilled(value) { try {
                step(generator.next(value));
            }
            catch (e) {
                reject(e);
            } }
            function rejected(value) { try {
                step(generator["throw"](value));
            }
            catch (e) {
                reject(e);
            } }
            function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
            step((generator = generator.apply(thisArg, _arguments || [])).next());
        });
    }
    function __generator(thisArg, body) {
        var _ = { label: 0, sent: function () { if (t[0] & 1)
                throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
        return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function () { return this; }), g;
        function verb(n) { return function (v) { return step([n, v]); }; }
        function step(op) {
            if (f)
                throw new TypeError("Generator is already executing.");
            while (_)
                try {
                    if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done)
                        return t;
                    if (y = 0, t)
                        op = [op[0] & 2, t.value];
                    switch (op[0]) {
                        case 0:
                        case 1:
                            t = op;
                            break;
                        case 4:
                            _.label++;
                            return { value: op[1], done: false };
                        case 5:
                            _.label++;
                            y = op[1];
                            op = [0];
                            continue;
                        case 7:
                            op = _.ops.pop();
                            _.trys.pop();
                            continue;
                        default:
                            if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) {
                                _ = 0;
                                continue;
                            }
                            if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) {
                                _.label = op[1];
                                break;
                            }
                            if (op[0] === 6 && _.label < t[1]) {
                                _.label = t[1];
                                t = op;
                                break;
                            }
                            if (t && _.label < t[2]) {
                                _.label = t[2];
                                _.ops.push(op);
                                break;
                            }
                            if (t[2])
                                _.ops.pop();
                            _.trys.pop();
                            continue;
                    }
                    op = body.call(thisArg, _);
                }
                catch (e) {
                    op = [6, e];
                    y = 0;
                }
                finally {
                    f = t = 0;
                }
            if (op[0] & 5)
                throw op[1];
            return { value: op[0] ? op[1] : void 0, done: true };
        }
    }
    var __createBinding = Object.create ? (function (o, m, k, k2) {
        if (k2 === undefined)
            k2 = k;
        Object.defineProperty(o, k2, { enumerable: true, get: function () { return m[k]; } });
    }) : (function (o, m, k, k2) {
        if (k2 === undefined)
            k2 = k;
        o[k2] = m[k];
    });
    function __exportStar(m, o) {
        for (var p in m)
            if (p !== "default" && !Object.prototype.hasOwnProperty.call(o, p))
                __createBinding(o, m, p);
    }
    function __values(o) {
        var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
        if (m)
            return m.call(o);
        if (o && typeof o.length === "number")
            return {
                next: function () {
                    if (o && i >= o.length)
                        o = void 0;
                    return { value: o && o[i++], done: !o };
                }
            };
        throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }
    function __read(o, n) {
        var m = typeof Symbol === "function" && o[Symbol.iterator];
        if (!m)
            return o;
        var i = m.call(o), r, ar = [], e;
        try {
            while ((n === void 0 || n-- > 0) && !(r = i.next()).done)
                ar.push(r.value);
        }
        catch (error) {
            e = { error: error };
        }
        finally {
            try {
                if (r && !r.done && (m = i["return"]))
                    m.call(i);
            }
            finally {
                if (e)
                    throw e.error;
            }
        }
        return ar;
    }
    function __spread() {
        for (var ar = [], i = 0; i < arguments.length; i++)
            ar = ar.concat(__read(arguments[i]));
        return ar;
    }
    function __spreadArrays() {
        for (var s = 0, i = 0, il = arguments.length; i < il; i++)
            s += arguments[i].length;
        for (var r = Array(s), k = 0, i = 0; i < il; i++)
            for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
                r[k] = a[j];
        return r;
    }
    ;
    function __await(v) {
        return this instanceof __await ? (this.v = v, this) : new __await(v);
    }
    function __asyncGenerator(thisArg, _arguments, generator) {
        if (!Symbol.asyncIterator)
            throw new TypeError("Symbol.asyncIterator is not defined.");
        var g = generator.apply(thisArg, _arguments || []), i, q = [];
        return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;
        function verb(n) { if (g[n])
            i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }
        function resume(n, v) { try {
            step(g[n](v));
        }
        catch (e) {
            settle(q[0][3], e);
        } }
        function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
        function fulfill(value) { resume("next", value); }
        function reject(value) { resume("throw", value); }
        function settle(f, v) { if (f(v), q.shift(), q.length)
            resume(q[0][0], q[0][1]); }
    }
    function __asyncDelegator(o) {
        var i, p;
        return i = {}, verb("next"), verb("throw", function (e) { throw e; }), verb("return"), i[Symbol.iterator] = function () { return this; }, i;
        function verb(n, f) { i[n] = o[n] ? function (v) { return (p = !p) ? { value: __await(o[n](v)), done: n === "return" } : f ? f(v) : v; } : f; }
    }
    function __asyncValues(o) {
        if (!Symbol.asyncIterator)
            throw new TypeError("Symbol.asyncIterator is not defined.");
        var m = o[Symbol.asyncIterator], i;
        return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
        function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
        function settle(resolve, reject, d, v) { Promise.resolve(v).then(function (v) { resolve({ value: v, done: d }); }, reject); }
    }
    function __makeTemplateObject(cooked, raw) {
        if (Object.defineProperty) {
            Object.defineProperty(cooked, "raw", { value: raw });
        }
        else {
            cooked.raw = raw;
        }
        return cooked;
    }
    ;
    var __setModuleDefault = Object.create ? (function (o, v) {
        Object.defineProperty(o, "default", { enumerable: true, value: v });
    }) : function (o, v) {
        o["default"] = v;
    };
    function __importStar(mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k))
                    __createBinding(result, mod, k);
        __setModuleDefault(result, mod);
        return result;
    }
    function __importDefault(mod) {
        return (mod && mod.__esModule) ? mod : { default: mod };
    }
    function __classPrivateFieldGet(receiver, privateMap) {
        if (!privateMap.has(receiver)) {
            throw new TypeError("attempted to get private field on non-instance");
        }
        return privateMap.get(receiver);
    }
    function __classPrivateFieldSet(receiver, privateMap, value) {
        if (!privateMap.has(receiver)) {
            throw new TypeError("attempted to set private field on non-instance");
        }
        privateMap.set(receiver, value);
        return value;
    }

    /**
     * Created by jayhamilton on 2/3/17.
     */
    var PropertyBase = /** @class */ (function () {
        function PropertyBase(props) {
            if (props === void 0) { props = {}; }
            this.value = props.value;
            this.key = props.key || '';
            this.label = props.label || '';
            this.required = !props.required;
            this.order = props.order === undefined ? 1 : props.order;
            this.controlType = props.controlType || '';
            this.options = props.options;
        }
        return PropertyBase;
    }());

    var DropdownProperty = /** @class */ (function (_super) {
        __extends(DropdownProperty, _super);
        function DropdownProperty(options) {
            if (options === void 0) { options = {}; }
            var _this = _super.call(this, options) || this;
            _this.controlType = 'dropdown';
            _this.options = [];
            _this.options = options['options'] || [];
            return _this;
        }
        return DropdownProperty;
    }(PropertyBase));

    var TextboxProperty = /** @class */ (function (_super) {
        __extends(TextboxProperty, _super);
        function TextboxProperty(options) {
            if (options === void 0) { options = {}; }
            var _this = _super.call(this, options) || this;
            _this.controlType = 'textbox';
            _this.type = options['type'] || '';
            return _this;
        }
        return TextboxProperty;
    }(PropertyBase));

    var HiddenProperty = /** @class */ (function (_super) {
        __extends(HiddenProperty, _super);
        function HiddenProperty(options) {
            if (options === void 0) { options = {}; }
            var _this = _super.call(this, options) || this;
            _this.controlType = 'hidden';
            _this.type = 'hidden';
            return _this;
        }
        return HiddenProperty;
    }(PropertyBase));

    var CheckboxProperty = /** @class */ (function (_super) {
        __extends(CheckboxProperty, _super);
        function CheckboxProperty(options) {
            if (options === void 0) { options = {}; }
            var _this = _super.call(this, options) || this;
            _this.controlType = 'checkbox';
            _this.type = 'checkbox'; // options['type'] || '';
            return _this;
        }
        return CheckboxProperty;
    }(PropertyBase));

    var DynamicDropdownProperty = /** @class */ (function (_super) {
        __extends(DynamicDropdownProperty, _super);
        function DynamicDropdownProperty(options) {
            if (options === void 0) { options = {}; }
            var _this = _super.call(this, options) || this;
            _this.controlType = 'dynamicdropdown';
            _this.options = [];
            _this.options = options['options'] || [];
            return _this;
        }
        return DynamicDropdownProperty;
    }(PropertyBase));

    var NumberProperty = /** @class */ (function (_super) {
        __extends(NumberProperty, _super);
        function NumberProperty(options) {
            if (options === void 0) { options = {}; }
            var _this = _super.call(this, options) || this;
            _this.controlType = 'number';
            _this.type = 'number'; // options['type']|| '';
            return _this;
        }
        return NumberProperty;
    }(PropertyBase));

    var GadgetPropertyService = /** @class */ (function () {
        function GadgetPropertyService() {
        }
        GadgetPropertyService.prototype.setPropertiesAndValues = function (defaultProperties, properties) {
            var ctrl;
            properties.length = 0;
            defaultProperties.forEach(function (property) {
                if (property.controlType === 'dropdown') {
                    ctrl = new DropdownProperty(property);
                    properties.push(ctrl);
                }
                else if (property.controlType === 'textbox') {
                    ctrl = new TextboxProperty(property);
                    properties.push(ctrl);
                }
                else if (property.controlType === 'checkbox') {
                    ctrl = new CheckboxProperty(property);
                    properties.push(ctrl);
                }
                else if (property.controlType === 'hidden') {
                    ctrl = new HiddenProperty(property);
                    properties.push(ctrl);
                }
                else if (property.controlType === 'number') {
                    ctrl = new NumberProperty(property);
                    properties.push(ctrl);
                }
                else if (property.controlType === 'dynamicdropdown') {
                    ctrl = new DynamicDropdownProperty(property);
                    properties.push(ctrl);
                }
            });
            properties.sort(function (a, b) { return a.order - b.order; });
        };
        GadgetPropertyService.prototype.setPropertyPagesAndProperties = function (defaultPropertyPages, propertyPages) {
            var me = this;
            // for each defaultPropertyPage object, get the properties
            defaultPropertyPages.forEach(function (propertyPage) {
                var newPropertyPage = {};
                for (var property in propertyPage) {
                    if (propertyPage.hasOwnProperty(property)) {
                        if (property !== 'properties') {
                            newPropertyPage[property] = propertyPage[property];
                        }
                        else {
                            var properties = [];
                            me.setPropertiesAndValues(propertyPage.properties, properties);
                            newPropertyPage['properties'] = properties;
                        }
                    }
                }
                propertyPages.push(newPropertyPage);
            });
        };
        return GadgetPropertyService;
    }());
    GadgetPropertyService.decorators = [
        { type: i0.Injectable }
    ];
    GadgetPropertyService.ctorParameters = function () { return []; };

    var MenuModule = /** @class */ (function () {
        function MenuModule() {
        }
        return MenuModule;
    }());
    MenuModule.decorators = [
        { type: i0.NgModule, args: [{
                    imports: [
                        common.CommonModule,
                        NotificationModule,
                        AddGadgetModule,
                        LayoutModule,
                        AboutModule,
                        ConfigurationModule,
                        TypeAheadInputModule,
                        button.MatButtonModule,
                        icon.MatIconModule
                    ],
                    providers: [
                        RuntimeService,
                        ConfigurationService,
                        GadgetPropertyService,
                        ObservableWebSocketService,
                        MenuEventService
                    ],
                    declarations: [
                        MenuComponent
                    ],
                    exports: [
                        MenuComponent
                    ]
                },] }
    ];

    /**a
     * Board component
     *
     */
    var BoardComponent = /** @class */ (function () {
        function BoardComponent() {
            this.title = 'app works';
            this.showMenu = true;
        }
        return BoardComponent;
    }());
    BoardComponent.decorators = [
        { type: i0.Component, args: [{
                    selector: 'dashboard-board',
                    moduleId: module.id,
                    template: "<!--\n\nThe grid is primarily controlled by MenuService Events. You can change the behavior by\nchanging the MenuComponent and have it call methods within the Grid component via Output Events within the Menu component or by using the @ViewChild approach.\n\nIn the current code, the MenuService is shared by the GridComponent and MenuComponent. The MenuComponent relays events from the various components that make up the MenuComponent. The GridComponent listens for those events via an Observable. The GridComponent will also raise/emit an event that will be picked up by the MenuComponent via the MenuService through an Observable as well.\n\n-->\n<dashboard-menu [show]=\"showMenu\"></dashboard-menu>\n<dashboard-grid></dashboard-grid>\n"
                },] }
    ];
    BoardComponent.propDecorators = {
        title: [{ type: i0.Input }],
        showMenu: [{ type: i0.Input }]
    };

    var ToastComponent = /** @class */ (function () {
        function ToastComponent(_toastService) {
            this._toastService = _toastService;
        }
        ToastComponent.prototype.ngOnInit = function () {
            var _this = this;
            this.messages = this._toastService.getMessages();
            rxjs.timer(7000, 7000).subscribe(function (t) {
                _this.messages.forEach(function (message) {
                    _this.dismiss(message.id);
                });
            });
        };
        ToastComponent.prototype.dismiss = function (id) {
            this._toastService.dismissMessage(id);
        };
        return ToastComponent;
    }());
    ToastComponent.decorators = [
        { type: i0.Component, args: [{
                    selector: 'dashboard-toast',
                    template: "<div class=\"wrapper\">\n    <div *ngFor=\"let message of messages\" class=\"toast\">\n        <table (click)=\"dismiss(message.id)\" *ngIf=\"message.dismissed == false\" [@showHide] class=\"ui table\">\n            <tr>\n                <td class=\"toast-icon\" width='50px'><i\n                    class=\"large inverted check icon\"></i></td>\n                <td>\n                    <div class=\"toast-message\">\n                        {{message.content}}\n                    </div>\n                </td>\n            </tr>\n        </table>\n    </div>\n</div>\n",
                    animations: [
                        animations.trigger('showHide', [
                            animations.transition(':enter', [
                                animations.style({ opacity: 0 }),
                                animations.animate('500ms', animations.style({ opacity: 1 }))
                            ]),
                            animations.transition(':leave', [
                                animations.style({ opacity: 1 }),
                                animations.animate('500ms', animations.style({ opacity: 0 }))
                            ])
                        ])
                    ],
                    styles: [".wrapper{position:absolute;right:10px;top:50px;width:400px;z-index:1000}.toast{width:inherit}.toast-icon{background-color:#384b5f;text-align:center}.ui[class*=\"top right attached\"].label{background-color:transparent}.toast-message{display:table-cell;height:40px;vertical-align:middle}"]
                },] }
    ];
    ToastComponent.ctorParameters = function () { return [
        { type: ToastService }
    ]; };

    var ReversePipe = /** @class */ (function () {
        function ReversePipe() {
        }
        ReversePipe.prototype.transform = function (value, args) {
            value = lodash.filter(value, ['dismissed', false]);
            return lodash.reverse(value);
        };
        return ReversePipe;
    }());
    ReversePipe.decorators = [
        { type: i0.Pipe, args: [{
                    name: 'reverse'
                },] }
    ];

    var ToastModule = /** @class */ (function () {
        function ToastModule() {
        }
        return ToastModule;
    }());
    ToastModule.decorators = [
        { type: i0.NgModule, args: [{
                    imports: [
                        common.CommonModule
                    ],
                    declarations: [ToastComponent, ReversePipe],
                    providers: [ToastService],
                    exports: [ToastComponent]
                },] }
    ];

    /**
     * Created by jayhamilton on 1/28/17.
     */
    /**
     * todo - the name of this service does not represent the file name. This should be refactored. Consider moving this service to the gadget module instead.
     */
    var GadgetInstanceService = /** @class */ (function () {
        function GadgetInstanceService() {
            this.concreteGadgetInstances = [];
            this.subject = new rxjs.Subject();
            this.subscribers = [];
        }
        GadgetInstanceService.prototype.addInstance = function (gadget) {
            var gadgetFound = this.concreteGadgetInstances.findIndex(function (instance) {
                return gadget.instanceId === instance['instance']['instanceId'];
            }) >= 0;
            if (gadgetFound === false) {
                this.concreteGadgetInstances.push(gadget);
            }
        };
        GadgetInstanceService.prototype.enableConfigureMode = function () {
            this.concreteGadgetInstances.forEach(function (gadget) {
                gadget.instance.toggleConfigMode();
            });
        };
        GadgetInstanceService.prototype.removeInstance = function (id) {
            console.log('REMOVING GADGET');
            // remove instance representation from model
            this.model.rows.forEach(function (row) {
                row.columns.forEach(function (column) {
                    if (column.gadgets) {
                        for (var i = column.gadgets.length - 1; i >= 0; i--) {
                            if (column.gadgets[i].instanceId === id) {
                                column.gadgets.splice(i, 1);
                                break;
                            }
                        }
                    }
                });
            });
            // removes concrete instance from service
            for (var x = this.concreteGadgetInstances.length - 1; x >= 0; x--) {
                if (this.concreteGadgetInstances[x].instance.instanceId === id) {
                    var _gadget = this.concreteGadgetInstances.splice(x, 1);
                    _gadget[0].destroy();
                    break;
                }
            }
            // raise an event indicating a gadget was removed
            this.subject.next('gadget id: ' + id);
        };
        GadgetInstanceService.prototype.getInstanceCount = function () {
            return this.concreteGadgetInstances.length;
        };
        /*
         this allows this service to update the board when a delete operation occurs
         */
        GadgetInstanceService.prototype.setCurrentModel = function (model) {
            this.model = model;
        };
        /*
         raise an event that the grid.component is listening for when a gadget is removed.
         */
        GadgetInstanceService.prototype.listenForInstanceRemovedEventsFromGadgets = function () {
            return this.subject.asObservable();
        };
        GadgetInstanceService.prototype.addSubscriber = function (subscriber) {
            this.subscribers.push(subscriber);
        };
        GadgetInstanceService.prototype.unSubscribeAll = function () {
            this.subscribers.forEach(function (subscription) {
                subscription.unsubscribe();
            });
            this.subscribers.length = 0;
            this.clearAllInstances();
        };
        GadgetInstanceService.prototype.clearAllInstances = function () {
            this.concreteGadgetInstances.length = 0;
        };
        return GadgetInstanceService;
    }());
    GadgetInstanceService.decorators = [
        { type: i0.Injectable }
    ];
    GadgetInstanceService.ctorParameters = function () { return []; };

    /**
     * Created by jayhamilton on 6/15/17.
     */
    var GadgetConfigModel = /** @class */ (function () {
        function GadgetConfigModel(config) {
            var _this = this;
            this.propertyPages = [];
            config.propertyPages.forEach(function (page) {
                var props = [];
                page.properties.forEach(function (prop) {
                    switch (prop.controlType) {
                        case 'textbox':
                        case 'dropdown':
                        case 'dynamicdropdown':
                            props.push(new PropertyBase(prop));
                            break;
                        case 'checkbox':
                            props.push(new PropertyBase(prop));
                            break;
                        case 'hidden':
                            props.push(new PropertyBase(prop));
                            break;
                        default:
                            props.push(new PropertyBase(prop));
                            break;
                    }
                });
                _this.propertyPages.push(new PropertyPage(page.displayName, page.groupId, page.position, props));
            });
        }
        return GadgetConfigModel;
    }());
    var PropertyPage = /** @class */ (function () {
        function PropertyPage(displayName, groupId, position, properties) {
            this.displayName = displayName;
            this.groupId = groupId;
            this.position = position;
            this.properties = properties;
        }
        return PropertyPage;
    }());

    var GridComponent = /** @class */ (function () {
        /**
         * Todo - split model and board operations. This class should really focus on an individual board model's operations
         * within the grid. The board specific operations should be moved to the board component.
         * @param _gadgetInstanceService
         * @param _procmonConfigurationService
         */
        function GridComponent(_gadgetInstanceService, _configurationService, _gadgetLibraryService, _toastService, _menuEventService) {
            this._gadgetInstanceService = _gadgetInstanceService;
            this._configurationService = _configurationService;
            this._gadgetLibraryService = _gadgetLibraryService;
            this._toastService = _toastService;
            this._menuEventService = _menuEventService;
            this.boardUpdateEvent = new i0.EventEmitter();
            this.model = {};
            this.noGadgets = true;
            this.dropZone1 = null;
            this.dropZone2 = null;
            this.dropZone3 = null;
            this.gadgetLibrary = [];
            /** todo
             * Temporary objects for experimenting with AI
             * @type
             */
            this.gridInsertionPosition = {
                x: 0,
                y: 0
            };
            this.removeOldListeners();
            this.setupEventListeners();
            this.initializeBoard();
            this.getGadgetLibrary();
        }
        /**
         * todo - This is a temporary attempt to avoid emitting events from stale listeners.
         * Most severe symptom is when you drill down and then change the layout.
         * Multiple events are triggered per action due to the services not
         * getting destroyed when coming into the main board from a child route. The end result is multiple gadget instances
         * appearing. The following code improves the condition but there still are issues with multiple gadgets appearing
         * when changing the layout.
         *
         */
        GridComponent.prototype.removeOldListeners = function () {
            this._gadgetInstanceService.unSubscribeAll();
            this._menuEventService.unSubscribeAll();
        };
        GridComponent.prototype.setupEventListeners = function () {
            var _this = this;
            var gadgetRemoveEventSubscriber = this._gadgetInstanceService
                .listenForInstanceRemovedEventsFromGadgets().subscribe(function (message) {
                _this.saveBoard('Gadget Removed From Board: ' + message, false);
            });
            var menuEventSubscriber = this._menuEventService.listenForMenuEvents().subscribe(function (event) {
                var edata = event['data'];
                switch (event['name']) {
                    case 'boardChangeLayoutEvent':
                        _this.updateBoardLayout(edata);
                        break;
                    case 'boardSelectEvent':
                        _this.loadBoard(edata);
                        break;
                    case 'boardCreateEvent':
                        _this.createBoard(edata);
                        break;
                    case 'boardEditEvent':
                        _this.editBoard(edata);
                        break;
                    case 'boardDeleteEvent':
                        _this.deleteBoard(edata);
                        break;
                    case 'boardAddGadgetEvent':
                        _this.addGadget(edata);
                        break;
                    case 'boardAIAddGadgetEvent':
                        _this.addGadgetUsingArtificialIntelligence(edata);
                        break;
                }
            });
            this._gadgetInstanceService.addSubscriber(gadgetRemoveEventSubscriber);
            this._menuEventService.addSubscriber(menuEventSubscriber);
        };
        /**
         *
         * This is experimental code that deals with AI
         */
        GridComponent.prototype.getGadgetLibrary = function () {
            var _this = this;
            this._gadgetLibraryService.getGadgetLibrary().subscribe(function (data) {
                _this.gadgetLibrary.length = 0;
                var me = _this;
                data.library.forEach(function (item) {
                    me.gadgetLibrary.push(item);
                });
            });
        };
        GridComponent.prototype.getGadgetFromLibrary = function (gadgetType) {
            var gadgetObject = null;
            this.gadgetLibrary.forEach(function (gadget) {
                if (gadgetType.localeCompare(gadget['componentType']) === 0) {
                    gadgetObject = gadget;
                }
            });
            return gadgetObject;
        };
        GridComponent.prototype.addGadgetUsingArtificialIntelligence = function (aiObject) {
            /** todo
             * make confidence code configurable
             */
            if (aiObject && aiObject.operation) {
                switch (aiObject.operation) {
                    case 'get_storage':
                        this.addGadget(this.getGadgetFromLibrary('StorageObjectListComponent'));
                        break;
                    case 'get_cpu':
                        this.addGadget(this.getGadgetFromLibrary('CPUGadgetComponent'));
                        break;
                }
            }
        };
        /**
         * This is the end of the experimental AI code.
         */
        GridComponent.prototype.updateGadgetPositionInBoard = function ($event, columnNumber, rowNumber, type) {
            var _this = this;
            console.log($event['item']['data']);
            console.log(columnNumber + ' ' + rowNumber);
            var moveComplete = false;
            this.getModel().rows.forEach(function (row) {
                var colpos = 0;
                row.columns.forEach(function (column) {
                    var gadgetpos = 0;
                    if (column.gadgets) {
                        column.gadgets.forEach(function (_gadget) {
                            if (_gadget.instanceId === $event['item']['data'] && !moveComplete) {
                                var gadget = column.gadgets.splice(gadgetpos, 1);
                                if (!_this.getModel().rows[rowNumber].columns[columnNumber].gadgets) {
                                    _this.getModel().rows[rowNumber].columns[columnNumber].gadgets = [];
                                }
                                _this.getModel().rows[rowNumber].columns[columnNumber].gadgets.push(gadget[0]);
                                _this.saveBoard('drag drop operation', false);
                                moveComplete = true;
                            }
                            gadgetpos++;
                        });
                        colpos++;
                    }
                });
            });
        };
        GridComponent.prototype.createBoard = function (name) {
            this.loadNewBoard(name);
        };
        GridComponent.prototype.editBoard = function (name) {
        };
        GridComponent.prototype.deleteBoard = function (name) {
            var _this = this;
            this._configurationService.deleteBoard(name).subscribe(function (data) {
                _this.initializeBoard();
            }, function (error) { return console.error('Deletion error', error); }, function () { return console.debug('Board Deletion: ' + name); });
        };
        GridComponent.prototype.addGadget = function (gadget) {
            var _gadget = Object.assign({}, gadget);
            _gadget.instanceId = new Date().getTime();
            _gadget.config = new GadgetConfigModel(gadget.config);
            this.setGadgetInsertPosition();
            var x = this.gridInsertionPosition.x;
            var y = this.gridInsertionPosition.y;
            if (!this.getModel().rows[x].columns[y].gadgets) {
                this.getModel().rows[x].columns[y].gadgets = [];
            }
            this.getModel().rows[x].columns[y].gadgets.push(_gadget);
            this.saveBoard('Adding Gadget To The Board', false);
        };
        GridComponent.prototype.updateBoardLayout = function (structure) {
            console.log('IN UPDATE BOARD LAYOUT');
            // user selected the currently selected layout
            if (structure.id === this.getModel().id) {
                return;
            }
            // copy the current board's model
            var _model = Object.assign({}, this.getModel());
            // get just the columns that contain gadgets from all rows
            var originalColumns = this.readColumnsFromOriginalModel(_model);
            // reset the copied model's rows, which include columns
            _model.rows.length = 0;
            // copy the contents of the requested structure into the temporary model
            // we now have a board model we can populate with the original board's gadgets
            Object.assign(_model.rows, structure.rows);
            _model.structure = structure.structure;
            _model.id = structure.id;
            var originalColumnIndexToStartProcessingFrom = 0;
            /* For each column from the original board, copy its gadgets to the new structure.
             The requested layout may have more or less columns than defined by the original layout. So the fillGridStructure method
             will copy column content into the target. If there are more columns than the target,
             the fillGridStructure will return the count of remaining columns to be processed and then process those.
             */
            while (originalColumnIndexToStartProcessingFrom < originalColumns.length) {
                originalColumnIndexToStartProcessingFrom = this.fillGridStructure(_model, originalColumns, originalColumnIndexToStartProcessingFrom);
            }
            // This will copy the just processed model and present it to the board
            this.setModel(_model);
            // clear temporary object
            for (var member in _model) {
                delete _model[member];
            }
            // persist the board change
            this.saveBoard('Grid Layout Update', false);
        };
        GridComponent.prototype.enableConfigMode = function () {
            this._gadgetInstanceService.enableConfigureMode();
        };
        GridComponent.prototype.setModel = function (model) {
            this.model = Object.assign({}, model);
        };
        GridComponent.prototype.getModel = function () {
            return this.model;
        };
        GridComponent.prototype.onDrop = function (data) {
            console.log(data);
        };
        GridComponent.prototype.updateGridState = function () {
            var gadgetCount = 0;
            if (this.getModel().rows) {
                this.getModel().rows.forEach(function (row) {
                    row.columns.forEach(function (column) {
                        if (column.gadgets) {
                            column.gadgets.forEach(function (gadget) {
                                gadgetCount++;
                            });
                        }
                    });
                });
            }
            this.noGadgets = !gadgetCount;
            this.dashedStyle = {
                'border-style': this.noGadgets ? 'dashed' : 'none',
                'border-width': this.noGadgets ? '2px' : 'none',
                'border-color': this.noGadgets ? 'darkgray' : 'none',
                'padding': this.noGadgets ? '5px' : 'none'
            };
        };
        GridComponent.prototype.readColumnsFromOriginalModel = function (_model) {
            var columns = [];
            _model.rows.forEach(function (row) {
                row.columns.forEach(function (col) {
                    columns.push(col);
                });
            });
            return columns;
        };
        GridComponent.prototype.fillGridStructure = function (destinationModelStructure, originalColumns, counter) {
            var me = this;
            destinationModelStructure.rows.forEach(function (row) {
                row.columns.forEach(function (destinationColumn) {
                    if (!destinationColumn.gadgets) {
                        destinationColumn.gadgets = [];
                    }
                    if (originalColumns[counter]) {
                        me.copyGadgets(originalColumns[counter], destinationColumn);
                        counter++;
                    }
                });
            });
            return counter;
        };
        GridComponent.prototype.copyGadgets = function (source, target) {
            if (source.gadgets && source.gadgets.length > 0) {
                var w = source.gadgets.shift();
                while (w) {
                    target.gadgets.push(w);
                    w = source.gadgets.shift();
                }
            }
        };
        GridComponent.prototype.initializeBoard = function () {
            var _this = this;
            this._configurationService.getBoards().subscribe(function (boards) {
                if (boards && boards instanceof Array && boards.length) {
                    _this.boards = boards.sort(function (a, b) {
                        return a.boardInstanceId - b.boardInstanceId;
                    });
                    _this.loadBoard(_this.boards[0].title);
                }
                else {
                    _this.loadDefaultBoard();
                }
            });
        };
        GridComponent.prototype.loadBoard = function (boardTitle) {
            this.clearGridModelAndGadgetInstanceStructures();
            var selectedBoard = this.boards.find(function (board) { return board.title === boardTitle; });
            this.setModel(selectedBoard);
            this.updateServicesAndGridWithModel();
            this.boardUpdateEvent.emit(boardTitle);
        };
        GridComponent.prototype.loadDefaultBoard = function () {
            var _this = this;
            this.clearGridModelAndGadgetInstanceStructures();
            this._configurationService.getDefaultBoard().subscribe(function (board) {
                _this.setModel(board);
                _this.updateServicesAndGridWithModel();
                _this.saveBoard('Initialization of a default board', true);
            });
        };
        GridComponent.prototype.loadNewBoard = function (name) {
            var _this = this;
            this.clearGridModelAndGadgetInstanceStructures();
            this._configurationService.getDefaultBoard().subscribe(function (res) {
                _this.setModel(res);
                _this.getModel().title = name;
                _this.getModel().boardInstanceId = new Date().getTime();
                _this.updateServicesAndGridWithModel();
                _this.saveBoard('Initialization of a new board', true);
            });
        };
        GridComponent.prototype.updateServicesAndGridWithModel = function () {
            this._gadgetInstanceService.setCurrentModel(this.getModel());
            this._configurationService.setCurrentModel(this.getModel());
            this.updateGridState();
        };
        GridComponent.prototype.saveBoard = function (operation, alertBoardListenerThatTheMenuShouldBeUpdated) {
            var _this = this;
            this.updateServicesAndGridWithModel();
            this._configurationService.saveBoard(this.getModel()).subscribe(function (result) {
                _this._toastService.sendMessage(_this.getModel().title + ' has been updated!', '');
                if (alertBoardListenerThatTheMenuShouldBeUpdated) {
                    _this._menuEventService.raiseGridEvent({ name: 'boardUpdateEvent', data: _this.getModel().title });
                }
            }, function (error) { return console.error('Error' + error); }, function () { return console.debug('Saving configuration to store!'); });
        };
        GridComponent.prototype.clearGridModelAndGadgetInstanceStructures = function () {
            // clear gadgetInstances
            this._gadgetInstanceService.clearAllInstances();
            // clear current model
            for (var prop in this.getModel()) {
                if (this.model.hasOwnProperty(prop)) {
                    delete this.model[prop];
                }
            }
        };
        GridComponent.prototype.setGadgetInsertPosition = function () {
            for (var x = 0; x < this.getModel().rows.length; x++) {
                for (var y = 0; y < this.getModel().rows[x].columns.length; y++) {
                    if (this.getModel().rows[x].columns[y].gadgets && this.getModel().rows[x].columns[y].gadgets.length === 0) {
                        this.gridInsertionPosition.x = x;
                        this.gridInsertionPosition.y = y;
                        return;
                    }
                }
            }
            // we go here because the board is either empty or full
            // insert in the top left most cell
            this.gridInsertionPosition.y = 0;
            if (this.noGadgets) {
                // there are no gadgets so insert in top row
                this.gridInsertionPosition.x = 0;
            }
            else {
                // board is full so insert in the last row
                this.gridInsertionPosition.x = this.getModel().rows.length - 1;
            }
        };
        return GridComponent;
    }());
    GridComponent.decorators = [
        { type: i0.Component, args: [{
                    moduleId: module.id,
                    selector: 'dashboard-grid',
                    template: "<div class=\"ui centered stackable grid\" style=\"margin-left: 5px; margin-right: 5px\"\n>\n    <div *ngFor=\"let row of model.rows; let r = index\" [ngClass]=\"{'short-row': row.columns.length > 3, 'tall-row': row.columns.length <= 3 }\"\n         class=\"row\">\n\n        <ng-container cdkDropListGroup>\n            <div (cdkDropListDropped)=\"updateGadgetPositionInBoard($event, c , r, 'column')\" *ngFor=\"let col of row.columns; let c = index\" cdkDropList\n                 class=\"{{col.styleClass}} column\">\n\n                <div *ngIf=\"!col.gadgets || col.gadgets.length== 0\" [ngClass]=\"{'short-row': row.columns.length > 3, 'tall-row': row.columns.length <= 3 }\"\n                     class=\"ui center aligned massive info message\"\n                     style=\"text-align:center\">\n\n                    <span style=\"font-size: .75em\">Add Gadget!</span>\n\n                </div>\n\n                <dashboard-grid-cell *ngFor=\"let cellGadget of col.gadgets\" [cdkDragData]=\"cellGadget.instanceId\" [gadgetConfig]=\"cellGadget.config\"\n                                     [gadgetInstanceId]=\"cellGadget.instanceId\"\n                                     [gadgetTags]=\"cellGadget.tags\"\n                                     [gadgetType]=\"cellGadget.componentType\"\n                                     cdkDrag\n                                     class=\"target-box\"\n                                     style=\"display:inline-block; width:100%\"\n                >\n                    <div *cdkDragPlaceholder class=\"target-box-placeholder\"></div>\n                </dashboard-grid-cell>\n            </div>\n        </ng-container>\n    </div>\n</div>\n<dashboard-toast></dashboard-toast>\n",
                    styles: [".ui.info.message{background-color:#c8cbce;color:#fff}.ui.attached.info.message,.ui.info.message{box-shadow:inset 0 0 0 0 #c8cbce,0 0 0 0 transparent}.short-row{min-height:150px}.tall-row{min-height:370px}.target-box-placeholder{background:transparent;border:none;min-height:400px;transition:transform .25s cubic-bezier(0,0,.2,1)}.target-box{align-items:center;box-sizing:border-box;color:rgba(0,0,0,.87);cursor:move;display:inline-block;flex-direction:row;font-size:14px;justify-content:center;padding:5px}"]
                },] }
    ];
    GridComponent.ctorParameters = function () { return [
        { type: GadgetInstanceService },
        { type: ConfigurationService },
        { type: AddGadgetService },
        { type: ToastService },
        { type: MenuEventService }
    ]; };
    GridComponent.propDecorators = {
        boardUpdateEvent: [{ type: i0.Output }]
    };

    var GadgetFactory = /** @class */ (function () {
        function GadgetFactory() {
        }
        GadgetFactory.getComponentType = function (gadgetType) {
            return GadgetFactory.componentMap.get(gadgetType);
        };
        GadgetFactory.setComponentType = function (gadgetType, componentRef) {
            GadgetFactory.componentMap.set(gadgetType, componentRef);
        };
        return GadgetFactory;
    }());
    GadgetFactory.componentMap = new Map();

    /*
     this class handles the dynamic creation of components
     */
    var CellComponent = /** @class */ (function () {
        function CellComponent(cfr, gadgetInstanceService) {
            this.cfr = cfr;
            this.gadgetInstanceService = gadgetInstanceService;
        }
        CellComponent.prototype.ngOnInit = function () {
            /*
             create component instance dynamically
             */
            var component = GadgetFactory.getComponentType(this.gadgetType);
            if (component) {
                var compFactory = this.cfr.resolveComponentFactory(component);
                var gadgetRef = this.viewContainerRef.createComponent(compFactory);
                /*
                 we need to pass the input parameters (instance id and config) back into the newly created component.
                 */
                gadgetRef.instance.configureGadget(this.gadgetInstanceId, this.gadgetConfig, this.gadgetTags);
                /*
                 add concrete component to service for tracking
                 */
                this.gadgetInstanceService.addInstance(gadgetRef);
            }
        };
        return CellComponent;
    }());
    CellComponent.decorators = [
        { type: i0.Component, args: [{
                    selector: 'dashboard-grid-cell',
                    template: '<ng-template #container></ng-template>'
                },] }
    ];
    CellComponent.ctorParameters = function () { return [
        { type: i0.ComponentFactoryResolver },
        { type: GadgetInstanceService }
    ]; };
    CellComponent.propDecorators = {
        gadgetType: [{ type: i0.Input }],
        gadgetConfig: [{ type: i0.Input }],
        gadgetInstanceId: [{ type: i0.Input }],
        gadgetTags: [{ type: i0.Input }],
        viewContainerRef: [{ type: i0.ViewChild, args: ['container', { static: true, read: i0.ViewContainerRef },] }]
    };

    var GridModule = /** @class */ (function () {
        function GridModule() {
        }
        GridModule.withComponents = function (components) {
            return {
                ngModule: GridModule,
                providers: [
                    { provide: i0.ANALYZE_FOR_ENTRY_COMPONENTS, useValue: components, multi: true }
                ]
            };
        };
        return GridModule;
    }());
    GridModule.decorators = [
        { type: i0.NgModule, args: [{
                    imports: [
                        common.CommonModule,
                        forms.FormsModule,
                        ToastModule,
                        i1.HttpClientModule,
                        dragDrop.DragDropModule
                    ],
                    declarations: [
                        GridComponent,
                        CellComponent
                    ],
                    exports: [
                        GridComponent
                    ],
                    providers: [
                        GadgetInstanceService,
                        ConfigurationService,
                        AddGadgetService
                    ]
                },] }
    ];

    var BoardModule = /** @class */ (function () {
        function BoardModule() {
        }
        return BoardModule;
    }());
    BoardModule.decorators = [
        { type: i0.NgModule, args: [{
                    imports: [
                        common.CommonModule,
                        GridModule,
                        MenuModule,
                    ],
                    providers: [],
                    declarations: [
                        BoardComponent
                    ],
                    exports: [
                        BoardComponent
                    ]
                },] }
    ];

    /**
     * Created by jayhamilton on 2/28/17.
     */
    /**
     * Created by jayhamilton on 2/26/17.
     */
    var GadgetHeaderComponent = /** @class */ (function () {
        function GadgetHeaderComponent() {
            this.removeEvent = new i0.EventEmitter();
            this.toggleConfigModeEvent = new i0.EventEmitter();
            this.runEvent = new i0.EventEmitter();
            this.stopEvent = new i0.EventEmitter();
            this.helpEvent = new i0.EventEmitter();
        }
        GadgetHeaderComponent.prototype.remove = function () {
            this.removeEvent.emit();
        };
        GadgetHeaderComponent.prototype.toggleConfigMode = function () {
            this.toggleConfigModeEvent.emit();
        };
        GadgetHeaderComponent.prototype.run = function () {
            this.runEvent.emit();
        };
        GadgetHeaderComponent.prototype.stop = function () {
            this.stopEvent.emit();
        };
        GadgetHeaderComponent.prototype.help = function () {
            this.helpEvent.emit();
        };
        return GadgetHeaderComponent;
    }());
    GadgetHeaderComponent.decorators = [
        { type: i0.Component, args: [{
                    moduleId: module.id,
                    selector: 'dashboard-gadget-header',
                    template: "<div class=\"ui top attached label\">\n\n    <button (click)=\"remove()\"\n            *ngIf=\"showControls || globalOptions.enableHover\" class=\"compact ui button right floated\">\n        <i class=\"trash icon\" style=\"margin-right:0 !important\"></i>\n    </button>\n    <button (click)=\"toggleConfigMode()\"\n            *ngIf=\"showControls || globalOptions.enableHover && showConfigurationControl\" class=\"compact ui button right floated\">\n        <i class=\"setting  icon\" style=\"margin-right:0 !important\"></i>\n    </button>\n    <button (click)=\"help()\"\n            *ngIf=\"showControls || globalOptions.enableHover\" class=\"compact ui button right floated\">\n        <i class=\"info icon\" style=\"margin-right:0 !important\"></i>\n    </button>\n\n    <dashboard-gadget-operation-control (runEvent)=\"run()\"\n                                        (stopEvent)=\"stop()\"\n                                        *ngIf=\"showControls || globalOptions.enableHover\"\n                                        [actionInitiated]=\"actionInitiated\"\n                                        [gadgetHasOperationControls]=\"gadgetHasOperationControls\"\n                                        [inConfig]=\"inConfig\"\n                                        [inRun]=\"inRun\"\n                                        [showOperationControls]=\"showOperationControls\">\n\n    </dashboard-gadget-operation-control>\n\n    <div class=\"ui top left floated label ct-title\">{{title}}</div>\n</div>\n<div class=\"spacer\"></div>\n",
                    styles: [".gadget{color:grey;font-family:Helvetica Neue,Helvetica,Open Sans,Arial,Lucida Grande,sans-serif;margin-bottom:15px!important;min-height:inherit!important}:host /deep/ div:not(.proppages){display:block!important;margin-left:auto;margin-right:auto}.ui.form .field>label{color:grey;font-family:Helvetica Neue,Helvetica,Open Sans,Arial,Lucida Grande,sans-serif;font-size:1.3em!important;font-weight:300}#bt{text-align:center!important}input{border:none!important;font-size:1.3em!important;font-weight:300!important;outline:none!important;padding:5px;width:75%!important}.ct-title,input{font-family:Helvetica Neue,Helvetica,Open Sans,Arial,Lucida Grande,sans-serif}.ct-title{color:#585858!important;font-size:1.5em!important;font-weight:500!important;text-align:left!important}.ui.top.attached.label,.ui.top.left.floated.label{background-color:#fff}hr{border-style:solid;color:#000;font-weight:300!important;height:1px}.ui[class*=\"top attached\"].label:before{border-bottom:0 solid #d0d3d6;bottom:5px;content:\"\";height:1px;left:2%;position:absolute;width:95%}.spacer{height:2em}.ui.items>.item>.content>.description{color:#a9a9a9!important}.suggestions>li{background-color:#fff!important}td{color:grey}td,th{font-family:Helvetica Neue,Helvetica,Open Sans,Arial,Lucida Grande,sans-serif}th{color:#369}.example-full-width{width:100%}h4{font-weight:400}.ui.segment{border:none;box-shadow:none}"]
                },] }
    ];
    GadgetHeaderComponent.propDecorators = {
        title: [{ type: i0.Input }],
        showControls: [{ type: i0.Input }],
        inRun: [{ type: i0.Input }],
        inConfig: [{ type: i0.Input }],
        actionInitiated: [{ type: i0.Input }],
        showOperationControls: [{ type: i0.Input }],
        showConfigurationControl: [{ type: i0.Input }],
        gadgetHasOperationControls: [{ type: i0.Input }],
        globalOptions: [{ type: i0.Input }],
        removeEvent: [{ type: i0.Output }],
        toggleConfigModeEvent: [{ type: i0.Output }],
        runEvent: [{ type: i0.Output }],
        stopEvent: [{ type: i0.Output }],
        helpEvent: [{ type: i0.Output }]
    };

    /**
     * Created by jayhamilton on 6/29/17.
     */
    var GadgetOperationComponent = /** @class */ (function () {
        function GadgetOperationComponent() {
            this.runEvent = new i0.EventEmitter();
            this.stopEvent = new i0.EventEmitter();
        }
        GadgetOperationComponent.prototype.run = function () {
            this.runEvent.emit();
        };
        GadgetOperationComponent.prototype.stop = function () {
            this.stopEvent.emit();
        };
        return GadgetOperationComponent;
    }());
    GadgetOperationComponent.decorators = [
        { type: i0.Component, args: [{
                    moduleId: module.id,
                    selector: 'dashboard-gadget-operation-control',
                    template: "\n\n        <button class=\"compact ui button right floated\"\n                *ngIf=\"!inRun && !actionInitiated && showOperationControls && gadgetHasOperationControls\"\n                (click)=\"run()\"><i class=\"green play icon\" style=\"margin-right:0 !important\"></i>\n        </button>\n\n        <button class=\"compact ui button right floated\"\n                *ngIf=\"!inRun &&\n        actionInitiated &&\n        showOperationControls &&\n        gadgetHasOperationControls\">\n            <i class=\"black spinner loading icon\" style=\"margin-right:0 !important\"></i>\n        </button>\n\n        <button class=\"compact ui button right floated\"\n                *ngIf=\"inRun && !actionInitiated && showOperationControls && gadgetHasOperationControls\"\n                (click)=\"stop()\"><i class=\"red stop icon\" style=\"margin-right:0 !important\"></i>\n        </button>\n    "
                },] }
    ];
    GadgetOperationComponent.propDecorators = {
        runEvent: [{ type: i0.Output }],
        stopEvent: [{ type: i0.Output }],
        inRun: [{ type: i0.Input }],
        actionInitiated: [{ type: i0.Input }],
        inConfig: [{ type: i0.Input }],
        showOperationControls: [{ type: i0.Input }],
        gadgetHasOperationControls: [{ type: i0.Input }]
    };

    /**
     * Created by jayhamilton on 1/24/17.
     */
    /**
     * Message Modal - clasable modal with message
     *
     * Selector message-modal
     *
     * Methods
     *      popMessageModal - display a message modal for a sepcified duration
     *      showMessageModal - show the message modal
     *      hideMessageModal - hide the message modal
     */
    var HelpModalComponent = /** @class */ (function () {
        function HelpModalComponent() {
        }
        HelpModalComponent.prototype.showMessageModal = function (icon, header, message) {
            this.modalicon = icon;
            this.modalheader = header;
            this.modalconfig = message;
            this.configModal.modal('show');
        };
        HelpModalComponent.prototype.hideMessageModal = function () {
            this.modalicon = '';
            this.modalheader = '';
            this.modalconfig = '';
            this.configModal.modal('hide');
        };
        HelpModalComponent.prototype.ngAfterViewInit = function () {
            this.configModal = jQuery(this.helpmodalaRef.nativeElement);
            this.configModal.modal('hide');
        };
        HelpModalComponent.prototype.showHelp = function () {
            this.showMessageModal(null, 'Help', 'Get me out of here!');
        };
        return HelpModalComponent;
    }());
    HelpModalComponent.decorators = [
        { type: i0.Component, args: [{
                    selector: 'dashboard-help-modal',
                    moduleId: module.id,
                    template: "<div #helpmodal_tag class=\"ui long modal\">\n    <div class=\"header\">\n        Help\n    </div>\n\n    <div class=\"ui basic segment\">\n        <br>\n        <br>\n        <p></p>\n        <br><br>\n    </div>\n    <div class=\"actions\">\n        <div class=\"ui approve button\">Close</div>\n    </div>\n</div>\n"
                },] }
    ];
    HelpModalComponent.ctorParameters = function () { return []; };
    HelpModalComponent.propDecorators = {
        topic: [{ type: i0.Input }],
        helpmodalaRef: [{ type: i0.ViewChild, args: ['helpmodal_tag', { static: true },] }]
    };

    /**
     * Created by jayhamilton on 1/24/17.
     */
    /**
     * Message Modal - clasable modal with message
     *
     * Selector message-modal
     *
     * Methods
     *      popMessageModal - display a message modal for a sepcified duration
     *      showMessageModal - show the message modal
     *      hideMessageModal - hide the message modal
     */
    var VisDrillDownComponent = /** @class */ (function () {
        function VisDrillDownComponent() {
            this.objectList = [];
            this.objectTitleList = [];
            this.placeHolderText = 'Begin typing vm name';
            this.layoutColumnOneWidth = 'six';
            this.layoutColumnTwoWidth = 'ten';
            this.facetTags = [];
        }
        VisDrillDownComponent.prototype.showMessageModal = function (icon, header, message) {
            this.modalicon = icon;
            this.modalheader = header;
            this.modalconfig = message;
            this.configModal.modal('show');
        };
        VisDrillDownComponent.prototype.hideMessageModal = function () {
            this.modalicon = '';
            this.modalheader = '';
            this.modalconfig = '';
            this.configModal.modal('hide');
        };
        VisDrillDownComponent.prototype.ngAfterViewInit = function () {
            this.configModal = jQuery(this.vismodalaRef.nativeElement);
            this.configModal.modal('hide');
        };
        VisDrillDownComponent.prototype.showDrillDownDetail = function ($event) {
            var data = JSON.stringify($event, null, 4);
            this.showMessageModal(null, 'Detail', data);
        };
        VisDrillDownComponent.prototype.showDetail = function ($event) {
            var data = JSON.stringify($event, null, 4);
            this.showMessageModal(null, 'Detail', data);
        };
        return VisDrillDownComponent;
    }());
    VisDrillDownComponent.decorators = [
        { type: i0.Component, args: [{
                    selector: 'dashboard-vis-drill-down-modal',
                    moduleId: module.id,
                    template: "<div #vismodal_tag class=\"ui long modal\">\n    <div class=\"header\">\n        <h2>{{modalheader}}</h2>\n    </div>\n\n\n    <div class=\"content\">\n\n        {{modalconfig}}\n\n    </div>\n\n\n    <div class=\"actions\">\n        <div class=\"ui approve button\">Close</div>\n    </div>\n</div>\n",
                    animations: [
                        animations.trigger('contentSwitch', [
                            animations.state('inactive', animations.style({
                                opacity: 0
                            })),
                            animations.state('active', animations.style({
                                opacity: 1
                            })),
                            animations.transition('inactive => active', animations.animate('750ms ease-in')),
                            animations.transition('active => inactive', animations.animate('750ms ease-out'))
                        ]),
                        animations.trigger('tabSwitch', [
                            animations.state('inactive', animations.style({
                                opacity: .75
                            })),
                            animations.state('active', animations.style({
                                opacity: 1
                            })),
                            animations.transition('inactive => active', animations.animate('750ms ease-in')),
                            animations.transition('active => inactive', animations.animate('750ms ease-out'))
                        ])
                    ]
                },] }
    ];
    VisDrillDownComponent.ctorParameters = function () { return []; };
    VisDrillDownComponent.propDecorators = {
        vismodalaRef: [{ type: i0.ViewChild, args: ['vismodal_tag', { static: true },] }]
    };

    var GadgetSharedModule = /** @class */ (function () {
        function GadgetSharedModule() {
        }
        return GadgetSharedModule;
    }());
    GadgetSharedModule.decorators = [
        { type: i0.NgModule, args: [{
                    imports: [
                        common.CommonModule,
                        progressBar.MatProgressBarModule,
                    ],
                    declarations: [
                        GadgetHeaderComponent,
                        GadgetOperationComponent,
                        HelpModalComponent,
                        VisDrillDownComponent
                    ],
                    exports: [
                        GadgetHeaderComponent,
                        GadgetOperationComponent,
                        HelpModalComponent,
                        VisDrillDownComponent
                    ]
                },] }
    ];

    var DetailService = /** @class */ (function () {
        function DetailService(_http) {
            this._http = _http;
            this.testURL = 'http://localhost:8080';
            this.detailURL = '/detail';
            this.configure();
        }
        DetailService.prototype.configure = function () {
            if (!environment.production) {
                this.detailURL = this.testURL + this.detailURL;
            }
        };
        DetailService.prototype.getDetailByChartSeriesSelected = function (chartType, chartSeries, chartMetric, endPointName) {
            /**
             * Todo - review these parameters and make them more meaningful. The goal is to send to the service
             * a pameterized URL /endpointURL?detailParameter={value or id}
             */
            var p = new i1.HttpParams();
            p = p.append('detailParam', chartSeries);
            p = p.append('detailMetric', chartMetric);
            return this._http.get(this.detailURL, { params: p })
                .pipe(operators.catchError(RuntimeService.handleError));
        };
        DetailService.prototype.getDetail = function (url) {
            return this._http.get(url)
                .pipe(operators.catchError(RuntimeService.handleError));
        };
        DetailService.prototype.getRecord = function (url) {
            return this._http.get(url)
                .pipe(operators.catchError(RuntimeService.handleError));
        };
        return DetailService;
    }());
    DetailService.decorators = [
        { type: i0.Injectable }
    ];
    DetailService.ctorParameters = function () { return [
        { type: i1.HttpClient }
    ]; };

    /**a
     * Detail component
     *
     */
    var DetailComponent = /** @class */ (function () {
        function DetailComponent(_router, _route, _detailService) {
            var _this = this;
            this._router = _router;
            this._route = _route;
            this._detailService = _detailService;
            this.chartType = '';
            this.chartSeries = '';
            this.chartMetric = '';
            this.endPointName = '';
            this.data = [];
            this.searchText = '';
            this.navRoutes = [];
            this.objectAsArray = [];
            this.navigationSubscription = this._router.events.subscribe(function (e) {
                // If it is a NavigationEnd event re-initalise the component
                if (e instanceof router.NavigationEnd) {
                    _this.getObjectsByMetric(true);
                }
            });
        }
        DetailComponent.prototype.ngOnInit = function () {
            this.chartType = this._route.snapshot.queryParams['chartType'];
            this.chartSeries = this._route.snapshot.queryParams['chartSeries'];
            this.chartMetric = this._route.snapshot.queryParams['chartMetric'];
            this.endPointName = this._route.snapshot.queryParams['endPointName'];
            this.getObjectsByMetric(false);
        };
        DetailComponent.prototype.getObject = function (record) {
            var _this = this;
            this.clearDetailDisplay();
            Object.keys(record).forEach(function (key) {
                if (key.indexOf('link') < 0) {
                    _this.objectAsArray.push({ 'key': key, 'value': record[key] });
                }
            });
        };
        DetailComponent.prototype.getObjectsByHateoasLink = function (detail) {
            var _this = this;
            this.clearDetailDisplay();
            var href = '';
            detail.links.forEach(function (link) {
                if (link.rel === 'self') {
                    href = link.href;
                }
            });
            var navArray = href.split('/');
            //trying to get data for the current record so do nothing. Avoid this altogether by removing the link from the table.
            if (navArray[navArray.length - 1] == this.navRoutes[this.navRoutes.length - 1]) {
                return;
            }
            this.navRoutes.push(navArray[navArray.length - 1]);
            this._detailService.getDetail(href).subscribe(function (data) {
                _this.data = data.slice();
            });
        };
        DetailComponent.prototype.getObjectsByMetric = function (isReload) {
            var _this = this;
            this.clearDetailDisplay();
            this._detailService.getDetailByChartSeriesSelected(this.chartType, this.chartSeries, this.chartMetric, this.endPointName).subscribe(function (data) {
                _this.data = data.slice();
            });
            if (!isReload) {
                this.navRoutes.push(this.chartMetric);
            }
            else {
                /**
                 * todo - this won't work for many routes.
                 */
                while (this.navRoutes.length > 1) {
                    this.navRoutes.splice(this.navRoutes.length - 1, 1);
                }
            }
        };
        DetailComponent.prototype.goHome = function () {
            this._router.navigate(['/main-board']);
        };
        DetailComponent.prototype.gotToRoute = function (nav, disabled) {
            if (!disabled) {
                this._router.navigate(['/detail'], {
                    queryParams: {
                        chartType: this.chartType,
                        chartSeries: this.chartSeries,
                        chartMetric: this.chartMetric,
                        endPointName: this.endPointName
                    }
                });
            }
        };
        DetailComponent.prototype.clearDetailDisplay = function () {
            this.objectAsArray = [];
        };
        return DetailComponent;
    }());
    DetailComponent.decorators = [
        { type: i0.Component, args: [{
                    moduleId: module.id,
                    template: "<div class=\"ui centered stackable grid\" style=\"margin-left: 5px; margin-right: 5px\">\n    <div class=\"row\">\n        <div class=\"three wide column\"></div>\n        <div class=\"ten wide column left aligned\">\n\n            <div class=\"ui breadcrumb\">\n                <a (click)=\"goHome()\" class=\"section\">Board</a>\n\n                <span *ngFor=\"let nav of navRoutes,let i = index\">\n                    <span class=\"divider\">/</span>\n                    <a (click)=\"gotToRoute(nav, navRoutes.length -1 == i)\" [ngClass]=\"{'active': navRoutes.length -1 == i }\"\n                       class=\"section\">{{nav}}</a>\n                </span>\n\n            </div>\n\n            <div class=\"ui segment\">\n                <div class=\"ui top right attached label\"><a href=\"http://localhost:8080/detail/downloadFile\"\n                                                            target=\"_self\"><i class=\"big grey download icon\"></i></a>\n                </div>\n\n                <div class=\"ui  basic segment\">\n                     <span class=\"detail-left-header\">\n                        Type: {{chartSeries}}\n                    </span>\n                    <span class=\"detail-right-header\">\n                       Count: {{data.length}} Tasks\n                    </span>\n\n                </div>\n                <div class=\"ui fluid icon input\">\n                    <i class=\"search icon\"></i>\n                    <input [(ngModel)]=\"searchText\" placeholder=\"search text goes here\">\n                </div>\n                <table class=\"ui celled compact fixed table\">\n                    <thead>\n                    <tr>\n                        <th></th>\n                        <th style=\"text-align: center\">Status</th>\n                        <th>Master</th>\n                        <th>Job Name</th>\n                        <th>Sub Type</th>\n                        <th>Job Id</th>\n                        <th>Task Id</th>\n                        <th>Node</th>\n                        <th>Disk</th>\n                        <th>Task Start</th>\n                        <th>Task End</th>\n                    </tr>\n                    </thead>\n                    <tbody>\n                    <tr *ngFor=\"let task of data | filter : searchText; let i = index \">\n                        <td [ngClass]=\"{'negative': task.status != 'Completed', 'positive': task.status == 'Completed' }\">\n                            {{i + 1}}\n                        </td>\n                        <td style=\"text-align: center\"><i [ngClass]=\"{\n                        'red ban': task.status != 'Completed',\n                        'green check': task.status == 'Completed'\n                        }\" class=\"icon large \"></i></td>\n                        <td>{{task.master}}</td>\n                        <td>{{task.jobName}}</td>\n                        <td>{{task.subType}}</td>\n                        <td><a (click)=\"getObjectsByHateoasLink(task)\">{{task.jobId}}</a></td>\n                        <td><a (click)=\"getObject(task)\">{{task.taskId}}</a></td>\n                        <td>{{task.node}}</td>\n                        <td>{{task.disk}}</td>\n                        <td>{{task.taskStart}}</td>\n                        <td>{{task.taskEnd}}</td>\n                    </tr>\n                    </tbody>\n                </table>\n            </div>\n        </div>\n\n        <div class=\"three wide column\">\n            <div class=\"ui breadcrumb\">\n                &nbsp;\n            </div>\n            <div *ngIf=\"objectAsArray.length\" class=\"ui  segment\">\n                <div class=\"ui top right attached label\"><i class=\"big download icon\"></i></div>\n\n                <div class=\"ui  basic segment\">\n                    <span class=\"detail-left-header\">Detail</span>\n                </div>\n\n                <table class=\"ui celled compact fixed table\">\n                    <tr *ngFor=\"let property of objectAsArray; \">\n                        <td class=\"field\">{{property.key}}</td>\n                        <td>{{property.value}}</td>\n                    </tr>\n\n                </table>\n            </div>\n        </div>\n    </div>\n</div>\n\n",
                    styles: ["a,a label{cursor:pointer}.field{background:#4a95c3;color:#fff}.ui .breadcrumb{font-size:1.28571429rem}.detail-right-header{float:right}.detail-left-header,.detail-right-header{color:#616161;font-size:1.2rem}.label{background:transparent}.active{color:grey!important;font-weight:400!important}"]
                },] }
    ];
    DetailComponent.ctorParameters = function () { return [
        { type: router.Router },
        { type: router.ActivatedRoute },
        { type: DetailService }
    ]; };

    var FilterPipe = /** @class */ (function () {
        function FilterPipe() {
        }
        FilterPipe.prototype.transform = function (items, searchText) {
            if (!items) {
                return [];
            }
            if (!searchText) {
                return items;
            }
            searchText = searchText.toLowerCase();
            return items.filter(function (it) {
                return it.node.toLowerCase().includes(searchText)
                    || it.master.toLowerCase().includes(searchText)
                    || it.jobName.toLowerCase().includes(searchText)
                    || it.status.toLowerCase().includes(searchText);
            });
        };
        return FilterPipe;
    }());
    FilterPipe.decorators = [
        { type: i0.Pipe, args: [{
                    name: 'filter'
                },] }
    ];

    var DetailModule = /** @class */ (function () {
        function DetailModule() {
        }
        return DetailModule;
    }());
    DetailModule.decorators = [
        { type: i0.NgModule, args: [{
                    imports: [
                        common.CommonModule,
                        forms.FormsModule
                    ],
                    declarations: [
                        DetailComponent,
                        FilterPipe
                    ],
                    providers: [
                        DetailService
                    ],
                    exports: [
                        DetailComponent
                    ]
                },] }
    ];

    /**
     * Created by jayhamilton on 2/5/17.
     */
    var DynamicFormPropertyComponent = /** @class */ (function () {
        function DynamicFormPropertyComponent() {
            this.endPoints = [];
            this.updateEndPointList();
        }
        Object.defineProperty(DynamicFormPropertyComponent.prototype, "isValid", {
            get: function () {
                return this.form.controls[this.property.key].valid;
            },
            enumerable: false,
            configurable: true
        });
        DynamicFormPropertyComponent.prototype.updateEndPointList = function () {
            /*
            this.endPointService.getEndPoints().subscribe(data => {

                this.endPoints = data['endPoint'].slice();

            });
             */
        };
        DynamicFormPropertyComponent.prototype.ngAfterViewInit = function () {
            //filter endpoints based on the gadgets tags
            var me = this;
            var eligibleEndpoints = [];
            this.endPoints.forEach(function (point, index, object) {
                var found = false;
                point['tags'].forEach(function (tag) {
                    me.gadgetTags.forEach(function (_gt) {
                        if (_gt.name.trim().toLowerCase() === tag.name.trim().toLowerCase()) {
                            found = true;
                        }
                    });
                });
                if (found) {
                    eligibleEndpoints.push(point);
                }
            });
            this.endPoints = eligibleEndpoints.slice();
        };
        return DynamicFormPropertyComponent;
    }());
    DynamicFormPropertyComponent.decorators = [
        { type: i0.Component, args: [{
                    moduleId: module.id,
                    selector: 'dashboard-df-property',
                    template: "<div [formGroup]=\"form\">\n\n    <div class=\"field\">\n        <label [attr.for]=\"property.key\">{{property.label}}</label>\n\n        <div [ngSwitch]=\"property.controlType\">\n\n            <input *ngSwitchCase=\"'textbox'\"\n                   [formControlName]=\"property.key\"\n                   [id]=\"property.key\"\n                   [type]=\"property.type\">\n\n            <select *ngSwitchCase=\"'dropdown'\"\n                    [formControlName]=\"property.key\"\n                    [id]=\"property.key\">\n\n                <option *ngFor=\"let opt of property.options\"\n                        [value]=\"opt.value\">{{opt.key}}\n                </option>\n            </select>\n\n            <!--todo - fix checkbox type. The value is not properly set by the data so using hard coded type -->\n            <input (change)=\"property.value = $event.target.checked\"\n                   *ngSwitchCase=\"'checkbox'\"\n                   [(ngModel)]=\"property.value\"\n                   [attr.checked]=\"property.value ? true : null\"\n                   [formControlName]=\"property.key\"\n                   [id]=\"property.key\"\n                   [type]=\"'checkbox'\">\n\n            <input *ngSwitchCase=\"'number'\"\n                   [formControlName]=\"property.key\"\n                   [id]=\"property.key\"\n                   [type]=\"'number'\">\n\n\n            <input *ngSwitchCase=\"'hidden'\"\n                   [formControlName]=\"property.key\"\n                   [id]=\"property.key\"\n                   [type]=\"property.type\">\n\n            <select (change)=\"property.value = $event.target.toString()\"\n                    *ngSwitchCase=\"'dynamicdropdown'\"\n                    [formControlName]=\"property.key\"\n                    [id]=\"property.key\">\n\n                <option></option>\n                <option *ngFor=\"let opt of endPoints\"\n                        [value]=\"opt.name\">{{opt.name}}\n                </option>\n\n            </select>\n        </div>\n    </div>\n\n    <div *ngIf=\"!isValid\" [@showHideAnimation] class=\"ui error message\">{{property.label}} is required</div>\n\n\n</div>\n",
                    animations: [
                        animations.trigger('showHideAnimation', [
                            animations.transition(':enter', [
                                animations.style({ opacity: 0 }),
                                animations.animate(750, animations.style({ opacity: 1 }))
                            ]),
                            animations.transition(':leave', [
                                animations.animate(750, animations.style({ opacity: 0 }))
                            ])
                        ])
                    ],
                    styles: [".field>label{color:#767676!important;font-size:1.1em!important;font-weight:300}.ui.tabular.menu .active.item{border-top-color:#2185d0!important;border-top-width:3px!important}.ui.segment{border:none}select{color:rgba(97,97,97,.68)!important;height:3em!important}input{border:thin solid #d3d3d3;border-radius:5px;line-height:2em;padding:5px;width:100%}"]
                },] }
    ];
    DynamicFormPropertyComponent.ctorParameters = function () { return []; };
    DynamicFormPropertyComponent.propDecorators = {
        property: [{ type: i0.Input }],
        form: [{ type: i0.Input }],
        gadgetTags: [{ type: i0.Input }]
    };

    /**
     * Created by jayhamilton on 2/3/17.
     */
    var PropertyControlService = /** @class */ (function () {
        function PropertyControlService() {
        }
        PropertyControlService.prototype.toFormGroupFromPP = function (propertyPages) {
            var group = {};
            propertyPages.forEach(function (propertyPage) {
                propertyPage.properties.forEach(function (property) {
                    group[property.key] = property.required ? new forms.FormControl(property.value
                        || '', forms.Validators.required) : new forms.FormControl(property.value
                        || '');
                });
            });
            return new forms.FormGroup(group);
        };
        return PropertyControlService;
    }());
    PropertyControlService.decorators = [
        { type: i0.Injectable }
    ];
    PropertyControlService.ctorParameters = function () { return []; };

    /**
     * Created by jayhamilton on 2/5/17.
     */
    var DynamicFormComponent = /** @class */ (function () {
        function DynamicFormComponent(pcs, configService, changeDetectionRef) {
            this.pcs = pcs;
            this.configService = configService;
            this.changeDetectionRef = changeDetectionRef;
            this.updatePropertiesEvent = new i0.EventEmitter(true);
            this.currentTab = 'run';
            this.state = 'inactive';
            this.lastActiveTab = {};
            this.form = new forms.FormGroup({});
            this.payLoad = '';
            this.showMessage = false;
        }
        Object.defineProperty(DynamicFormComponent.prototype, "isPropertyPageValid", {
            get: function () {
                return this.form.valid;
            },
            enumerable: false,
            configurable: true
        });
        /* better solution that solves error: Expression has changed after it was checked exception resolution*/
        DynamicFormComponent.prototype.ngAfterViewInit = function () {
            this.changeDetectionRef.detectChanges();
        };
        DynamicFormComponent.prototype.ngOnInit = function () {
            this.form = this.pcs.toFormGroupFromPP(this.propertyPages);
        };
        DynamicFormComponent.prototype.onSubmit = function () {
            this.payLoad = JSON.stringify(this.form.value);
            console.debug('Saving Form!');
            this.updatePropertiesEvent.emit(this.payLoad);
            console.debug('Sending configuration to the config service!');
            this.configService.notifyGadgetOnPropertyChange(this.payLoad, this.instanceId);
            if (this.payLoad) {
                this.showMessage = true;
                setTimeout(function () {
                    this.showMessage = false;
                }.bind(this), 2000);
            }
        };
        DynamicFormComponent.prototype.setCurrentTab = function (tab) {
            this.currentTab = tab.groupId;
        };
        return DynamicFormComponent;
    }());
    DynamicFormComponent.decorators = [
        { type: i0.Component, args: [{
                    /* solves error: Expression has changed after it was checked exception resolution - https://www.youtube.com/watch?v=K_BRcal-JfI*/
                    // changeDetection: ChangeDetectionStrategy.OnPush,
                    moduleId: module.id,
                    selector: 'dashboard-dynamic-form',
                    template: "<div class=\"ui form\" style=\"text-align: left !important\">\n    <form (ngSubmit)=\"onSubmit()\" [formGroup]=\"form\">\n\n        <div #tabComponentTag class=\"proppages ui top attached tabular menu\">\n            <a #tabComponentTag (click)=\"setCurrentTab(tab)\" *ngFor=\"let tab of propertyPages; let i = index\"\n               [ngClass]=\"{'active': tab.groupId == currentTab }\"\n               attr.data-tab=\"{{tab.groupId}}\" class=\"item\">{{tab.displayName}} </a>\n        </div>\n\n        <div *ngFor=\"let page of propertyPages; let i = index\"\n             [@contentSwitch]=\"page.groupId == currentTab ? 'active':'inactive'\" [ngClass]=\"{'active': page.groupId == currentTab }\"\n             attr.data-tab=\"{{page.groupId}}\"\n             class=\"proppages ui bottom attached tab segment\">\n\n\n            <div *ngFor=\"let property of page.properties\"\n                 class=\"form-row\">\n                <dashboard-df-property\n                    [form]=\"form\"\n                    [gadgetTags]=\"gadgetTags\"\n                    [property]=\"property\"\n                >\n\n                </dashboard-df-property>\n                <br>\n            </div>\n\n        </div>\n        <div class=\"form-row field\">\n            <button [disabled]=\"!form.valid || !form.dirty\" class=\"ui green submit button\" type=\"submit\"> Save</button>\n        </div>\n    </form>\n\n    <div *ngIf=\"showMessage\" [@showHideAnimation] class=\"form-row\">\n        <br>\n        <div class=\"ui success message\">Saved!</div>\n    </div>\n\n</div>\n",
                    animations: [
                        animations.trigger('contentSwitch', [
                            animations.state('inactive', animations.style({
                                opacity: 0
                            })),
                            animations.state('active', animations.style({
                                opacity: 1
                            })),
                            animations.transition('inactive => active', animations.animate('750ms ease-in')),
                            animations.transition('active => inactive', animations.animate('750ms ease-out'))
                        ]),
                        animations.trigger('showHideAnimation', [
                            animations.transition(':enter', [
                                animations.style({ opacity: 0 }),
                                animations.animate(750, animations.style({ opacity: 1 }))
                            ]),
                            animations.transition(':leave', [
                                animations.animate(750, animations.style({ opacity: 0 }))
                            ])
                        ])
                    ],
                    providers: [PropertyControlService],
                    styles: [".field>label{color:#767676!important;font-size:1.1em!important;font-weight:300}.ui.tabular.menu .active.item{border-top-color:#2185d0!important;border-top-width:3px!important}.ui.segment{border:none}select{color:rgba(97,97,97,.68)!important;height:3em!important}input{border:thin solid #d3d3d3;border-radius:5px;line-height:2em;padding:5px;width:100%}"]
                },] }
    ];
    DynamicFormComponent.ctorParameters = function () { return [
        { type: PropertyControlService },
        { type: ConfigurationService },
        { type: i0.ChangeDetectorRef }
    ]; };
    DynamicFormComponent.propDecorators = {
        gadgetTags: [{ type: i0.Input }],
        propertyPages: [{ type: i0.Input }],
        instanceId: [{ type: i0.Input }],
        updatePropertiesEvent: [{ type: i0.Output }]
    };

    var DynamicFormModule = /** @class */ (function () {
        function DynamicFormModule() {
        }
        return DynamicFormModule;
    }());
    DynamicFormModule.decorators = [
        { type: i0.NgModule, args: [{
                    imports: [
                        common.CommonModule,
                        forms.FormsModule,
                        forms.ReactiveFormsModule
                    ],
                    declarations: [
                        DynamicFormComponent,
                        DynamicFormPropertyComponent
                    ],
                    providers: [PropertyControlService],
                    exports: [
                        DynamicFormComponent,
                        DynamicFormPropertyComponent
                    ]
                },] }
    ];

    /**
     * Created by jayhamilton on 6/22/17.
     */
    var GadgetBase = /** @class */ (function () {
        function GadgetBase(_gadgetInstanceService, _propertyService, changeDetectionRef, _optionsService) {
            var _this = this;
            this._gadgetInstanceService = _gadgetInstanceService;
            this._propertyService = _propertyService;
            this.changeDetectionRef = changeDetectionRef;
            this._optionsService = _optionsService;
            /**
             * Used to determine when to show the controls that appear in the gadgets
             * heading area. This is set by the mouseover/mouseout events.
             * @type {boolean}
             */
            this.showControls = false;
            /**
             * determines whether to show the gadgets property page
             * @type {boolean}
             */
            this.inConfig = false;
            /**
             * Determines if a gadget is runnning or not
             * @type {boolean}
             */
            this.inRun = false;
            /**
             * When a gadget is manually put into run mode this property will be used to
             * display a spinning icon and will be enabled between intiating an operation (run or stop)
             * to the operation is enabled
             * @type {boolean}
             */
            this.actionInitiated = false;
            /**
             * Gadgets that are of type realtime have a run/stop set of controls.
             * Those gadgets should set this property to true. This property's visibility
             * will also be controlled by whether the gadget's configuration form is valid.
             * @type {boolean}
             */
            this.showOperationControls = false;
            /**
             * This property is used to simply allow the gadget to not show any run/stop controls.
             * This is needed because the showOperationControls does something similar but not exactly the same.
             * The showOperationControls property allows the gadgetBase to determine if the run control, if the gadget
             * uses it, to be displayed when the gadget has a valid configuration.
             *
             * Default: true - Gadgets without a need for run/stop control should override this value.
             * @type {boolean}
             */
            this.gadgetHasOperationControls = true;
            /**
             * Most gadgets need configuration so gadgets that don't can override this property
             * @type {boolean}
             */
            this.showConfigurationControl = true;
            // internally controls dynamic form properties
            this.propertyPages = [];
            this.errorExists = false;
            this._optionsService.listenForGlobalOptionsChanges().subscribe(function (options) {
                /**
                 * This is called when there is a change to the options tab within the configuration modal.
                 * The following method needs to also be called when the gadget is initially instantiated.
                 * When the gadget is instantiated the option values need to come from the persistent store.
                 */
                _this.updateGadgetWithGlobalOptions(options);
            });
            this.updateGadgetWithGlobalOptions(this._optionsService.getBoardOptions());
        }
        GadgetBase.prototype.ngOnInit = function () {
            this.toggleConfigMode();
            this.changeDetectionRef.detectChanges();
        };
        GadgetBase.prototype.ngAfterViewInit = function () {
            if (this.propertyPageForm) {
                if (this.propertyPageForm.isPropertyPageValid) {
                    this.toggleConfigMode();
                    this.changeDetectionRef.detectChanges();
                    this.showOperationControls = true;
                    this.changeDetectionRef.detectChanges();
                }
                else {
                    this.showOperationControls = false;
                    this.changeDetectionRef.detectChanges();
                }
            }
            this.preRun();
        };
        GadgetBase.prototype.initializeState = function () {
        };
        GadgetBase.prototype.toggleConfigMode = function () {
            if (!this.inConfig) {
                this.initializeProperties();
            }
            this.inConfig = !this.inConfig;
        };
        GadgetBase.prototype.initializeProperties = function () {
            if (this.propertyPages.length === 0 && this.config.propertyPages) {
                this._propertyService.setPropertyPagesAndProperties(this.config.propertyPages, this.propertyPages);
            }
        };
        GadgetBase.prototype.handleError = function (error) {
            this.inRun = false;
            this.actionInitiated = false;
            this.errorExists = true;
            this.errorObject = error;
        };
        GadgetBase.prototype.initializeRunState = function (forceRunState) {
            this.errorExists = false;
            this.actionInitiated = true;
            this.inConfig = false;
            if (forceRunState) {
                this.setInRunState();
            }
        };
        GadgetBase.prototype.setInRunState = function () {
            this.inRun = true;
            this.actionInitiated = false;
        };
        GadgetBase.prototype.setStopState = function (longRunningStopAction) {
            /**
             *  If the gadget indicates longRunningStopAction then the gadget has to set this value to
             *  false once the operation is complete
             */
            this.actionInitiated = longRunningStopAction;
            this.inRun = false;
        };
        GadgetBase.prototype.remove = function () {
            this._gadgetInstanceService.removeInstance(this.instanceId);
        };
        GadgetBase.prototype.showGadgetControls = function (enable) {
            this.showControls = enable;
        };
        /**
         * called from cell.component after the gadget is created during runtime
         * intanceId, config, title and endpoint are common to all gadgets. Once the gadgets are configured
         * we give them an opportunity to perform an action during the preRun() method. For example,
         * the statistic gadget uses preRun() to make a single call to the endpoint to update its display.
         * */
        GadgetBase.prototype.configureGadget = function (instanceId, config, tags) {
            this.instanceId = instanceId;
            this.config = config;
            this.gadgetTags = tags.slice();
            this.setTitle(this.getPropFromPropertyPages('title'));
            this.setEndPoint(this.getPropFromPropertyPages('endpoint'));
            /**
             *  Todo - remove this prerun call and refactor remaining code. Prerun was called twice and had an impact on the barchart api calls.
             *  API calls continued after route changes which is undesirable. See ngAfterViewInit where it is also called from.
             */
            //this.preRun();
        };
        GadgetBase.prototype.ngOnDestroy = function () {
        };
        GadgetBase.prototype.updateGadgetWithGlobalOptions = function (options) {
            this.globalOptions = Object.assign({}, options);
        };
        GadgetBase.prototype.setEndPoint = function (endpoint) {
            /*this._endPointService.getEndPoints().subscribe(data => {

                if (data['endPoint']) {
                    data['endPoint'].forEach(item => {
                        if (item.name === endpoint) {
                            this.endpointObject = item;
                        }
                    });
                }
            });*/
        };
        GadgetBase.prototype.getEndPoint = function () {
            return this.endpointObject;
        };
        GadgetBase.prototype.setTitle = function (title) {
            this.title = title;
        };
        GadgetBase.prototype.getPropFromPropertyPages = function (prop) {
            for (var x = 0; x < this.config.propertyPages.length; x++) {
                for (var i = 0; i < this.config.propertyPages[x].properties.length; i++) {
                    if (this.config.propertyPages[x].properties[i].key === prop) {
                        return this.config.propertyPages[x].properties[i].value;
                    }
                }
            }
            return 'Unknown';
        };
        return GadgetBase;
    }());
    GadgetBase.propDecorators = {
        propertyPageForm: [{ type: i0.ViewChild, args: [DynamicFormComponent, { static: true },] }]
    };

    var ErrorHandlerComponent = /** @class */ (function () {
        function ErrorHandlerComponent() {
        }
        ErrorHandlerComponent.prototype.closeMessage = function () {
            this.errorExists = false;
        };
        return ErrorHandlerComponent;
    }());
    ErrorHandlerComponent.decorators = [
        { type: i0.Component, args: [{
                    moduleId: module.id,
                    selector: 'dashboard-error-handler',
                    template: "<div *ngIf=\"errorExists\" [@error]=\"errorExists\" class=\"ui bottom attached negative message\">\n    <i (click)=\"closeMessage()\" class=\"close icon\"></i>\n    <div class=\"header\">\n        {{errorObject.summary}}\n    </div>\n    <p>\n        {{errorObject.detail}}\n    </p>\n\n    <p>\n        resource: {{errorObject.resource}}\n    </p>\n\n    <span class=\"error-heading\"> Potential solutions</span>\n    <hr style=\"color:rgba(194,197,200,0.37)\">\n    <div *ngFor=\"let solution of errorObject.solutions\" style=\"text-align: left !important\">\n        <br><span class=\"error-heading\">article:&nbsp;</span> {{solution.link}}\n        <br><span class=\"error-heading\">summary:&nbsp;</span> {{solution.summary}}\n    </div>\n</div>\n",
                    animations: [
                        animations.trigger('error', [
                            animations.transition(':enter', [
                                animations.style({ opacity: 0 }),
                                animations.animate('1000ms', animations.style({ opacity: 1 }))
                            ]),
                            animations.transition(':leave', [
                                animations.style({ opacity: 1 }),
                                animations.animate('1000ms', animations.style({ opacity: 0 }))
                            ])
                        ])
                    ],
                    styles: [".error-heading{color:grey!important;text-align:left}"]
                },] }
    ];
    ErrorHandlerComponent.ctorParameters = function () { return []; };
    ErrorHandlerComponent.propDecorators = {
        errorObject: [{ type: i0.Input }],
        errorExists: [{ type: i0.Input }]
    };

    var ErrorHandlerModule = /** @class */ (function () {
        function ErrorHandlerModule() {
        }
        return ErrorHandlerModule;
    }());
    ErrorHandlerModule.decorators = [
        { type: i0.NgModule, args: [{
                    imports: [
                        common.CommonModule
                    ],
                    declarations: [ErrorHandlerComponent],
                    exports: [ErrorHandlerComponent]
                },] }
    ];

    /*
     * Public API Surface of my-lib
     */

    /**
     * Generated bundle index. Do not edit.
     */

    exports.AboutComponent = AboutComponent;
    exports.AboutModule = AboutModule;
    exports.AboutService = AboutService;
    exports.AddGadgetComponent = AddGadgetComponent;
    exports.AddGadgetModule = AddGadgetModule;
    exports.AddGadgetService = AddGadgetService;
    exports.BoardComponent = BoardComponent;
    exports.BoardLayoutManagerComponent = BoardLayoutManagerComponent;
    exports.BoardModule = BoardModule;
    exports.BoardsConfigurationTabComponent = BoardsConfigurationTabComponent;
    exports.CapitalizeFirstPipe = CapitalizeFirstPipe;
    exports.CellComponent = CellComponent;
    exports.ConfigurationComponent = ConfigurationComponent;
    exports.ConfigurationModule = ConfigurationModule;
    exports.DetailComponent = DetailComponent;
    exports.DetailModule = DetailModule;
    exports.DetailService = DetailService;
    exports.DynamicFormComponent = DynamicFormComponent;
    exports.DynamicFormModule = DynamicFormModule;
    exports.DynamicFormPropertyComponent = DynamicFormPropertyComponent;
    exports.ErrorHandlerComponent = ErrorHandlerComponent;
    exports.ErrorHandlerModule = ErrorHandlerModule;
    exports.FacetComponent = FacetComponent;
    exports.FacetModule = FacetModule;
    exports.FacetTagProcessor = FacetTagProcessor;
    exports.FilterListComponent = FilterListComponent;
    exports.FilterPipe = FilterPipe;
    exports.FilterTagComponent = FilterTagComponent;
    exports.GadgetBase = GadgetBase;
    exports.GadgetConfigModel = GadgetConfigModel;
    exports.GadgetFactory = GadgetFactory;
    exports.GadgetHeaderComponent = GadgetHeaderComponent;
    exports.GadgetInstanceService = GadgetInstanceService;
    exports.GadgetOperationComponent = GadgetOperationComponent;
    exports.GadgetPropertyService = GadgetPropertyService;
    exports.GadgetSharedModule = GadgetSharedModule;
    exports.GridComponent = GridComponent;
    exports.GridModule = GridModule;
    exports.HelpModalComponent = HelpModalComponent;
    exports.LayoutModule = LayoutModule;
    exports.MenuComponent = MenuComponent;
    exports.MenuEventService = MenuEventService;
    exports.MenuModule = MenuModule;
    exports.NotificationComponent = NotificationComponent;
    exports.NotificationDetailComponent = NotificationDetailComponent;
    exports.NotificationService = NotificationService;
    exports.OptionsConfigurationTabComponent = OptionsConfigurationTabComponent;
    exports.OptionsService = OptionsService;
    exports.RuntimeService = RuntimeService;
    exports.ToastComponent = ToastComponent;
    exports.ToastModule = ToastModule;
    exports.TypeAheadInputComponent = TypeAheadInputComponent;
    exports.TypeAheadInputModule = TypeAheadInputModule;
    exports.VisDrillDownComponent = VisDrillDownComponent;
    exports.ɵa = ToastService;
    exports.ɵb = ConfigurationService;
    exports.ɵc = NotificationModule;
    exports.ɵd = DataListModule;
    exports.ɵe = DataListComponent;
    exports.ɵf = ObservableWebSocketService;
    exports.ɵg = ReversePipe;
    exports.ɵh = PropertyControlService;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=ngx-dynamic-dashboard.umd.js.map
