import { Injectable, Component, EventEmitter, Output, Input, ViewChild, NgModule, ɵɵdefineInjectable, ɵɵinject, Pipe, ElementRef, ContentChild, TemplateRef, ComponentFactoryResolver, ViewContainerRef, ANALYZE_FOR_ENTRY_COMPONENTS, ChangeDetectorRef } from '@angular/core';
import { Subject, Observable, EMPTY, throwError, timer } from 'rxjs';
import { HttpHeaders, HttpClient, HttpClientModule, HttpParams } from '@angular/common/http';
import { trigger, state, style, transition, animate } from '@angular/animations';
import { CommonModule } from '@angular/common';
import { FormsModule, ReactiveFormsModule, FormControl, Validators, FormGroup } from '@angular/forms';
import { MatSlideToggleModule } from '@angular/material/slide-toggle';
import { MatChipsModule } from '@angular/material/chips';
import { MatButtonModule } from '@angular/material/button';
import { MatIconModule } from '@angular/material/icon';
import { MatCheckboxModule } from '@angular/material/checkbox';
import { MatSelectModule } from '@angular/material/select';
import { MatOptionModule } from '@angular/material/core';
import { MatInputModule } from '@angular/material/input';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { catchError } from 'rxjs/operators';
import { filter, reverse } from 'lodash';
import { DragDropModule } from '@angular/cdk/drag-drop';
import { MatProgressBarModule } from '@angular/material/progress-bar';
import { NavigationEnd, Router, ActivatedRoute } from '@angular/router';

class OptionsService {
    constructor() {
        this.optionsCollectionName = 'dashboardOptions';
        this.defaultOptions = {
            'enableHover': false,
            'displayGadgetOptionsInSideBar': false
        };
        this.globalOptionsChangeEventSubject = new Subject();
    }
    getBoardOptions() {
        let databaseOptions = JSON.parse(localStorage.getItem(this.optionsCollectionName));
        if (databaseOptions == null) {
            this.persistDefautBoardOptions();
            return this.defaultOptions;
        }
        else {
            return databaseOptions;
        }
    }
    setBoardOptions(options) {
        /**
         * Todo this will need to change to support the update to individual options. Currently there is only one
         * option but once there is more than one this method must change to take the input and update just that
         * property of the options object.
         */
        localStorage.removeItem(this.optionsCollectionName);
        /**
         *  Raise an event to listeners, primarily the gadgets, when the global options change. The listeners can use
         * the event to change their behavior
         */
        this.globalOptionsChangeEventSubject.next(options);
        return localStorage.setItem(this.optionsCollectionName, JSON.stringify(options));
    }
    /**
     * The gadget-base can use this method to subscribe to events that are created when the global options change.
     */
    listenForGlobalOptionsChanges() {
        return this.globalOptionsChangeEventSubject.asObservable();
    }
    persistDefautBoardOptions() {
        localStorage.setItem(this.optionsCollectionName, JSON.stringify(this.defaultOptions));
    }
}
OptionsService.decorators = [
    { type: Injectable }
];
OptionsService.ctorParameters = () => [];

class Message {
    constructor(content, id, style) {
        this.dismissed = false;
        this.content = content;
        this.style = style || 'info';
        this.id = id;
    }
}

class ToastService {
    constructor() {
        this.messages = [];
    }
    getMessages() {
        return this.messages;
    }
    sendMessage(content, style) {
        const message = new Message(content, this.messages.length + 1, style);
        this.messages.push(message);
    }
    dismissMessage(messageKey) {
        this.purgeDismissedMessages();
        this.messages.forEach(message => {
            if (message.id === messageKey) {
                message.dismissed = true;
            }
        });
    }
    purgeDismissedMessages() {
        this.messages.forEach(function (message, index, object) {
            if (message.dismissed) {
                object.splice(index, 1);
            }
        });
    }
}
ToastService.decorators = [
    { type: Injectable }
];
ToastService.ctorParameters = () => [];

/**
 * Created by jayhamilton on 1/24/17.
 */
class OptionsConfigurationTabComponent {
    constructor(_optionsService, _toastService) {
        this._optionsService = _optionsService;
        this._toastService = _toastService;
        this.enableHover = this._optionsService.getBoardOptions()['enableHover'];
        this.displayGadgetOptionsInSideBar = this._optionsService.getBoardOptions()['displayGadgetOptionsInSideBar'];
    }
    onHooverOptionChange(value) {
        this._optionsService.setBoardOptions({
            'enableHover': value['checked'],
            'displayGadgetOptionsInSideBar': this.displayGadgetOptionsInSideBar
        });
        this._toastService.sendMessage('The board configuration has changed!', null);
    }
    onDisplayGadgetOptionsInSideBarChange(value) {
        this._optionsService.setBoardOptions({
            'enableHover': this.enableHover,
            'displayGadgetOptionsInSideBar': value['checked']
        });
        this._toastService.sendMessage('The board configuration has changed!', null);
    }
}
OptionsConfigurationTabComponent.decorators = [
    { type: Component, args: [{
                selector: 'dashboard-options-config-tab',
                moduleId: module.id,
                template: "<br>\nThe following options control various aspects of the board's and gadget's behavior. Changes take effect immediately.\n<br>\n<br>\n\n<mat-slide-toggle\n    (change)=\"onHooverOptionChange($event)\"\n    [checked]=\"enableHover\">\n    Always show the gadget buttons in the title. By default buttons show/hide when you hover over the title.\n</mat-slide-toggle>\n<br>\n<br>\n<mat-slide-toggle\n    (change)=\"onDisplayGadgetOptionsInSideBarChange($event)\"\n    [checked]=\"displayGadgetOptionsInSideBar\">\n    (Experimental) When set the options are displayed in the side bar. Otherwise it is displayed within the gadget.\n</mat-slide-toggle>\n",
                styles: [""]
            },] }
];
OptionsConfigurationTabComponent.ctorParameters = () => [
    { type: OptionsService },
    { type: ToastService }
];

const defaultBoard = {
    title: 'Dashboard',
    structure: '6-6',
    id: 5,
    boardInstanceId: 0,
    rows: [
        {
            columns: [
                {
                    styleClass: 'six wide',
                    gadgets: []
                },
                {
                    styleClass: 'six wide',
                    gadgets: []
                }
            ]
        }
    ]
};

const sampleBoardCollection = {
    'board': [{
            'title': 'Board Sample 1',
            'structure': '3-6-3',
            'id': 9,
            'boardInstanceId': 1,
            'rows': [
                {
                    'columns': [
                        {
                            'styleClass': 'three wide',
                            'gadgets': [
                                {
                                    'componentType': 'NewsGadgetComponent',
                                    'name': 'News',
                                    'description': 'What\'s new',
                                    'icon': 'images/news.png',
                                    'instanceId': 1500253814523,
                                    'tags': [
                                        {
                                            'facet': 'Informational',
                                            'name': 'news'
                                        },
                                        {
                                            'facet': 'List',
                                            'name': 'news'
                                        }
                                    ],
                                    'config': {
                                        'propertyPages': [
                                            {
                                                'displayName': 'Run',
                                                'groupId': 'run',
                                                'position': 10,
                                                'properties': [
                                                    {
                                                        'value': 'news',
                                                        'key': 'endpoint',
                                                        'label': 'News URL',
                                                        'required': false,
                                                        'order': 3,
                                                        'controlType': 'dynamicdropdown'
                                                    },
                                                    {
                                                        'value': 'News',
                                                        'key': 'title',
                                                        'label': 'Title',
                                                        'required': false,
                                                        'order': 1,
                                                        'controlType': 'textbox'
                                                    },
                                                    {
                                                        'value': 2,
                                                        'key': 'instanceId',
                                                        'required': false,
                                                        'order': -1,
                                                        'controlType': 'hidden'
                                                    }
                                                ]
                                            }
                                        ]
                                    }
                                }
                            ]
                        },
                        {
                            'styleClass': 'six wide',
                            'gadgets': [
                                {
                                    'componentType': 'CPUGadgetComponent',
                                    'name': 'CPU Chart',
                                    'description': 'Monitors CPU utilization for CDM.',
                                    'icon': 'images/cpu.png',
                                    'instanceId': 1499912922910,
                                    'tags': [
                                        {
                                            'facet': 'Performance',
                                            'name': 'real-time'
                                        },
                                        {
                                            'facet': 'Chart',
                                            'name': 'bar'
                                        }
                                    ],
                                    'config': {
                                        'propertyPages': [
                                            {
                                                'displayName': 'Run',
                                                'groupId': 'run',
                                                'position': 10,
                                                'properties': [
                                                    {
                                                        'value': 'CPU Utilization',
                                                        'key': 'title',
                                                        'label': 'Title',
                                                        'required': false,
                                                        'order': 1,
                                                        'controlType': 'textbox'
                                                    },
                                                    {
                                                        'value': 'Carlosappliance - Process Monitor',
                                                        'key': 'endpoint',
                                                        'label': 'API Endpoints',
                                                        'required': false,
                                                        'order': 3,
                                                        'controlType': 'dynamicdropdown'
                                                    },
                                                    {
                                                        'value': 999,
                                                        'key': 'instanceId',
                                                        'required': false,
                                                        'order': -1,
                                                        'controlType': 'hidden'
                                                    }
                                                ]
                                            },
                                            {
                                                'displayName': 'Chart',
                                                'groupId': 'chart',
                                                'position': 11,
                                                'properties': [
                                                    {
                                                        'value': true,
                                                        'key': 'chart_properties',
                                                        'label': 'Show chart details',
                                                        'required': false,
                                                        'order': 3,
                                                        'controlType': 'checkbox'
                                                    }
                                                ]
                                            }
                                        ]
                                    }
                                },
                                {
                                    'componentType': 'TrendGadgetComponent',
                                    'name': 'Trend',
                                    'description': 'General trends.',
                                    'icon': 'images/trend.png',
                                    'instanceId': 1499912901569,
                                    'tags': [
                                        {
                                            'facet': 'Performance',
                                            'name': 'trend'
                                        },
                                        {
                                            'facet': 'Chart',
                                            'name': 'area'
                                        }
                                    ],
                                    'config': {
                                        'propertyPages': [
                                            {
                                                'displayName': 'Run',
                                                'groupId': 'run',
                                                'position': 10,
                                                'properties': [
                                                    {
                                                        'value': 'Devappliance - ECX',
                                                        'key': 'endpoint',
                                                        'label': 'API Endpoints',
                                                        'required': false,
                                                        'order': 2,
                                                        'controlType': 'dynamicdropdown'
                                                    },
                                                    {
                                                        'value': 'Trend',
                                                        'key': 'title',
                                                        'label': 'Title',
                                                        'required': false,
                                                        'order': 1,
                                                        'controlType': 'textbox'
                                                    },
                                                    {
                                                        'value': 2,
                                                        'key': 'instanceId',
                                                        'required': false,
                                                        'order': -1,
                                                        'controlType': 'hidden'
                                                    }
                                                ]
                                            },
                                            {
                                                'displayName': 'Chart',
                                                'groupId': 'chart',
                                                'position': 11,
                                                'properties': [
                                                    {
                                                        'value': true,
                                                        'key': 'chart_properties',
                                                        'label': 'Show chart details',
                                                        'required': false,
                                                        'order': 3,
                                                        'controlType': 'checkbox'
                                                    }
                                                ]
                                            }
                                        ]
                                    }
                                }
                            ]
                        },
                        {
                            'styleClass': 'three wide',
                            'gadgets': []
                        }
                    ]
                }
            ]
        }, {
            'title': 'Board Sample 2',
            'structure': '4-4-4-4/8-4-4',
            'id': 8,
            'boardInstanceId': 2,
            'rows': [
                {
                    'columns': [
                        {
                            'styleClass': 'four wide',
                            'gadgets': [
                                {
                                    'componentType': 'StatisticGadgetComponent',
                                    'name': 'Statistic',
                                    'description': 'General statistic.',
                                    'icon': 'images/statistic.png',
                                    'instanceId': 1499912861630,
                                    'tags': [
                                        {
                                            'facet': 'Statistic',
                                            'name': 'counter'
                                        }
                                    ],
                                    'config': {
                                        'propertyPages': [
                                            {
                                                'displayName': 'Run',
                                                'groupId': 'run',
                                                'position': 10,
                                                'properties': [
                                                    {
                                                        'value': 'database',
                                                        'key': 'resource',
                                                        'label': 'Resource',
                                                        'required': false,
                                                        'order': 2,
                                                        'controlType': 'dropdown',
                                                        'options': [
                                                            {
                                                                'key': 'vm',
                                                                'value': 'vm'
                                                            },
                                                            {
                                                                'key': 'database',
                                                                'value': 'database'
                                                            },
                                                            {
                                                                'key': 'volume',
                                                                'value': 'volume'
                                                            },
                                                            {
                                                                'key': 'job',
                                                                'value': 'job'
                                                            }
                                                        ]
                                                    },
                                                    {
                                                        'value': 'Devappliance - ECX',
                                                        'key': 'endpoint',
                                                        'label': 'API Endpoints',
                                                        'required': false,
                                                        'order': 3,
                                                        'controlType': 'dynamicdropdown'
                                                    },
                                                    {
                                                        'value': 'Statistic',
                                                        'key': 'title',
                                                        'label': 'Title',
                                                        'required': false,
                                                        'order': 1,
                                                        'controlType': 'textbox'
                                                    },
                                                    {
                                                        'value': 2,
                                                        'key': 'instanceId',
                                                        'required': true,
                                                        'order': -1,
                                                        'controlType': 'hidden'
                                                    }
                                                ]
                                            }
                                        ]
                                    }
                                }
                            ]
                        },
                        {
                            'styleClass': 'four wide',
                            'gadgets': [
                                {
                                    'componentType': 'StatisticGadgetComponent',
                                    'name': 'Statistic',
                                    'description': 'General statistic.',
                                    'icon': 'images/statistic.png',
                                    'instanceId': 1499912845375,
                                    'tags': [
                                        {
                                            'facet': 'Statistic',
                                            'name': 'counter'
                                        }
                                    ],
                                    'config': {
                                        'propertyPages': [
                                            {
                                                'displayName': 'Run',
                                                'groupId': 'run',
                                                'position': 10,
                                                'properties': [
                                                    {
                                                        'value': 'vm',
                                                        'key': 'resource',
                                                        'label': 'Resource',
                                                        'required': false,
                                                        'order': 2,
                                                        'controlType': 'dropdown',
                                                        'options': [
                                                            {
                                                                'key': 'vm',
                                                                'value': 'vm'
                                                            },
                                                            {
                                                                'key': 'database',
                                                                'value': 'database'
                                                            },
                                                            {
                                                                'key': 'volume',
                                                                'value': 'volume'
                                                            },
                                                            {
                                                                'key': 'job',
                                                                'value': 'job'
                                                            }
                                                        ]
                                                    },
                                                    {
                                                        'value': 'Carlosappliance - Process Monitor',
                                                        'key': 'endpoint',
                                                        'label': 'API Endpoints',
                                                        'required': false,
                                                        'order': 3,
                                                        'controlType': 'dynamicdropdown'
                                                    },
                                                    {
                                                        'value': 'Statistic',
                                                        'key': 'title',
                                                        'label': 'Title',
                                                        'required': false,
                                                        'order': 1,
                                                        'controlType': 'textbox'
                                                    },
                                                    {
                                                        'value': 2,
                                                        'key': 'instanceId',
                                                        'required': true,
                                                        'order': -1,
                                                        'controlType': 'hidden'
                                                    }
                                                ]
                                            }
                                        ]
                                    }
                                }
                            ]
                        },
                        {
                            'styleClass': 'four wide',
                            'gadgets': [
                                {
                                    'componentType': 'StatisticGadgetComponent',
                                    'name': 'Statistic',
                                    'description': 'General statistic.',
                                    'icon': 'images/statistic.png',
                                    'instanceId': 1500251223835,
                                    'tags': [
                                        {
                                            'facet': 'Statistic',
                                            'name': 'counter'
                                        }
                                    ],
                                    'config': {
                                        'propertyPages': [
                                            {
                                                'displayName': 'Run',
                                                'groupId': 'run',
                                                'position': 10,
                                                'properties': [
                                                    {
                                                        'value': 'volume',
                                                        'key': 'resource',
                                                        'label': 'Resource',
                                                        'required': false,
                                                        'order': 2,
                                                        'controlType': 'dropdown',
                                                        'options': [
                                                            {
                                                                'key': 'vm',
                                                                'value': 'vm'
                                                            },
                                                            {
                                                                'key': 'database',
                                                                'value': 'database'
                                                            },
                                                            {
                                                                'key': 'volume',
                                                                'value': 'volume'
                                                            },
                                                            {
                                                                'key': 'job',
                                                                'value': 'job'
                                                            }
                                                        ]
                                                    },
                                                    {
                                                        'value': 'Carlosappliance - Process Monitor',
                                                        'key': 'endpoint',
                                                        'label': 'API Endpoints',
                                                        'required': false,
                                                        'order': 3,
                                                        'controlType': 'dynamicdropdown'
                                                    },
                                                    {
                                                        'value': 'Statistic',
                                                        'key': 'title',
                                                        'label': 'Title',
                                                        'required': false,
                                                        'order': 1,
                                                        'controlType': 'textbox'
                                                    },
                                                    {
                                                        'value': 2,
                                                        'key': 'instanceId',
                                                        'required': true,
                                                        'order': -1,
                                                        'controlType': 'hidden'
                                                    }
                                                ]
                                            }
                                        ]
                                    }
                                }
                            ]
                        },
                        {
                            'styleClass': 'four wide',
                            'gadgets': [
                                {
                                    'componentType': 'StatisticGadgetComponent',
                                    'name': 'Statistic',
                                    'description': 'General statistic.',
                                    'icon': 'images/statistic.png',
                                    'instanceId': 1500251241829,
                                    'tags': [
                                        {
                                            'facet': 'Statistic',
                                            'name': 'counter'
                                        }
                                    ],
                                    'config': {
                                        'propertyPages': [
                                            {
                                                'displayName': 'Run',
                                                'groupId': 'run',
                                                'position': 10,
                                                'properties': [
                                                    {
                                                        'value': 'job',
                                                        'key': 'resource',
                                                        'label': 'Resource',
                                                        'required': false,
                                                        'order': 2,
                                                        'controlType': 'dropdown',
                                                        'options': [
                                                            {
                                                                'key': 'vm',
                                                                'value': 'vm'
                                                            },
                                                            {
                                                                'key': 'database',
                                                                'value': 'database'
                                                            },
                                                            {
                                                                'key': 'volume',
                                                                'value': 'volume'
                                                            },
                                                            {
                                                                'key': 'job',
                                                                'value': 'job'
                                                            }
                                                        ]
                                                    },
                                                    {
                                                        'value': 'Carlosappliance - Process Monitor',
                                                        'key': 'endpoint',
                                                        'label': 'API Endpoints',
                                                        'required': false,
                                                        'order': 3,
                                                        'controlType': 'dynamicdropdown'
                                                    },
                                                    {
                                                        'value': 'Statistic',
                                                        'key': 'title',
                                                        'label': 'Title',
                                                        'required': false,
                                                        'order': 1,
                                                        'controlType': 'textbox'
                                                    },
                                                    {
                                                        'value': 2,
                                                        'key': 'instanceId',
                                                        'required': true,
                                                        'order': -1,
                                                        'controlType': 'hidden'
                                                    }
                                                ]
                                            }
                                        ]
                                    }
                                }
                            ]
                        }
                    ]
                },
                {
                    'columns': [
                        {
                            'styleClass': 'eight wide',
                            'gadgets': [
                                {
                                    'componentType': 'TrendLineGadgetComponent',
                                    'name': 'Realtime Performance',
                                    'description': 'IOPs and Network Bandwidth.',
                                    'icon': 'images/trend-line.png',
                                    'instanceId': 1500251340757,
                                    'tags': [
                                        {
                                            'facet': 'Performance',
                                            'name': 'trend'
                                        },
                                        {
                                            'facet': 'Chart',
                                            'name': 'line'
                                        }
                                    ],
                                    'config': {
                                        'propertyPages': [
                                            {
                                                'displayName': 'Run',
                                                'groupId': 'run',
                                                'position': 10,
                                                'properties': [
                                                    {
                                                        'value': 'Devappliance - ECX',
                                                        'key': 'endpoint',
                                                        'label': 'API Endpoints',
                                                        'required': false,
                                                        'order': 2,
                                                        'controlType': 'dynamicdropdown'
                                                    },
                                                    {
                                                        'value': 'Performance',
                                                        'key': 'title',
                                                        'label': 'Title',
                                                        'required': false,
                                                        'order': 1,
                                                        'controlType': 'textbox'
                                                    },
                                                    {
                                                        'value': 2,
                                                        'key': 'instanceId',
                                                        'required': false,
                                                        'order': -1,
                                                        'controlType': 'hidden'
                                                    }
                                                ]
                                            },
                                            {
                                                'displayName': 'Chart',
                                                'groupId': 'chart',
                                                'position': 11,
                                                'properties': [
                                                    {
                                                        'value': true,
                                                        'key': 'chart_properties',
                                                        'label': 'Show chart details',
                                                        'required': false,
                                                        'order': 3,
                                                        'controlType': 'checkbox'
                                                    }
                                                ]
                                            }
                                        ]
                                    }
                                },
                                {
                                    'componentType': 'ServiceListGadgetComponent',
                                    'name': 'Service Status',
                                    'description': 'Monitors Service Status',
                                    'icon': 'images/service.png',
                                    'instanceId': 1500343727872,
                                    'tags': [
                                        {
                                            'facet': 'List',
                                            'name': 'health'
                                        }
                                    ],
                                    'config': {
                                        'propertyPages': [
                                            {
                                                'displayName': 'Run',
                                                'groupId': 'run',
                                                'position': 10,
                                                'properties': [
                                                    {
                                                        'value': 'Devappliance - Process Monitor',
                                                        'key': 'endpoint',
                                                        'label': 'API Endpoints',
                                                        'required': false,
                                                        'order': 2,
                                                        'controlType': 'dynamicdropdown'
                                                    },
                                                    {
                                                        'value': 'Service Status',
                                                        'key': 'title',
                                                        'label': 'Title',
                                                        'required': false,
                                                        'order': 1,
                                                        'controlType': 'textbox'
                                                    },
                                                    {
                                                        'value': 999,
                                                        'key': 'instanceId',
                                                        'required': false,
                                                        'order': -1,
                                                        'controlType': 'hidden'
                                                    }
                                                ]
                                            },
                                            {
                                                'displayName': 'List',
                                                'groupId': 'chart',
                                                'position': 11,
                                                'properties': [
                                                    {
                                                        'value': true,
                                                        'key': 'chart_properties',
                                                        'label': 'Show chart details',
                                                        'required': false,
                                                        'order': 3,
                                                        'controlType': 'checkbox'
                                                    }
                                                ]
                                            }
                                        ]
                                    }
                                }
                            ]
                        },
                        {
                            'styleClass': 'four wide',
                            'gadgets': [
                                {
                                    'componentType': 'DiskGadgetComponent',
                                    'name': 'Disk Utilization',
                                    'description': 'Disk consumption information for ECX.',
                                    'icon': 'images/donut.png',
                                    'instanceId': 1500253741130,
                                    'tags': [
                                        {
                                            'facet': 'Chart',
                                            'name': 'pie'
                                        },
                                        {
                                            'facet': 'Performance',
                                            'name': 'storage'
                                        }
                                    ],
                                    'config': {
                                        'propertyPages': [
                                            {
                                                'displayName': 'Run',
                                                'groupId': 'run',
                                                'position': 10,
                                                'properties': [
                                                    {
                                                        'value': 'Carlosappliance - Process Monitor',
                                                        'key': 'endpoint',
                                                        'label': 'API Endpoints',
                                                        'required': false,
                                                        'order': 4,
                                                        'controlType': 'dynamicdropdown'
                                                    },
                                                    {
                                                        'value': 'Disk Consumption',
                                                        'key': 'title',
                                                        'label': 'Title',
                                                        'required': false,
                                                        'order': 1,
                                                        'controlType': 'textbox'
                                                    },
                                                    {
                                                        'value': 50,
                                                        'key': 'threshold',
                                                        'label': 'Consumed Threshold',
                                                        'required': false,
                                                        'order': 3,
                                                        'controlType': 'number'
                                                    },
                                                    {
                                                        'value': 'g3g3egegre',
                                                        'key': 'diskid',
                                                        'label': 'Disk Id',
                                                        'required': false,
                                                        'order': 2,
                                                        'controlType': 'dropdown',
                                                        'options': [
                                                            {
                                                                'key': '2344112gdfgdfg',
                                                                'value': '2344112gdfgdfg'
                                                            },
                                                            {
                                                                'key': 'g3g3egegre',
                                                                'value': 'g3g3egegre'
                                                            },
                                                            {
                                                                'key': '8435f34',
                                                                'value': '8435f34'
                                                            }
                                                        ]
                                                    },
                                                    {
                                                        'value': 2,
                                                        'key': 'instanceId',
                                                        'required': false,
                                                        'order': -1,
                                                        'controlType': 'hidden'
                                                    }
                                                ]
                                            },
                                            {
                                                'displayName': 'Thresholds',
                                                'groupId': 'threshold',
                                                'position': 10,
                                                'properties': [
                                                    {
                                                        'value': 30,
                                                        'key': 'frequency',
                                                        'label': 'Frequency',
                                                        'required': false,
                                                        'order': 3,
                                                        'controlType': 'number'
                                                    },
                                                    {
                                                        'value': 30,
                                                        'key': 'retentionH',
                                                        'label': 'Retention High',
                                                        'required': false,
                                                        'order': 4,
                                                        'controlType': 'number'
                                                    },
                                                    {
                                                        'value': 180,
                                                        'key': 'retentionL',
                                                        'label': 'Retention Low',
                                                        'required': false,
                                                        'order': 4,
                                                        'controlType': 'number'
                                                    }
                                                ]
                                            },
                                            {
                                                'displayName': 'Chart',
                                                'groupId': 'chart',
                                                'position': 11,
                                                'properties': [
                                                    {
                                                        'value': true,
                                                        'key': 'chart_properties',
                                                        'label': 'Show chart details',
                                                        'required': false,
                                                        'order': 3,
                                                        'controlType': 'checkbox'
                                                    }
                                                ]
                                            }
                                        ]
                                    }
                                }
                            ]
                        },
                        {
                            'styleClass': 'four wide',
                            'gadgets': [
                                {
                                    'componentType': 'DiskGadgetComponent',
                                    'name': 'Disk Utilization',
                                    'description': 'Disk consumption information for ECX.',
                                    'icon': 'images/donut.png',
                                    'instanceId': 1500252483066,
                                    'tags': [
                                        {
                                            'facet': 'Chart',
                                            'name': 'pie'
                                        },
                                        {
                                            'facet': 'Performance',
                                            'name': 'storage'
                                        }
                                    ],
                                    'config': {
                                        'propertyPages': [
                                            {
                                                'displayName': 'Run',
                                                'groupId': 'run',
                                                'position': 10,
                                                'properties': [
                                                    {
                                                        'value': 'Carlosappliance - Process Monitor',
                                                        'key': 'endpoint',
                                                        'label': 'API Endpoints',
                                                        'required': false,
                                                        'order': 4,
                                                        'controlType': 'dynamicdropdown'
                                                    },
                                                    {
                                                        'value': 'Disk Consumption',
                                                        'key': 'title',
                                                        'label': 'Title',
                                                        'required': false,
                                                        'order': 1,
                                                        'controlType': 'textbox'
                                                    },
                                                    {
                                                        'value': 50,
                                                        'key': 'threshold',
                                                        'label': 'Consumed Threshold',
                                                        'required': false,
                                                        'order': 3,
                                                        'controlType': 'number'
                                                    },
                                                    {
                                                        'value': 'g3g3egegre',
                                                        'key': 'diskid',
                                                        'label': 'Disk Id',
                                                        'required': false,
                                                        'order': 2,
                                                        'controlType': 'dropdown',
                                                        'options': [
                                                            {
                                                                'key': '2344112gdfgdfg',
                                                                'value': '2344112gdfgdfg'
                                                            },
                                                            {
                                                                'key': 'g3g3egegre',
                                                                'value': 'g3g3egegre'
                                                            },
                                                            {
                                                                'key': '8435f34',
                                                                'value': '8435f34'
                                                            }
                                                        ]
                                                    },
                                                    {
                                                        'value': 2,
                                                        'key': 'instanceId',
                                                        'required': false,
                                                        'order': -1,
                                                        'controlType': 'hidden'
                                                    }
                                                ]
                                            },
                                            {
                                                'displayName': 'Thresholds',
                                                'groupId': 'threshold',
                                                'position': 10,
                                                'properties': [
                                                    {
                                                        'value': 30,
                                                        'key': 'frequency',
                                                        'label': 'Frequency',
                                                        'required': false,
                                                        'order': 3,
                                                        'controlType': 'number'
                                                    },
                                                    {
                                                        'value': 30,
                                                        'key': 'retentionH',
                                                        'label': 'Retention High',
                                                        'required': false,
                                                        'order': 4,
                                                        'controlType': 'number'
                                                    },
                                                    {
                                                        'value': 180,
                                                        'key': 'retentionL',
                                                        'label': 'Retention Low',
                                                        'required': false,
                                                        'order': 4,
                                                        'controlType': 'number'
                                                    }
                                                ]
                                            },
                                            {
                                                'displayName': 'Chart',
                                                'groupId': 'chart',
                                                'position': 11,
                                                'properties': [
                                                    {
                                                        'value': true,
                                                        'key': 'chart_properties',
                                                        'label': 'Show chart details',
                                                        'required': false,
                                                        'order': 3,
                                                        'controlType': 'checkbox'
                                                    }
                                                ]
                                            }
                                        ]
                                    }
                                },
                                {
                                    'componentType': 'TrendGadgetComponent',
                                    'name': 'Trend',
                                    'description': 'General trends.',
                                    'icon': 'images/trend.png',
                                    'instanceId': 1500252536414,
                                    'tags': [
                                        {
                                            'facet': 'Performance',
                                            'name': 'trend'
                                        },
                                        {
                                            'facet': 'Chart',
                                            'name': 'area'
                                        }
                                    ],
                                    'config': {
                                        'propertyPages': [
                                            {
                                                'displayName': 'Run',
                                                'groupId': 'run',
                                                'position': 10,
                                                'properties': [
                                                    {
                                                        'value': 'Carlosappliance - Process Monitor',
                                                        'key': 'endpoint',
                                                        'label': 'API Endpoints',
                                                        'required': false,
                                                        'order': 2,
                                                        'controlType': 'dynamicdropdown'
                                                    },
                                                    {
                                                        'value': 'Trend',
                                                        'key': 'title',
                                                        'label': 'Title',
                                                        'required': false,
                                                        'order': 1,
                                                        'controlType': 'textbox'
                                                    },
                                                    {
                                                        'value': 2,
                                                        'key': 'instanceId',
                                                        'required': false,
                                                        'order': -1,
                                                        'controlType': 'hidden'
                                                    }
                                                ]
                                            },
                                            {
                                                'displayName': 'Chart',
                                                'groupId': 'chart',
                                                'position': 11,
                                                'properties': [
                                                    {
                                                        'value': true,
                                                        'key': 'chart_properties',
                                                        'label': 'Show chart details',
                                                        'required': false,
                                                        'order': 3,
                                                        'controlType': 'checkbox'
                                                    }
                                                ]
                                            }
                                        ]
                                    }
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    ]
};

// The file contents for the current environment will overwrite these during build.
// The build system defaults to the dev environment which uses `environment.ts`, but if you do
// `ng build --env=prod` then `environment.prod.ts` will be used instead.
// The list of which env maps to which file can be found in `.angular-cli.json`.
const environment = {
    production: false,
    productName: 'Dynamic Dashboard Framework',
    productVersion: '0.0.5',
    menu: {
        documentation: true,
        aiSearch: false,
        notification: true
    },
    boardConfiguration: {
        board: true,
        ai: false,
        endpoint: true
    }
};

/**
 * Created by jayhamilton on 2/7/17.
 */
class ConfigurationService {
    constructor(_http) {
        this._http = _http;
        this.demo = true;
        /**
         * todo - fix this hard coded store
         * @type {string}
         */
        this.remoteConfigurationRepository = '';
        this.defaultBoard = Object.assign({}, defaultBoard);
        this.sampleBoardCollection = Object.assign({}, sampleBoardCollection);
        this.env = environment;
        this.seedLocalStorageWithSampleBoardCollection();
    }
    getBoards() {
        if (this.demo) {
            return new Observable(observer => {
                let data = JSON.parse(localStorage.getItem('board'));
                if (!data) {
                    data = { board: [] };
                }
                observer.next(data.board);
                return () => {
                };
            });
        }
        else {
            /**
             * todo - this call is based on an internal representation (admin console) of something called a store.
             * That concept requires refactoring.
             */
            return this._http.get(this.remoteConfigurationRepository);
        }
    }
    saveBoard(board) {
        this.model = board;
        if (Object.keys(board).length === 0 && board.constructor === Object) {
            return EMPTY;
        }
        if (this.demo) {
            return new Observable(observer => {
                let board_collection;
                // find and remove board from storage
                this.deleteBoardFromLocalStore(board.title);
                // get a collection object and add board to it
                if ((board_collection = JSON.parse(localStorage.getItem('board'))) == null) {
                    board_collection = {
                        board: []
                    };
                }
                board_collection['board'].push(board);
                // save
                localStorage.setItem('board', JSON.stringify(board_collection));
                observer.next({});
                return () => {
                };
            });
        }
        else {
            /**
             * todo - a delete must happen here
             *
             */
            const httpOptions = {
                headers: new HttpHeaders({
                    'Content-Type': 'application/json'
                })
            };
            return this._http.post(this.remoteConfigurationRepository + '?id=' + board.title, JSON.stringify(board), httpOptions);
        }
    }
    deleteBoard(boardTitle) {
        if (this.demo) {
            return new Observable(observer => {
                this.deleteBoardFromLocalStore(boardTitle);
                observer.next({});
                return () => {
                };
            });
        }
        else {
            return this._http.delete(this.remoteConfigurationRepository + '/' + boardTitle);
        }
    }
    getDefaultBoard() {
        return new Observable(observer => {
            observer.next(this.defaultBoard);
            return () => {
            };
        });
    }
    /*
     when a gadget instance's property page is updated and saved, the change gets communicated to all
     gadgets. The gadget instance id that caused the change will update their current instance. todo - this might be able to be
     improved. For now the utility of this approach allows the configuration service to capture the property page change in a way
     that allows us to update the persisted board model.
     */
    notifyGadgetOnPropertyChange(gadgetConfig, instanceId) {
        this.savePropertyPageConfigurationToStore(gadgetConfig, instanceId);
    }
    setCurrentModel(_currentModel) {
        this.currentModel = _currentModel;
    }
    savePropertyPageConfigurationToStore(gadgetConfig, instanceId) {
        this.currentModel.rows.forEach(row => {
            row.columns.forEach(column => {
                if (column.gadgets) {
                    column.gadgets.forEach(gadget => {
                        this.updateProperties(gadgetConfig, gadget, instanceId);
                    });
                }
            });
        });
        this.saveBoard(this.currentModel).subscribe(result => {
            /**
             * todo - create popup/toast to show configuration saved message
             */
            console.debug('The following configuration model was saved!');
        }, error => console.error('Error' + error), () => console.debug('Saving configuration to store!'));
    }
    updateProperties(updatedProperties, gadget, instanceId) {
        const updatedPropsObject = JSON.parse(updatedProperties);
        if (gadget.instanceId === instanceId) {
            gadget.config.propertyPages.forEach(function (propertyPage) {
                for (let x = 0; x < propertyPage.properties.length; x++) {
                    for (const prop in updatedPropsObject) {
                        if (updatedPropsObject.hasOwnProperty(prop)) {
                            if (prop === propertyPage.properties[x].key) {
                                propertyPage.properties[x].value = updatedPropsObject[prop];
                            }
                        }
                    }
                }
            });
        }
    }
    seedLocalStorageWithSampleBoardCollection() {
        if (localStorage.getItem('board') === null) {
            if (!this.env.production) {
                localStorage.setItem('board', JSON.stringify(this.sampleBoardCollection));
            }
        }
    }
    delete(board_collection) {
        localStorage.removeItem('board');
        localStorage.setItem('board', JSON.stringify(board_collection));
    }
    deleteBoardFromLocalStore(boardTitle) {
        const board_collection = JSON.parse(localStorage.getItem('board'));
        let index;
        if (board_collection && (index = board_collection['board'].findIndex(item => {
            return item.title === boardTitle;
        })) >= 0) {
            board_collection['board'].splice(index, 1);
            this.delete(board_collection);
        }
    }
}
ConfigurationService.decorators = [
    { type: Injectable }
];
ConfigurationService.ctorParameters = () => [
    { type: HttpClient }
];

/**
 * Created by jayhamilton on 1/24/17.
 */
/**
 * Message Modal - clasable modal with message
 *
 * Selector message-modal
 *
 * Methods
 *      popMessageModal - display a message modal for a sepcified duration
 *      showMessageModal - show the message modal
 *      hideMessageModal - hide the message modal
 */
class BoardsConfigurationTabComponent {
    constructor(_configurationService) {
        this._configurationService = _configurationService;
        this.dashboardCreateEvent = new EventEmitter();
        this.dashboardEditEvent = new EventEmitter();
        this.dashboardDeleteEvent = new EventEmitter();
        this.newDashboardItem = '';
        this.env = environment;
    }
    createBoard(name) {
        if (name !== '') {
            this.dashboardCreateEvent.emit(name);
            this.newDashboardItem = '';
        }
    }
    editBoard(name) {
        this.dashboardEditEvent.emit(name);
    }
    deleteBoard(name) {
        this.dashboardDeleteEvent.emit(name);
    }
}
BoardsConfigurationTabComponent.decorators = [
    { type: Component, args: [{
                selector: 'dashboard-config-tab',
                moduleId: module.id,
                template: "<table class=\"ui padded table\">\n    <thead>\n    <th class=\"index-column\">#</th>\n    <th>name</th>\n    <th></th>\n    </thead>\n    <tr *ngFor=\"let board of dashboardList; let i = index\" class=\"ui basic segment\">\n        <td class=\"index-column\">{{i + 1}}</td>\n        <td>{{board}}</td>\n        <td style=\"text-align:right\">\n            <a (click)=\"deleteBoard(board)\" mat-mini-fab routerLink=\".\">\n                <mat-icon>clear</mat-icon>\n            </a>\n        </td>\n    </tr>\n</table>\n\n<div class=\"ui basic segment\">\n\n    <div class=\"grid-row\">\n        <mat-form-field class=\"example-full-width\">\n            <input [(ngModel)]=\"newDashboardItem\" matInput placeholder=\"Enter Dashboard Name\">\n        </mat-form-field>\n\n        <a (click)=\"createBoard(newDashboardItem)\" color='primary' mat-mini-fab routerLink=\".\">\n            <mat-icon>add</mat-icon>\n        </a>\n    </div>\n\n</div>\n",
                styles: ["input{font-family:Roboto,sans-serif!important;font-size:1.1em;font-weight:300!important;outline:none!important;padding:5px}label{color:#4a4d50!important;font-family:Roboto,sans-serif;font-size:1.3em!important}label.endpoint{font-size:1em!important;line-height:1.5em!important}td.detail{background-color:#fff!important}.index-column{background-color:#ddebf5!important;border:none!important;width:5%!important}.index-column,.ui.table thead th td{color:#788ca0!important}.ui.table{color:rgba(97,97,97,.68)!important}.inLine{display:inline-block;padding-left:4px;padding-right:4px}.example-full-width{width:100%}.grid-row{align-items:center;display:flex;justify-content:space-around}"]
            },] }
];
BoardsConfigurationTabComponent.ctorParameters = () => [
    { type: ConfigurationService }
];
BoardsConfigurationTabComponent.propDecorators = {
    dashboardCreateEvent: [{ type: Output }],
    dashboardEditEvent: [{ type: Output }],
    dashboardDeleteEvent: [{ type: Output }],
    dashboardList: [{ type: Input }]
};

const tabsModel = [
    {
        groupId: 'boards',
        displayName: 'Boards'
    },
    {
        groupId: 'options',
        displayName: 'Options'
    }
];

/**
 * Created by jayhamilton on 1/24/17.
 */
/**
 * Message Modal - clasable modal with message
 *
 * Selector message-modal
 *
 * Methods
 *      popMessageModal - display a message modal for a sepcified duration
 *      showMessageModal - show the message modal
 *      hideMessageModal - hide the message modal
 */
class ConfigurationComponent {
    constructor() {
        this.dashboardCreateEvent = new EventEmitter();
        this.dashboardEditEvent = new EventEmitter();
        this.dashboardDeleteEvent = new EventEmitter();
        this.newDashboardItem = '';
        Object.assign(this, { tabsModel });
        this.setCurrentTab(0);
        this.env = environment;
    }
    showConfigurationModal(header) {
        this.modalheader = header;
        this.configModal.modal('show');
    }
    hideMessageModal() {
        this.modalicon = '';
        this.modalheader = '';
        this.modalconfig = '';
        this.configModal.modal('hide');
    }
    createBoard(name) {
        if (name !== '') {
            this.dashboardCreateEvent.emit(name);
            this.newDashboardItem = '';
        }
        console.log('Creating new board event from configuration component: ' + name);
    }
    editBoard(name) {
        this.dashboardEditEvent.emit(name);
    }
    deleteBoard(name) {
        this.dashboardDeleteEvent.emit(name);
    }
    ngAfterViewInit() {
        this.configModal = jQuery(this.boardconfigmodalaRef.nativeElement);
        this.configModal.modal('hide');
    }
    setCurrentTab(tab_index) {
        this.currentTab = this.tabsModel[tab_index].displayName;
    }
}
ConfigurationComponent.decorators = [
    { type: Component, args: [{
                selector: 'dashboard-configuration-modal',
                moduleId: module.id,
                template: "<div #boardconfigmodal_tag class=\"ui long modal\">\n    <div class=\"header\">\n        <h2>{{modalheader}}</h2>\n    </div>\n\n\n    <div class=\"ui top attached tabular menu\">\n\n        <a (click)=\"setCurrentTab(i)\" *ngFor=\"let tab of tabsModel; let i = index\"\n           [ngClass]=\"{'active': tab.displayName == currentTab }\"\n           attr.data-tab=\"{{tab.groupId}}\" class=\"item\">{{tab.displayName}}</a>\n\n    </div>\n\n    <div *ngFor=\"let tab of tabsModel; let i = index\" [@contentSwitch]=\"tab.displayName == currentTab ? 'active':'inactive'\"\n         [ngClass]=\"{'active': tab.displayName == currentTab }\"\n         attr.data-tab=\"{{tab.groupId}}\" class=\"ui bottom attached tab segment\">\n\n\n        <div *ngIf=\"tab.groupId == 'boards'\">\n\n            <dashboard-config-tab (dashboardCreateEvent)=\"createBoard($event)\"\n                                  (dashboardDeleteEvent)=\"deleteBoard($event)\"\n                                  (dashboardEditEvent)=\"editBoard($event)\"\n                                  [dashboardList]=\"dashboardList\"></dashboard-config-tab>\n        </div>\n\n        <div *ngIf=\"tab.groupId == 'options'\">\n\n            <dashboard-options-config-tab></dashboard-options-config-tab>\n\n        </div>\n    </div>\n\n\n    <div class=\"actions\">\n        <button (click)=\"hideMessageModal()\" class=\"ui button\">Close</button>\n    </div>\n</div>\n",
                animations: [
                    trigger('contentSwitch', [
                        state('inactive', style({
                            opacity: 0
                        })),
                        state('active', style({
                            opacity: 1
                        })),
                        transition('inactive => active', animate('750ms ease-in')),
                        transition('active => inactive', animate('750ms ease-out'))
                    ])
                ],
                styles: [".ui.tabular.menu .active.item{border-top:3px solid #3f51b5!important}.ui.attached.segment{margin:0;width:100%}"]
            },] }
];
ConfigurationComponent.ctorParameters = () => [];
ConfigurationComponent.propDecorators = {
    dashboardCreateEvent: [{ type: Output }],
    dashboardEditEvent: [{ type: Output }],
    dashboardDeleteEvent: [{ type: Output }],
    dashboardList: [{ type: Input }],
    boardconfigmodalaRef: [{ type: ViewChild, args: ['boardconfigmodal_tag', { static: true },] }]
};

class ConfigurationModule {
}
ConfigurationModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    MatButtonModule,
                    MatIconModule,
                    MatCheckboxModule,
                    MatInputModule,
                    MatSelectModule,
                    MatOptionModule,
                    FormsModule,
                    ReactiveFormsModule,
                    MatSlideToggleModule,
                    MatChipsModule
                ],
                declarations: [
                    BoardsConfigurationTabComponent,
                    OptionsConfigurationTabComponent,
                    ConfigurationComponent
                ],
                providers: [
                    OptionsService
                ],
                exports: [
                    BoardsConfigurationTabComponent,
                    OptionsConfigurationTabComponent,
                    ConfigurationComponent
                ]
            },] }
];

/**
 The grid is primarily controlled by MenuService Events. You can change the behavior by
 changing the MenuComponent and have it call methods within the Grid component via Output Events
 within the Menu component or by using the @ViewChild approach.

 In the current code, the MenuService is shared by the GridComponent and MenuComponent.
 The MenuComponent relays events from the various components that make up the MenuComponent.
 The GridComponent listens for those events via an Observable. The GridComponent will also
 raise/emit an event that will be picked up by the MenuComponent via the MenuService through an Observable as well.
 */
class MenuEventService {
    constructor() {
        this.menuSubject = new Subject();
        this.gridSubject = new Subject();
        this.subscribers = [];
    }
    raiseMenuEvent(event) {
        this.menuSubject.next(event);
    }
    listenForMenuEvents() {
        return this.menuSubject.asObservable();
    }
    raiseGridEvent(event) {
        this.menuSubject.next(event);
    }
    listenForGridEvents() {
        return this.gridSubject.asObservable();
    }
    addSubscriber(subscriber) {
        this.subscribers.push(subscriber);
    }
    unSubscribeAll() {
        this.subscribers.forEach(subscription => {
            subscription.unsubscribe();
        });
        this.subscribers.length = 0;
    }
}
MenuEventService.decorators = [
    { type: Injectable }
];
MenuEventService.ctorParameters = () => [];

/**a
 * Menu component
 *
 */
class MenuComponent {
    constructor(_configurationService, _menuEventService) {
        this._configurationService = _configurationService;
        this._menuEventService = _menuEventService;
        this.host = window.location.host;
        this.dashboardList = [];
        this.selectedBoard = '';
        this.placeHolderText = 'Ask the board to do something!';
        this.searchList = [];
        this.show = false;
        this.typeAheadIsInMenu = true;
        this.layoutId = 0;
        this._menuEventService.unSubscribeAll();
        this.setupEventListeners();
        this.env = environment;
    }
    setupEventListeners() {
        let gridEventSubscription = this._menuEventService.listenForGridEvents().subscribe((event) => {
            const edata = event['data'];
            switch (event['name']) {
                case 'boardUpdateEvent':
                    this.updateDashboardMenu(edata);
                    break;
            }
        });
        this._menuEventService.addSubscriber(gridEventSubscription);
    }
    ngOnInit() {
        this.updateDashboardMenu('');
    }
    ngAfterViewInit() {
        this.stickyMenu = jQuery(this.stickyMenuRef.nativeElement);
        this.stickyMenu.sticky();
    }
    emitBoardChangeLayoutEvent(event) {
        this._menuEventService.raiseMenuEvent({ name: 'boardChangeLayoutEvent', data: event });
    }
    emitBoardSelectEvent(event) {
        this.boardSelect(event);
        this._menuEventService.raiseMenuEvent({ name: 'boardSelectEvent', data: event });
    }
    emitBoardCreateEvent(event) {
        this._menuEventService.raiseMenuEvent({ name: 'boardCreateEvent', data: event });
        this.updateDashboardMenu(event);
    }
    emitBoardEditEvent(event) {
        this._menuEventService.raiseMenuEvent({ name: 'boardEditEvent', data: event });
    }
    emitBoardDeleteEvent(event) {
        this._menuEventService.raiseMenuEvent({ name: 'boardDeleteEvent', data: event });
        this.updateDashboardMenu('');
    }
    emitBoardAddGadgetEvent(event) {
        this._menuEventService.raiseMenuEvent({ name: 'boardAddGadgetEvent', data: event });
    }
    emitBoardAIAddGadgetEvent(event) {
        this._menuEventService.raiseMenuEvent({ name: 'boardAIAddGadgetEvent', data: event });
    }
    updateDashboardMenu(selectedBoard) {
        this._configurationService.getBoards().subscribe(data => {
            const me = this;
            if (data && data instanceof Array && data.length) {
                this.dashboardList.length = 0;
                // sort boards
                data.sort((a, b) => a.boardInstanceId - b.boardInstanceId);
                data.forEach(board => {
                    me.dashboardList.push(board.title);
                });
                if (selectedBoard === '') {
                    this.boardSelect(this.dashboardList[0]);
                }
                else {
                    this.boardSelect(selectedBoard);
                }
            }
        });
    }
    boardSelect(selectedBoard) {
        this.selectedBoard = selectedBoard;
    }
    toggleLayoutSideBar() {
        this.layoutSideBar = jQuery(this.layoutSideBarRef.nativeElement);
        this.layoutSideBar.sidebar('setting', 'transition', 'overlay');
        this.layoutSideBar.sidebar('toggle');
        this.layoutId = this._configurationService.currentModel.id;
    }
    toggleNotificationSideBar() {
        this.notificationSideBar = jQuery(this.notificationSideBarRef.nativeElement);
        this.notificationSideBar.sidebar('setting', 'transition', 'overlay');
        this.notificationSideBar.sidebar('toggle');
    }
    toggleAboutSideBar() {
        this.aboutSideBar = jQuery(this.aboutSideBarRef.nativeElement);
        this.aboutSideBar.sidebar('setting', 'transition', 'overlay');
        this.aboutSideBar.sidebar('toggle');
    }
}
MenuComponent.decorators = [
    { type: Component, args: [{
                moduleId: module.id,
                selector: 'dashboard-menu',
                template: "<div #stickymenu_tag class=\"ui inverted menu sticky\" ng-show=\"show\" style=\"height: 66px !important\">\n\n\n    <dashboard-typeahead-input (ArtificialIntelligenceEventEmitter)=\"emitBoardAIAddGadgetEvent($event)\" *ngIf=\"env.menu.aiSearch\" [placeHolderText]=\"placeHolderText\"\n                               [searchList]=\"searchList\"\n                               [typeAheadIsInMenu]=\"typeAheadIsInMenu\"\n                               class=\"item\"\n                               style=\"width:25%; padding-bottom:0\">\n    </dashboard-typeahead-input>\n\n\n    <div class=\"ui secondary inverted pointing menu board\">\n        <a (click)=\"emitBoardSelectEvent(board)\" *ngFor=\"let board of dashboardList; let i = index\"\n           [ngClass]=\"{'active':board == selectedBoard}\"\n           class=\"item\">\n            {{board}}\n        </a>\n    </div>\n\n\n    <div class=\"right inverted menu\">\n        <a (click)=\"toggleNotificationSideBar()\" *ngIf=\"env.menu.notification\"\n           class=\"item\">\n            <i class=\"large inverted alarm icon\"></i>\n            <div class=\"floating ui red label\">\n                <span style=\"font-size: 1.2em; font-weight: normal\">4</span>\n            </div>\n        </a>\n\n\n        <a (click)=\"addGadgetModal.showComponentLibraryModal('Add Gadget')\"\n           class=\"item\">\n            <i class=\"large inverted plus icon\"></i>\n        </a>\n\n\n        <a (click)=\"toggleLayoutSideBar()\"\n           class=\"item\">\n            <i class=\"large inverted grid layout icon\"></i>\n        </a>\n\n\n        <a (click)=\"boardConfigModal.showConfigurationModal('Configuration')\"\n           class=\"item\">\n            <i class=\"large inverted setting icon\"></i>\n        </a>\n\n\n        <a (click)=\"toggleAboutSideBar()\"\n           class=\"item\">\n            <i class=\"large inverted info icon\"></i>\n        </a>\n\n    </div>\n\n</div>\n<br>\n\n<!-- the modal will emit an event and call the grid component to add the requested cell to the board -->\n<dashboard-add-gadget-modal #addGadgetModal\n                            (addGadgetEvent)=\"emitBoardAddGadgetEvent($event)\">\n</dashboard-add-gadget-modal>\n\n<dashboard-configuration-modal #boardConfigModal\n                               (dashboardCreateEvent)=\"emitBoardCreateEvent($event)\"\n                               (dashboardDeleteEvent)=\"emitBoardDeleteEvent($event)\"\n                               (dashboardEditEvent)=\"emitBoardEditEvent($event)\"\n                               [dashboardList]=\"dashboardList\">\n\n\n</dashboard-configuration-modal>\n\n\n<!-- side bar content -->\n<div #notificationSideBar_tag class=\"ui sidebar inverted right vertical menu\">\n    <dashboard-notification-modal></dashboard-notification-modal>\n</div>\n\n<!-- side bar content -->\n<div #aboutSideBar_tag class=\"ui sidebar inverted right vertical menu\">\n    <dashboard-about-modal></dashboard-about-modal>\n</div>\n\n\n<div #layoutSideBar_tag class=\"ui sidebar inverted right vertical menu\">\n    <dashboard-layout-manager-modal (boardLayoutChangeEvent)=\"emitBoardChangeLayoutEvent($event)\"\n                                    [layoutId]=\"layoutId\">\n    </dashboard-layout-manager-modal>\n</div>\n\n",
                styles: [".ui.secondary.pointing.menu .active.item{border-color:#0f0!important;border-width:medium!important;color:#f9f9f9!important}.ui.secondary.pointing.menu .active.item,.ui.secondary.pointing.menu .item{font-family:Roboto,sans-serif!important;font-size:1.2em;font-weight:500!important}.ui.secondary.pointing.menu .item{color:hsla(0,0%,50.2%,.7)!important}.ui.secondary.pointing.menu{border-bottom:none!important}input{font-family:Roboto,sans-serif!important;font-size:1.1em;font-weight:300!important;outline:none!important;padding:5px}.ui.floating.label{margin:0;top:.5em}.ui.menu .item>.floating.label{padding:.3em .6em}.ui.menu{border-radius:0}.ui.inverted.menu{background-color:#1c2b36}.ui.menu .item>i.icon{margin:0}"]
            },] }
];
MenuComponent.ctorParameters = () => [
    { type: ConfigurationService },
    { type: MenuEventService }
];
MenuComponent.propDecorators = {
    show: [{ type: Input }],
    notificationSideBarRef: [{ type: ViewChild, args: ['notificationSideBar_tag', { static: false },] }],
    layoutSideBarRef: [{ type: ViewChild, args: ['layoutSideBar_tag', { static: false },] }],
    aboutSideBarRef: [{ type: ViewChild, args: ['aboutSideBar_tag', { static: false },] }],
    stickyMenuRef: [{ type: ViewChild, args: ['stickymenu_tag', { static: false },] }]
};

/**
 * Created by jayhamilton on 1/31/17.
 */
const boardLayouts = [
    {
        id: 0,
        boardInstanceId: 0,
        title: 'one-narrow',
        checked: false,
        structure: '8',
        rows: [{
                columns: [
                    {
                        styleClass: 'eight wide',
                    }
                ]
            }]
    },
    {
        id: 1,
        boardInstanceId: 1,
        title: 'single',
        checked: false,
        structure: '14',
        rows: [{
                columns: [
                    {
                        styleClass: 'fourteen wide',
                    }
                ]
            }]
    },
    {
        id: 2,
        boardId: 2,
        title: 'narrow-right',
        checked: false,
        structure: '10-6',
        rows: [{
                columns: [
                    {
                        styleClass: 'ten wide',
                    },
                    {
                        styleClass: 'six wide',
                    }
                ]
            }]
    },
    {
        id: 3,
        boardInstanceId: 3,
        title: 'wide-center',
        checked: false,
        structure: '4-8-4',
        rows: [{
                columns: [
                    {
                        styleClass: 'four wide',
                    },
                    {
                        styleClass: 'eight wide',
                    },
                    {
                        styleClass: 'four wide',
                    }
                ]
            }]
    },
    {
        id: 4,
        boardInstanceId: 4,
        title: 'narrow-left',
        checked: false,
        structure: '4-12',
        rows: [{
                columns: [
                    {
                        styleClass: 'four wide',
                    },
                    {
                        styleClass: 'twelve wide',
                    }
                ]
            }]
    },
    {
        id: 5,
        boardInstanceId: 5,
        title: 'two-even',
        checked: true,
        structure: '6-6',
        rows: [{
                columns: [
                    {
                        styleClass: 'six wide',
                    },
                    {
                        styleClass: 'six wide',
                    }
                ]
            }]
    },
    {
        id: 6,
        boardInstanceId: 6,
        title: 'three-even',
        checked: false,
        structure: '5-5-5',
        rows: [{
                columns: [
                    {
                        styleClass: 'four wide',
                    },
                    {
                        styleClass: 'four wide',
                    },
                    {
                        styleClass: 'four wide',
                    }
                ]
            }]
    },
    {
        id: 7,
        boardInstanceId: 7,
        title: 'wide-top',
        checked: false,
        structure: '16/8-8',
        rows: [
            {
                columns: [
                    {
                        styleClass: 'twelve wide'
                    }
                ]
            },
            {
                columns: [
                    {
                        styleClass: 'six wide'
                    },
                    {
                        styleClass: 'six wide'
                    }
                ]
            }
        ]
    },
    {
        id: 8,
        boardInstanceId: 8,
        title: 'ngadmin',
        checked: false,
        structure: '4-4-4-4/8-4-4',
        rows: [
            {
                columns: [
                    {
                        styleClass: 'four wide'
                    },
                    {
                        styleClass: 'four wide'
                    },
                    {
                        styleClass: 'four wide'
                    },
                    {
                        styleClass: 'four wide'
                    }
                ]
            },
            {
                columns: [
                    {
                        styleClass: 'eight wide'
                    },
                    {
                        styleClass: 'four wide'
                    },
                    {
                        styleClass: 'four wide'
                    }
                ]
            }
        ]
    },
    {
        id: 9,
        boardInstanceId: 9,
        title: 'google-layout',
        checked: false,
        structure: '3-6-3',
        rows: [
            {
                columns: [
                    {
                        styleClass: 'three wide'
                    },
                    {
                        styleClass: 'six wide'
                    },
                    {
                        styleClass: 'three wide'
                    }
                ]
            }
        ]
    }
];

/**
 * Created by jayhamilton on 1/24/17.
 */
/**
 * Message Modal - clasable modal with message
 *
 * Selector message-modal
 *
 * Methods
 *      popMessageModal - display a message modal for a sepcified duration
 *      showMessageModal - show the message modal
 *      hideMessageModal - hide the message modal
 */
class BoardLayoutManagerComponent {
    constructor() {
        this.boardLayoutChangeEvent = new EventEmitter();
        this.modalHeader = 'Layout';
        this.initializeLayouts();
    }
    selectBoardLayout(layoutId) {
        for (let x = 0; x < this.boardLayouts.length; x++) {
            if (this.boardLayouts[x].id === layoutId) {
                this.boardLayoutChangeEvent.emit(this.boardLayouts[x]);
                this.layoutId = layoutId;
                break;
            }
        }
    }
    initializeLayouts() {
        Object.assign(this, { boardLayouts });
    }
}
BoardLayoutManagerComponent.decorators = [
    { type: Component, args: [{
                selector: 'dashboard-layout-manager-modal',
                moduleId: 'layout',
                template: "<br>\n\n<div style=\"text-align: center\">\n    <h2>{{modalHeader}}</h2>\n</div>\n<hr>\n\n<br>\n<br>\n<ul>\n    <li *ngFor=\"let layout of boardLayouts; let i = index\"\n        style='list-style: none'>\n        <img (click)=\"selectBoardLayout(layout.id)\" [ngClass]=\"{'layout-selected': layoutId == layout.id }\"\n             src=\"/assets/images/{{layout.title}}.png\">\n\n    </li>\n</ul>\n\n\n\n",
                styles: [".layout-selected{border-top:solid #7cfc00;padding-top:5px}ul{text-align:center}h2{color:#fff!important}dir,menu,ul{-webkit-padding-start:5px}"]
            },] }
];
BoardLayoutManagerComponent.ctorParameters = () => [];
BoardLayoutManagerComponent.propDecorators = {
    layoutId: [{ type: Input }],
    boardLayoutChangeEvent: [{ type: Output }]
};

class LayoutModule {
}
LayoutModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ],
                declarations: [BoardLayoutManagerComponent],
                exports: [BoardLayoutManagerComponent]
            },] }
];

/**
 * Created by jayhamilton on 2/7/17.
 */
class AddGadgetService {
    constructor(_http) {
        this._http = _http;
    }
    getGadgetLibrary() {
        // TODO: make this configurable. the application need to define this.
        const gadgetLibraryJson = 'gadget-library-model.json';
        return this._http.get('/assets/api/' + gadgetLibraryJson);
    }
}
AddGadgetService.ɵprov = ɵɵdefineInjectable({ factory: function AddGadgetService_Factory() { return new AddGadgetService(ɵɵinject(HttpClient)); }, token: AddGadgetService, providedIn: "root" });
AddGadgetService.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
AddGadgetService.ctorParameters = () => [
    { type: HttpClient }
];

class Facet {
    constructor(name, tags) {
        this.name = name;
        this.tags = tags;
    }
}
class Tag {
    constructor(name) {
        this.name = name;
        this.count = 1;
    }
}

class FacetTagProcessor {
    constructor(objectList) {
        this.facet_tags = [];
        this.objectList = objectList;
    }
    getFacetTags() {
        const me = this;
        this.objectList.forEach(function (item) {
            me.formatAndUpdateTagList(item.tags);
        });
        return this.facet_tags;
    }
    formatAndUpdateTagList(items) {
        items.forEach(tag => {
            // add the first facet and tag to the facet_tag array
            if (!this.facet_tags.length) {
                this.createFacetAndAddItToTheFacetTagArray(tag);
            }
            else {
                let facetExists = false;
                this.facet_tags.forEach(facet => {
                    if (facet.name.toLowerCase() === tag.facet.toLowerCase()) {
                        facetExists = true;
                    }
                });
                if (facetExists) {
                    this.updateFacetWithTag(tag);
                }
                else {
                    this.createFacetAndAddItToTheFacetTagArray(tag);
                }
            }
        });
    }
    createFacetAndAddItToTheFacetTagArray(tag) {
        const _tags = [];
        const _tag = this.createTag(this.capitalize(tag.name));
        _tags.push(_tag);
        const facet = new Facet(tag.facet, _tags);
        this.facet_tags.push(facet);
    }
    createTag(tag) {
        return new Tag(tag);
    }
    updateFacetWithTag(tag) {
        // find the facet and then add the tag or update the count
        this.facet_tags.forEach(facet => {
            if (facet.name.toLowerCase() === tag.facet.toLowerCase()) {
                let tagExists = false;
                facet.tags.forEach(_tag => {
                    if (_tag.name.toLowerCase() === tag.name.toLowerCase()) {
                        tagExists = true;
                        _tag.count = _tag.count + 1;
                    }
                });
                if (!tagExists) {
                    facet.tags.push(this.createTag(this.capitalize(tag.name)));
                }
            }
        });
    }
    capitalize(value) {
        return value.charAt(0).toUpperCase() + value.slice(1).toLowerCase();
    }
}

/**
 * Created by jayhamilton on 1/24/17.
 */
/**
 * Message Modal - clasable modal with message
 *
 * Selector message-modal
 *
 * Methods
 *      popMessageModal - display a message modal for a sepcified duration
 *      showMessageModal - show the message modal
 *      hideMessageModal - hide the message modal
 */
class AddGadgetComponent {
    constructor(_addGadgetService) {
        this._addGadgetService = _addGadgetService;
        this.addGadgetEvent = new EventEmitter();
        this.gadgetObjectList = [];
        this.gadgetObjectTitleList = [];
        this.placeHolderText = 'Begin typing gadget name';
        this.layoutColumnOneWidth = 'six';
        this.layoutColumnTwoWidth = 'ten';
        this.listHeader = 'Gadgets';
        this.color = 'white';
        this.typeAheadIsInMenu = false;
        this.getObjectList();
    }
    actionHandler(actionItem, actionName) {
        this.addGadgetEvent.emit(actionItem);
        this.hideMessageModal();
    }
    showMessageModal(icon, header, message) {
        this.modalicon = icon;
        this.modalheader = header;
        this.modalmessage = message;
        this.messageModal.modal('show');
    }
    showComponentLibraryModal(header) {
        this.modalheader = header;
        this.messageModal.modal('show');
    }
    hideMessageModal() {
        this.modalicon = '';
        this.modalheader = '';
        this.modalmessage = '';
        this.messageModal.modal('hide');
    }
    ngAfterViewInit() {
        this.messageModal = jQuery(this.messagemodalRef.nativeElement);
    }
    getObjectList() {
        this._addGadgetService.getGadgetLibrary().subscribe(data => {
            this.gadgetObjectList.length = 0;
            const me = this;
            data.library.forEach(function (item) {
                me.gadgetObjectList.push(item);
                me.gadgetObjectTitleList.push(item.name);
            });
            const facetTagProcess = new FacetTagProcessor(this.gadgetObjectList);
            this.facetTags = facetTagProcess.getFacetTags();
        });
    }
}
AddGadgetComponent.decorators = [
    { type: Component, args: [{
                selector: 'dashboard-add-gadget-modal',
                moduleId: module.id,
                template: "<div #messagemodal_tag class=\"ui long modal\">\n    <div class=\"header\">\n        <h2>{{modalheader}}</h2>\n    </div>\n\n    <dashboard-data-list\n\n        [facetTags]=\"facetTags\"\n        [layoutColumnOneWidth]=\"layoutColumnOneWidth\"\n        [layoutColumnTwoWidth]=\"layoutColumnTwoWidth\"\n        [listHeader]=\"listHeader\"\n        [objectList]=\"gadgetObjectList\"\n        [objectTitleList]=\"gadgetObjectTitleList\"\n        [placeHolderText]=\"placeHolderText\"\n        [typeAheadIsInMenu]=\"typeAheadIsInMenu\">\n\n        <ng-template let-item=\"$implicit\">\n\n            <div [@showHideAnimation] class=\"ui segment\" style=\"background-color:white;\">\n                <div class=\"ui large middle aligned divided list\">\n                    <div class=\"item\">\n                        <div class=\"right floated content\">\n                            <!-- todo : if there are more than one action display a button with\n                            vertical ellipse and implement onHover to show all action buttons\n                            -->\n                            <a (click)=\"actionHandler(item, item.actions[0].name)\"\n                               *ngIf=\"item.actions.length == 1\"\n                               color=\"primary\"\n                               mat-raised-button routerLink=\".\">\n                                {{item.actions[0].name}}\n                            </a>\n                        </div>\n                        <img class=\"ui image\" src=\"{{item.icon}}\">\n                        <div class=\"content\">\n                            <div class=\"header\">{{item.name}}</div>\n                            {{item.description}}\n                        </div>\n                    </div>\n                </div>\n                <div class=\"ui bottom attached\">\n                    <button *ngFor=\"let tag of item.tags\" class=\"ui basic gray compact button\">{{tag.name}}\n                    </button>\n                </div>\n            </div>\n\n        </ng-template>\n\n    </dashboard-data-list>\n\n    <div class=\"actions\">\n        <div class=\"ui approve button\">Close</div>\n    </div>\n</div>\n",
                animations: [
                    trigger('showHideAnimation', [
                        transition(':enter', [
                            style({ opacity: 0 }),
                            animate(750, style({ opacity: 1 }))
                        ]),
                        transition(':leave', [
                            animate(750, style({ opacity: 0 }))
                        ])
                    ])
                ],
                styles: [".modal{background-color:#f7f7f7}"]
            },] }
];
AddGadgetComponent.ctorParameters = () => [
    { type: AddGadgetService }
];
AddGadgetComponent.propDecorators = {
    addGadgetEvent: [{ type: Output }],
    messagemodalRef: [{ type: ViewChild, args: ['messagemodal_tag', { static: true },] }]
};

/**
 * Created by jayhamilton on 7/11/17.
 */
class FacetComponent {
    constructor() {
        this.tagSelectEvent = new EventEmitter();
    }
    ngOnInit() {
        if (this.openFacet) {
            this.facetOpen = 'in';
        }
        else {
            this.facetOpen = 'out';
        }
    }
    toggleAccordion() {
        this.facetOpen = this.facetOpen === 'out' ? 'in' : 'out';
    }
    tagSelect(tagName) {
        this.tagSelectEvent.emit(tagName);
    }
}
FacetComponent.decorators = [
    { type: Component, args: [{
                moduleId: module.id,
                selector: 'dashboard-facet',
                template: `

        <hr style='max-width: 100%; margin-left:0;'>
        <br>
        <div class='ui container grid'>
            <div class='eight wide column' style="margin-left: 0 !important;padding-left: 0 !important;">
                <h4>{{facet.name}}</h4>
            </div>
            <div class='eight wide column'>
                <div class='ui top right attached label' style='background-color: whitesmoke'>
                    <i
                        class='chevron icon' [ngClass]= "{'up': facetOpen == 'in', 'down': facetOpen=='out'}"
                        (click)='toggleAccordion()'
                        style='color:grey'>
                    </i>
                </div>
            </div>
        </div>
        <div [@accordion]='facetOpen'>
            <table class='ui very basic table' [@accordion2]='facetOpen'>
                <tbody>
                <tr *ngFor='let tag of facet.tags'>
                    <td style="width: 35px !important">
                        <mat-checkbox (click)='tagSelect(tag.name)'></mat-checkbox>
                        <!--<input type='checkbox' (click)='tagSelect(tag.name)'>-->
                    </td>
                    <td style='color:grey'>{{tag.name}} &nbsp;( {{tag.count}} )</td>
                </tr>
                </tbody>
            </table>
        </div>
        <br>

    `,
                animations: [
                    trigger('accordion', [
                        state('in', style({
                            height: '*'
                        })),
                        state('out', style({
                            opacity: '0',
                            height: '0px'
                        })),
                        transition('in => out', animate('700ms ease-in-out')),
                        transition('out => in', animate('300ms ease-in-out'))
                    ]),
                    trigger('accordion2', [
                        state('in', style({
                            height: '*'
                        })),
                        state('out', style({
                            opacity: '0',
                            height: '0px'
                        })),
                        transition('in => out', animate('300ms ease-in-out')),
                        transition('out => in', animate('800ms ease-in-out'))
                    ])
                ],
                styles: [".ui.avatar.image{height:4em!important;width:4.4em!important}hr{background:hsla(0,0%,43.1%,.1)!important;border:0;color:hsla(0,0%,43.1%,.1)!important;font-size:0;height:2px!important}.ui.input.focus input,.ui.input input:focus{border-color:#e9e9e9!important}h3,h4{color:rgba(66,66,66,.74)!important;font-size:.85em;font-weight:400!important}.segment,.ui.basic.segment{border-radius:5px!important}.ui.table tr td{border-top:none!important}.ui.table td{font-size:.8em;padding:.3em!important}"]
            },] }
];
FacetComponent.ctorParameters = () => [];
FacetComponent.propDecorators = {
    tagSelectEvent: [{ type: Output }],
    facet: [{ type: Input }],
    openFacet: [{ type: Input }]
};

/**
 * Created by jayhamilton on 6/27/17.
 */
class FilterListComponent {
    constructor() {
        this.tagSelectEvent = new EventEmitter();
    }
    tagSelect(tagName) {
        this.tagSelectEvent.emit(tagName);
    }
}
FilterListComponent.decorators = [
    { type: Component, args: [{
                moduleId: module.id,
                selector: 'dashboard-filter-list',
                template: `
        <br>
        <div *ngFor='let facet of facet_tags ;let i = index'>

            <dashboard-facet [facet]='facet' (tagSelectEvent)='tagSelect($event)' [openFacet]='i < 2'></dashboard-facet>

        </div>
    `,
                styles: [".ui.avatar.image{height:4em!important;width:4.4em!important}hr{background:hsla(0,0%,43.1%,.1)!important;border:0;color:hsla(0,0%,43.1%,.1)!important;font-size:0;height:2px!important}.ui.input.focus input,.ui.input input:focus{border-color:#e9e9e9!important}h3,h4{color:rgba(66,66,66,.74)!important;font-size:.85em;font-weight:400!important}.segment,.ui.basic.segment{border-radius:5px!important}.ui.table tr td{border-top:none!important}.ui.table td{font-size:.8em;padding:.3em!important}"]
            },] }
];
FilterListComponent.propDecorators = {
    tagSelectEvent: [{ type: Output }],
    facet_tags: [{ type: Input }]
};

/**
 * Created by jayhamilton on 6/27/17.
 */
class FilterTagComponent {
    constructor() {
        this.updateFilterListEvent = new EventEmitter();
        this.filterList = [];
    }
    updateFilterList(filter) {
        filter = filter.toLocaleLowerCase();
        const index = this.filterList.indexOf(filter);
        if (index !== -1) {
            this.filterList.splice(index, 1);
        }
        else {
            this.filterList.push(filter);
        }
        this.updateFilterListEvent.emit(this.filterList);
    }
}
FilterTagComponent.decorators = [
    { type: Component, args: [{
                moduleId: module.id,
                selector: 'dashboard-filter-tag',
                template: `
        <div class='ui basic segment' style='background-color:white; min-height: 4.5em; border-radius:5px'>
            <div class='ui large circular labels'>
               <a class='ui label' [@showHideAnimation] *ngFor='let tag of filterList'>
                    {{tag}}
                </a>
            </div>
        </div>
    `,
                animations: [
                    trigger('showHideAnimation', [
                        transition(':enter', [
                            style({ opacity: 0 }),
                            animate(750, style({ opacity: 1 }))
                        ]),
                        transition(':leave', [
                            animate(750, style({ opacity: 0 }))
                        ])
                    ])
                ],
                styles: [".gadget{color:grey;font-family:Helvetica Neue,Helvetica,Open Sans,Arial,Lucida Grande,sans-serif;margin-bottom:15px!important;min-height:inherit!important}:host /deep/ div:not(.proppages){display:block!important;margin-left:auto;margin-right:auto}.ui.form .field>label{color:grey;font-family:Helvetica Neue,Helvetica,Open Sans,Arial,Lucida Grande,sans-serif;font-size:1.3em!important;font-weight:300}#bt{text-align:center!important}input{border:none!important;font-size:1.3em!important;font-weight:300!important;outline:none!important;padding:5px;width:75%!important}.ct-title,input{font-family:Helvetica Neue,Helvetica,Open Sans,Arial,Lucida Grande,sans-serif}.ct-title{color:#585858!important;font-size:1.5em!important;font-weight:500!important;text-align:left!important}.ui.top.attached.label,.ui.top.left.floated.label{background-color:#fff}hr{border-style:solid;color:#000;font-weight:300!important;height:1px}.ui[class*=\"top attached\"].label:before{border-bottom:0 solid #d0d3d6;bottom:5px;content:\"\";height:1px;left:2%;position:absolute;width:95%}.spacer{height:2em}.ui.items>.item>.content>.description{color:#a9a9a9!important}.suggestions>li{background-color:#fff!important}td{color:grey}td,th{font-family:Helvetica Neue,Helvetica,Open Sans,Arial,Lucida Grande,sans-serif}th{color:#369}.example-full-width{width:100%}h4{font-weight:400}.ui.segment{border:none;box-shadow:none}"]
            },] }
];
FilterTagComponent.ctorParameters = () => [];
FilterTagComponent.propDecorators = {
    updateFilterListEvent: [{ type: Output }]
};

/*
 * Capitalize the first letter of the string
 * Takes a string as a value.
 * Usage:
 *  value | capitalizefirst
 * Example:
 *  // value.name = daniel
 *  {{ value.name | capitalizefirst  }}
 *  fromats to: Daniel
 */
class CapitalizeFirstPipe {
    transform(value, args) {
        if (value === null) {
            return 'Not assigned';
        }
        return value.charAt(0).toUpperCase() + value.slice(1);
    }
}
CapitalizeFirstPipe.decorators = [
    { type: Pipe, args: [{
                name: 'capitalizeFirst'
            },] }
];

class FacetModule {
}
FacetModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    FormsModule,
                    MatCheckboxModule
                ],
                declarations: [
                    FacetComponent,
                    FilterListComponent,
                    FilterTagComponent,
                    CapitalizeFirstPipe
                ],
                providers: [
                    AddGadgetService
                ],
                exports: [
                    FacetComponent,
                    FilterListComponent,
                    FilterTagComponent,
                    CapitalizeFirstPipe
                ]
            },] }
];

/**
 * Created by jayhamilton on 6/22/17.
 */
class ErrorObject {
    constructor(detail, summary, solutions, resource) {
        this.detail = detail;
        this.summary = summary;
        this.solutions = solutions;
        this.resource = resource;
    }
}
class SolutionObject {
    constructor(summary, articleId, link) {
        this.summary = summary;
        this.articleId = articleId;
        this.link = link;
    }
}

/**
 * Created by jayhamilton on 7/5/17.
 */
class ErrorHandler {
    static getErrorObject(errMsg) {
        return new ErrorObject(errMsg.statusText, 'Some description', ErrorHandler.getSolutionList(errMsg.status
            + ' '
            + errMsg.statusText), errMsg.resource);
    }
    /**
     * todo - fix this error handling logic. Move it to its own class.
     * @param errMsg
     * @returns {SolutionObject[]}
     */
    static getSolutionList(errMsg) {
        console.log('ERROR CONDITION:  ' + errMsg);
        const solutionList = [];
        switch (ErrorHandler.getErrorType(errMsg.toLowerCase())) {
            case 'ERR_CERTIFICATE':
                solutionList.push(new SolutionObject('Check to see if your browser has accepted the certificate', 0, 'http://link1'));
                break;
            case 'ERR_CROSS_ORIGIN_RESOURCE_SHARING':
                solutionList.push(new SolutionObject('Check to see if your browser has accepted the certificate', 0, 'http://link1'));
                break;
            case 'ERR_CONNECTION_REFUSED':
                solutionList.push(new SolutionObject('Check to see if the host/service you are attempting to connect to is up.', 0, 'http://link1'));
                break;
            case 'ERR_NOT_FOUND':
                solutionList.push(new SolutionObject('Resource not found.', 0, 'http://link1'));
                break;
            case 'ERR_CONNECTION_TIMEOUT':
                solutionList.push(new SolutionObject('A timeout occurred. The default timeout on a connection is 60 seconds. ' +
                    'Check the endpoint to see if you are able to access the ip and port using ping. ' +
                    'If 60 seconds is to short of a timeout go into configuration and increase it.', 0, 'http://link1'));
                break;
            default: {
                solutionList.push(new SolutionObject(errMsg, 0, 'http://link1'));
            }
        }
        return solutionList;
    }
    static getErrorType(errMsg) {
        if (errMsg.indexOf('trust') > -1) {
            return 'ERR_CERTIFICATE';
        }
        if (errMsg.indexOf('cors') > -1) {
            return 'ERR_CROSS_ORIGIN_RESOURCE_SHARING';
        }
        if (errMsg.indexOf('refuse') > -1) {
            return 'ERR_CONNECTION_REFUSED';
        }
        if (errMsg.indexOf('timeout') > -1) {
            return 'ERR_CONNECTION_TIMEOUT';
        }
        if (errMsg.indexOf('404') > -1) {
            return 'ERR_NOT_FOUND';
        }
        return errMsg;
    }
    static getWebSocketErrorReason(error) {
        let reason = 'There was probably a problem with an attempt to connect to the endpoint!';
        switch (error.code) {
            case 1000:
                reason = 'Normal closure';
                break;
            case 1001:
                reason = 'An endpoint is going away';
                break;
            case 1002:
                reason = 'An endpoint is terminating the connection due to a protocol error.';
                break;
            case 1003:
                reason = 'An endpoint is terminating the connection because it has received a type of data it cannot accept';
                break;
            case 1004:
                reason = 'Reserved. The specific meaning might be defined in the future.';
                break;
            case 1005:
                reason = 'No status code was actually present';
                break;
            case 1006:
                reason = 'The connection was closed abnormally';
                break;
            case 1007:
                reason = 'The endpoint is terminating the connection because a message was received that contained inconsistent data';
                break;
            case 1008:
                reason = 'The endpoint is terminating the connection because it received a message that violates its policy';
                break;
            case 1009:
                reason = 'The endpoint is terminating the connection because a data frame was received that is too large';
                break;
            case 1010:
                reason = 'The client is terminating the connection because it expected the server to negotiate one or more extension, but the server didn\'t.';
                break;
            case 1011:
                reason = 'The server is terminating the connection because it encountered an unexpected condition that prevented it from fulfilling the request.';
                break;
            case 1012:
                reason = 'The server is terminating the connection because it is restarting';
                break;
            case 1013:
                reason = 'The server is terminating the connection due to a temporary condition';
                break;
            case 1015:
                reason = 'The connection was closed due to a failure to perform a TLS handshake';
                break;
        }
        return reason;
    }
}

/**
 * Created by jayhamilton on 1/18/17.
 */
class RuntimeService {
    constructor() {
    }
    static handleError(err) {
        const errMsg = {
            status: '-1',
            statusText: '',
            resource: ''
        };
        if (err.error instanceof Error) {
            errMsg.statusText = err.error.message;
            console.log('Client error');
        }
        else {
            errMsg.status = err.status;
            errMsg.statusText = 'A backend error occurred. In all likelihood the server/api service is not running.';
            errMsg.resource = err.url;
        }
        return throwError(ErrorHandler.getErrorObject(errMsg));
    }
}
RuntimeService.decorators = [
    { type: Injectable }
];
RuntimeService.ctorParameters = () => [];

/**
 * Created by jayhamilton on 2/26/17.
 */
class TypeAheadInputComponent {
    constructor(myElement, _runtimeService) {
        this._runtimeService = _runtimeService;
        this.selectionEvent = new EventEmitter();
        this.ArtificialIntelligenceEventEmitter = new EventEmitter();
        this.requestCounter = 0;
        this.maxAttempts = 5;
        this.query = '';
        this.filteredList = [];
        this.elementRef = myElement;
    }
    filter() {
        if (this.query !== '') {
            this.filteredList = this.searchList.filter(function (el) {
                return el.toLowerCase().indexOf(this.query.toLowerCase()) > -1;
            }.bind(this));
        }
        else {
            this.filteredList = [];
        }
        this.selectionEvent.emit(this.query);
        this.requestCounter++;
    }
    select(item) {
        this.query = item;
        this.filteredList = [];
        this.selectionEvent.emit(item);
    }
}
TypeAheadInputComponent.decorators = [
    { type: Component, args: [{
                moduleId: module.id,
                selector: 'dashboard-typeahead-input',
                template: "<div [ngClass]=\"{inverted:typeAheadIsInMenu == true}\" class=\"ui fluid transparent icon input\">\n    <input (keyup)=filter() [(ngModel)]=query placeholder={{placeHolderText}} type=\"text\" width=\"100%\">\n    <i *ngIf=\"!query\" class=\"large grey search icon\"></i> <i (click)=\"query=''\" *ngIf=\"query\"\n                                                             class=\"large grey remove link icon\"></i>\n</div>\n\n<div *ngIf='filteredList.length > 0' class='suggestions'>\n\n    <ul style='list-style-type:none; background: white; border-radius: 7px;'>\n        <li *ngFor='let item of filteredList' style='padding:5px'>\n            <a (click)='select(item)'>{{item}}</a>\n        </li>\n    </ul>\n</div>\n",
                styles: [".full-width{width:100%}/deep/ .ui.menu .item>.input input{font-size:1.1em!important}/deep/ .mat-form-field-underline{height:0!important}"]
            },] }
];
TypeAheadInputComponent.ctorParameters = () => [
    { type: ElementRef },
    { type: RuntimeService }
];
TypeAheadInputComponent.propDecorators = {
    searchList: [{ type: Input }],
    placeHolderText: [{ type: Input }],
    typeAheadIsInMenu: [{ type: Input }],
    selectionEvent: [{ type: Output }],
    ArtificialIntelligenceEventEmitter: [{ type: Output }]
};

class TypeAheadInputModule {
}
TypeAheadInputModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    FormsModule
                ],
                declarations: [
                    TypeAheadInputComponent
                ],
                providers: [],
                exports: [
                    TypeAheadInputComponent
                ]
            },] }
];

class DataListComponent {
    constructor() {
        this.color = 'white';
        this.objectListCopy = [];
    }
    filterListByTags(filterList) {
        this.copyObjectList();
        this.objectList = this.objectListCopy.filter(object => {
            let tagFound = false;
            if (!filterList.length) {
                return true;
            }
            else {
                object.tags.forEach(tag => {
                    filterList.forEach(filter => {
                        if (tag.name.toLocaleLowerCase() === filter.toLocaleLowerCase()) {
                            tagFound = true;
                        }
                    });
                });
                return tagFound;
            }
        });
    }
    filterListBySearchString(searchString) {
        this.copyObjectList();
        this.objectList = this.objectListCopy.filter(object => {
            if (searchString.localeCompare('') === 0) {
                return true;
            }
            else {
                if (object.name.toLowerCase().indexOf(searchString.toLowerCase()) !== -1) {
                    return true;
                }
            }
        });
    }
    /**
     * todo - find a better way to manage the list that is displayed and filtered.
     */
    copyObjectList() {
        if (this.objectListCopy.length === 0) {
            this.objectList.forEach(item => {
                this.objectListCopy.push(item);
            });
        }
    }
}
DataListComponent.decorators = [
    { type: Component, args: [{
                selector: 'dashboard-data-list',
                moduleId: module.id,
                template: "<div class=\"content\">\n    <div class=\"ui grid\">\n        <div class=\"{{layoutColumnOneWidth}} wide column\">\n            <dashboard-typeahead-input\n                (selectionEvent)=\"filterListBySearchString($event)\"\n                [placeHolderText]=\"placeHolderText\"\n                [searchList]=\"objectTitleList\"\n                [typeAheadIsInMenu]=\"typeAheadIsInMenu\">\n            </dashboard-typeahead-input>\n            <dashboard-filter-list #filterComponent\n                                   (tagSelectEvent)=\"filterTag.updateFilterList($event)\"\n                                   [facet_tags]=\"facetTags\">\n\n            </dashboard-filter-list>\n        </div>\n        <div class=\"{{layoutColumnTwoWidth}} wide column\">\n            <br>\n            <dashboard-filter-tag #filterTag\n                                  (updateFilterListEvent)=\"filterListByTags($event)\">\n\n            </dashboard-filter-tag>\n\n            <div class=\"list-content\">\n                <ng-template [ngForOf]=\"objectList\" [ngForTemplate]=\"template\" ngFor></ng-template>\n            </div>\n        </div>\n    </div>\n</div>\n",
                styles: [".content{background-color:#f7f7f7;margin-left:25px;margin-top:25px;padding:5px}hr{background:hsla(0,0%,43.1%,.1)!important;border:0;color:hsla(0,0%,43.1%,.1)!important;font-size:0;height:2px!important}.list-content{margin-top:20px;max-height:700px;overflow-y:scroll;padding-right:10px}"]
            },] }
];
DataListComponent.propDecorators = {
    objectList: [{ type: Input }],
    objectTitleList: [{ type: Input }],
    placeHolderText: [{ type: Input }],
    layoutColumnOneWidth: [{ type: Input }],
    layoutColumnTwoWidth: [{ type: Input }],
    listHeader: [{ type: Input }],
    typeAheadIsInMenu: [{ type: Input }],
    facetTags: [{ type: Input }],
    template: [{ type: ContentChild, args: [TemplateRef, { static: true },] }]
};

class DataListModule {
}
DataListModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    FacetModule,
                    TypeAheadInputModule,
                    HttpClientModule,
                    BrowserAnimationsModule
                ],
                declarations: [
                    DataListComponent
                ],
                providers: [],
                exports: [
                    DataListComponent
                ]
            },] }
];

class AddGadgetModule {
}
AddGadgetModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    DataListModule,
                    HttpClientModule
                ],
                declarations: [
                    AddGadgetComponent
                ],
                providers: [
                    AddGadgetService
                ],
                exports: [
                    AddGadgetComponent
                ]
            },] }
];

class NotificationComponent {
    constructor() {
        this.notificationFilters = [];
        this.currentNotificationFilter = {
            name: '',
            count: 0
        };
        this.modalHeader = 'Notifications';
        this.notificationFilters.push({ name: 'Unacknowledged', count: 4 });
        this.notificationFilters.push({ name: 'All Notifications', count: 4 });
        this.notificationFilters.push({ name: 'Trend Gadget', count: 4 });
        this.notificationFilters.push({ name: 'CPU Gadget', count: 0 });
        this.setSelectedNotificationFilter({ name: 'Unacknowledged', count: 4 });
    }
    ngOnInit() {
    }
    ngAfterViewInit() {
    }
    setSelectedNotificationFilter(item) {
        this.currentNotificationFilter = item;
    }
}
NotificationComponent.decorators = [
    { type: Component, args: [{
                moduleId: module.id,
                selector: 'dashboard-notification-modal',
                template: "<!-- <div class=\"ui long modal\" #notificationModalTag> -->\n<br>\n<div class=\"header\">\n    <h2><span style=\"padding: 30px; color:white\">{{modalHeader}}</span></h2>\n</div>\n\n\n<!--todo this should be two tables, one for the top filters and another for the component specific filters -->\n<table class=\"ui inverted selectable celled table\">\n    <tbody>\n    <tr (click)=\"setSelectedNotificationFilter(item)\"\n        *ngFor=\"let item of notificationFilters; let idx = index\">\n        <td [ngClass]=\"{'selected': item.name == currentNotificationFilter.name, 'components': idx==1}\">\n            <div style=\"float:left;text-align:left\">{{item.name}}</div>\n            <div style=\"float:right;text-align: right\">{{item.count}}</div>\n        </td>\n    </tr>\n    </tbody>\n</table>\n\n<div>\n\n    <div *ngIf=\"currentNotificationFilter\">\n\n        <dashboard-notification-detail></dashboard-notification-detail>\n    </div>\n\n</div>\n\n\n\n",
                styles: [".components{border-bottom:solid!important;border-bottom-color:hsla(0,0%,50.2%,.69)!important;border-bottom-width:thin!important}td.selected{background-color:#4fa6d3!important;color:#fff;font-weight:lighter}td{line-height:1.7}.ui.table tr td{border:none}tr{cursor:pointer}"]
            },] }
];
NotificationComponent.ctorParameters = () => [];

class NotificationService {
    constructor() {
    }
}
NotificationService.decorators = [
    { type: Injectable }
];
NotificationService.ctorParameters = () => [];

class Notification {
    constructor(name, description) {
        this.name = name;
        this.description = description;
    }
    setState(state) {
        this.state = state;
    }
    getState() {
        return this.state;
    }
    setIcon(icon) {
        this.icon = icon;
    }
    getIcon() {
        return this.icon;
    }
    setWhen(when) {
        this.when = when;
    }
    getWhen() {
        return this.when;
    }
}

/**
 * Created by jayhamilton on 5/16/17.
 */
class NotificationDetailComponent {
    constructor() {
        this.notifications = [];
        let notification = new Notification('CPU Gadget', 'this is a cpu gadget error');
        notification.setWhen((new Date()).toString());
        this.notifications.push(notification);
        notification = new Notification('Trend Gadget', 'this is a trend gadget error2');
        notification.setWhen((new Date()).toString());
        this.notifications.push(notification);
        notification = new Notification('Disk Gadget', 'Disk 1 alert');
        notification.setWhen((new Date()).toString());
        this.notifications.push(notification);
        notification = new Notification('Disk Gadget', 'Disk 2 alert');
        notification.setWhen((new Date()).toString());
        this.notifications.push(notification);
    }
}
NotificationDetailComponent.decorators = [
    { type: Component, args: [{
                selector: 'dashboard-notification-detail',
                moduleId: module.id,
                template: "<h3>&nbsp;&nbsp;Details</h3>\n<table class=\"ui inverted table\">\n    <tr *ngFor=\"let notification of notifications\">\n        <td style=\"font-weight:100 !important;\">\n            <div>\n                {{notification.description}}\n                <br>\n                {{notification.when}}\n            </div>\n        </td>\n    </tr>\n</table>\n\n\n\n\n\n"
            },] }
];
NotificationDetailComponent.ctorParameters = () => [];

class NotificationModule {
}
NotificationModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ],
                declarations: [NotificationComponent, NotificationDetailComponent],
                providers: [NotificationService],
                exports: [NotificationComponent]
            },] }
];

class ObservableWebSocketService {
    createObservableWebSocket(url) {
        this.ws = new WebSocket(url);
        return new Observable(observer => {
            this.ws.onmessage = (event) => observer.next(event.data);
            this.ws.onerror = (event) => observer.error(event);
            this.ws.onclose = (event) => observer.complete();
        });
    }
    sendMessage(message) {
        this.ws.send(message);
    }
}
ObservableWebSocketService.decorators = [
    { type: Injectable }
];

class AboutService {
    constructor(_http) {
        this._http = _http;
        this.env = environment;
    }
    getAPIVersion() {
        const url = '/assets/api/version-model.json';
        return this._http.get(url)
            .pipe(catchError(RuntimeService.handleError));
    }
}
AboutService.ɵprov = ɵɵdefineInjectable({ factory: function AboutService_Factory() { return new AboutService(ɵɵinject(HttpClient)); }, token: AboutService, providedIn: "root" });
AboutService.decorators = [
    { type: Injectable, args: [{ providedIn: 'root' },] }
];
AboutService.ctorParameters = () => [
    { type: HttpClient }
];

/**
 * Created by jayhamilton on 1/24/17.
 */
/**
 * Message Modal - clasable modal with message
 *
 * Selector message-modal
 *
 * Methods
 *      popMessageModal - display a message modal for a sepcified duration
 *      showMessageModal - show the message modal
 *      hideMessageModal - hide the message modal
 */
class AboutComponent {
    constructor(_aboutService) {
        this._aboutService = _aboutService;
        this.modalHeader = 'About';
        this.env = environment;
    }
    ngAfterViewInit() {
        this.getVersion();
    }
    getVersion() {
        this._aboutService.getAPIVersion().subscribe(data => {
            this.apiVersion = data['version'];
        });
    }
}
AboutComponent.decorators = [
    { type: Component, args: [{
                selector: 'dashboard-about-modal',
                moduleId: module.id,
                template: "<br>\n\n<div style=\"text-align: center\">\n    <h2>{{modalHeader}}</h2>\n</div>\n\n<hr>\n\n<br>\n<br>\n\n<div style=\"text-align: center\">\n    <h4>{{env.productName}}</h4>\n</div>\n\n\n<div style=\"text-align: center\">\n    <h4>Version {{env.productVersion}}</h4>\n</div>\n\n<br>\n<br>\n\n<div style=\"text-align: center\">\n    <h4>API Version</h4>\n</div>\n\n<div style=\"text-align: center\">\n    <h4>{{apiVersion}}</h4>\n</div>\n\n\n\n\n\n      ",
                styles: ["ul{text-align:center}h2,h3,h4{color:#fff!important;font-weight:400!important}"]
            },] }
];
AboutComponent.ctorParameters = () => [
    { type: AboutService }
];

class AboutModule {
}
AboutModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ],
                declarations: [AboutComponent],
                exports: [AboutComponent],
                providers: [AboutService]
            },] }
];

/**
 * Created by jayhamilton on 2/3/17.
 */
class PropertyBase {
    constructor(props = {}) {
        this.value = props.value;
        this.key = props.key || '';
        this.label = props.label || '';
        this.required = !props.required;
        this.order = props.order === undefined ? 1 : props.order;
        this.controlType = props.controlType || '';
        this.options = props.options;
    }
}

/**
 * Created by jayhamilton on 2/3/17.
 */
class DropdownProperty extends PropertyBase {
    constructor(options = {}) {
        super(options);
        this.controlType = 'dropdown';
        this.options = [];
        this.options = options['options'] || [];
    }
}

/**
 * Created by jayhamilton on 2/3/17.
 */
class TextboxProperty extends PropertyBase {
    constructor(options = {}) {
        super(options);
        this.controlType = 'textbox';
        this.type = options['type'] || '';
    }
}

/**
 * Created by jayhamilton on 2/3/17.
 */
class HiddenProperty extends PropertyBase {
    constructor(options = {}) {
        super(options);
        this.controlType = 'hidden';
        this.type = 'hidden';
    }
}

/**
 * Created by jayhamilton on 2/3/17.
 */
class CheckboxProperty extends PropertyBase {
    constructor(options = {}) {
        super(options);
        this.controlType = 'checkbox';
        this.type = 'checkbox'; // options['type'] || '';
    }
}

/**
 * Created by jayhamilton on 2/3/17.
 */
class DynamicDropdownProperty extends PropertyBase {
    constructor(options = {}) {
        super(options);
        this.controlType = 'dynamicdropdown';
        this.options = [];
        this.options = options['options'] || [];
    }
}

/**
 * Created by jayhamilton on 2/3/17.
 */
class NumberProperty extends PropertyBase {
    constructor(options = {}) {
        super(options);
        this.controlType = 'number';
        this.type = 'number'; // options['type']|| '';
    }
}

class GadgetPropertyService {
    constructor() {
    }
    setPropertiesAndValues(defaultProperties, properties) {
        let ctrl;
        properties.length = 0;
        defaultProperties.forEach(function (property) {
            if (property.controlType === 'dropdown') {
                ctrl = new DropdownProperty(property);
                properties.push(ctrl);
            }
            else if (property.controlType === 'textbox') {
                ctrl = new TextboxProperty(property);
                properties.push(ctrl);
            }
            else if (property.controlType === 'checkbox') {
                ctrl = new CheckboxProperty(property);
                properties.push(ctrl);
            }
            else if (property.controlType === 'hidden') {
                ctrl = new HiddenProperty(property);
                properties.push(ctrl);
            }
            else if (property.controlType === 'number') {
                ctrl = new NumberProperty(property);
                properties.push(ctrl);
            }
            else if (property.controlType === 'dynamicdropdown') {
                ctrl = new DynamicDropdownProperty(property);
                properties.push(ctrl);
            }
        });
        properties.sort((a, b) => a.order - b.order);
    }
    setPropertyPagesAndProperties(defaultPropertyPages, propertyPages) {
        const me = this;
        // for each defaultPropertyPage object, get the properties
        defaultPropertyPages.forEach(function (propertyPage) {
            const newPropertyPage = {};
            for (const property in propertyPage) {
                if (propertyPage.hasOwnProperty(property)) {
                    if (property !== 'properties') {
                        newPropertyPage[property] = propertyPage[property];
                    }
                    else {
                        const properties = [];
                        me.setPropertiesAndValues(propertyPage.properties, properties);
                        newPropertyPage['properties'] = properties;
                    }
                }
            }
            propertyPages.push(newPropertyPage);
        });
    }
}
GadgetPropertyService.decorators = [
    { type: Injectable }
];
GadgetPropertyService.ctorParameters = () => [];

class MenuModule {
}
MenuModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    NotificationModule,
                    AddGadgetModule,
                    LayoutModule,
                    AboutModule,
                    ConfigurationModule,
                    TypeAheadInputModule,
                    MatButtonModule,
                    MatIconModule
                ],
                providers: [
                    RuntimeService,
                    ConfigurationService,
                    GadgetPropertyService,
                    ObservableWebSocketService,
                    MenuEventService
                ],
                declarations: [
                    MenuComponent
                ],
                exports: [
                    MenuComponent
                ]
            },] }
];

/**a
 * Board component
 *
 */
class BoardComponent {
    constructor() {
        this.title = 'app works';
        this.showMenu = true;
    }
}
BoardComponent.decorators = [
    { type: Component, args: [{
                selector: 'dashboard-board',
                moduleId: module.id,
                template: "<!--\n\nThe grid is primarily controlled by MenuService Events. You can change the behavior by\nchanging the MenuComponent and have it call methods within the Grid component via Output Events within the Menu component or by using the @ViewChild approach.\n\nIn the current code, the MenuService is shared by the GridComponent and MenuComponent. The MenuComponent relays events from the various components that make up the MenuComponent. The GridComponent listens for those events via an Observable. The GridComponent will also raise/emit an event that will be picked up by the MenuComponent via the MenuService through an Observable as well.\n\n-->\n<dashboard-menu [show]=\"showMenu\"></dashboard-menu>\n<dashboard-grid></dashboard-grid>\n"
            },] }
];
BoardComponent.propDecorators = {
    title: [{ type: Input }],
    showMenu: [{ type: Input }]
};

class ToastComponent {
    constructor(_toastService) {
        this._toastService = _toastService;
    }
    ngOnInit() {
        this.messages = this._toastService.getMessages();
        timer(7000, 7000).subscribe(t => {
            this.messages.forEach(message => {
                this.dismiss(message.id);
            });
        });
    }
    dismiss(id) {
        this._toastService.dismissMessage(id);
    }
}
ToastComponent.decorators = [
    { type: Component, args: [{
                selector: 'dashboard-toast',
                template: "<div class=\"wrapper\">\n    <div *ngFor=\"let message of messages\" class=\"toast\">\n        <table (click)=\"dismiss(message.id)\" *ngIf=\"message.dismissed == false\" [@showHide] class=\"ui table\">\n            <tr>\n                <td class=\"toast-icon\" width='50px'><i\n                    class=\"large inverted check icon\"></i></td>\n                <td>\n                    <div class=\"toast-message\">\n                        {{message.content}}\n                    </div>\n                </td>\n            </tr>\n        </table>\n    </div>\n</div>\n",
                animations: [
                    trigger('showHide', [
                        transition(':enter', [
                            style({ opacity: 0 }),
                            animate('500ms', style({ opacity: 1 }))
                        ]),
                        transition(':leave', [
                            style({ opacity: 1 }),
                            animate('500ms', style({ opacity: 0 }))
                        ])
                    ])
                ],
                styles: [".wrapper{position:absolute;right:10px;top:50px;width:400px;z-index:1000}.toast{width:inherit}.toast-icon{background-color:#384b5f;text-align:center}.ui[class*=\"top right attached\"].label{background-color:transparent}.toast-message{display:table-cell;height:40px;vertical-align:middle}"]
            },] }
];
ToastComponent.ctorParameters = () => [
    { type: ToastService }
];

class ReversePipe {
    transform(value, args) {
        value = filter(value, ['dismissed', false]);
        return reverse(value);
    }
}
ReversePipe.decorators = [
    { type: Pipe, args: [{
                name: 'reverse'
            },] }
];

class ToastModule {
}
ToastModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ],
                declarations: [ToastComponent, ReversePipe],
                providers: [ToastService],
                exports: [ToastComponent]
            },] }
];

/**
 * Created by jayhamilton on 1/28/17.
 */
/**
 * todo - the name of this service does not represent the file name. This should be refactored. Consider moving this service to the gadget module instead.
 */
class GadgetInstanceService {
    constructor() {
        this.concreteGadgetInstances = [];
        this.subject = new Subject();
        this.subscribers = [];
    }
    addInstance(gadget) {
        const gadgetFound = this.concreteGadgetInstances.findIndex((instance) => {
            return gadget.instanceId === instance['instance']['instanceId'];
        }) >= 0;
        if (gadgetFound === false) {
            this.concreteGadgetInstances.push(gadget);
        }
    }
    enableConfigureMode() {
        this.concreteGadgetInstances.forEach(function (gadget) {
            gadget.instance.toggleConfigMode();
        });
    }
    removeInstance(id) {
        console.log('REMOVING GADGET');
        // remove instance representation from model
        this.model.rows.forEach(function (row) {
            row.columns.forEach(function (column) {
                if (column.gadgets) {
                    for (let i = column.gadgets.length - 1; i >= 0; i--) {
                        if (column.gadgets[i].instanceId === id) {
                            column.gadgets.splice(i, 1);
                            break;
                        }
                    }
                }
            });
        });
        // removes concrete instance from service
        for (let x = this.concreteGadgetInstances.length - 1; x >= 0; x--) {
            if (this.concreteGadgetInstances[x].instance.instanceId === id) {
                const _gadget = this.concreteGadgetInstances.splice(x, 1);
                _gadget[0].destroy();
                break;
            }
        }
        // raise an event indicating a gadget was removed
        this.subject.next('gadget id: ' + id);
    }
    getInstanceCount() {
        return this.concreteGadgetInstances.length;
    }
    /*
     this allows this service to update the board when a delete operation occurs
     */
    setCurrentModel(model) {
        this.model = model;
    }
    /*
     raise an event that the grid.component is listening for when a gadget is removed.
     */
    listenForInstanceRemovedEventsFromGadgets() {
        return this.subject.asObservable();
    }
    addSubscriber(subscriber) {
        this.subscribers.push(subscriber);
    }
    unSubscribeAll() {
        this.subscribers.forEach(subscription => {
            subscription.unsubscribe();
        });
        this.subscribers.length = 0;
        this.clearAllInstances();
    }
    clearAllInstances() {
        this.concreteGadgetInstances.length = 0;
    }
}
GadgetInstanceService.decorators = [
    { type: Injectable }
];
GadgetInstanceService.ctorParameters = () => [];

/**
 * Created by jayhamilton on 6/15/17.
 */
class GadgetConfigModel {
    constructor(config) {
        this.propertyPages = [];
        config.propertyPages.forEach((page) => {
            const props = [];
            page.properties.forEach((prop) => {
                switch (prop.controlType) {
                    case 'textbox':
                    case 'dropdown':
                    case 'dynamicdropdown':
                        props.push(new PropertyBase(prop));
                        break;
                    case 'checkbox':
                        props.push(new PropertyBase(prop));
                        break;
                    case 'hidden':
                        props.push(new PropertyBase(prop));
                        break;
                    default:
                        props.push(new PropertyBase(prop));
                        break;
                }
            });
            this.propertyPages.push(new PropertyPage(page.displayName, page.groupId, page.position, props));
        });
    }
}
class PropertyPage {
    constructor(displayName, groupId, position, properties) {
        this.displayName = displayName;
        this.groupId = groupId;
        this.position = position;
        this.properties = properties;
    }
}

class GridComponent {
    /**
     * Todo - split model and board operations. This class should really focus on an individual board model's operations
     * within the grid. The board specific operations should be moved to the board component.
     * @param _gadgetInstanceService
     * @param _procmonConfigurationService
     */
    constructor(_gadgetInstanceService, _configurationService, _gadgetLibraryService, _toastService, _menuEventService) {
        this._gadgetInstanceService = _gadgetInstanceService;
        this._configurationService = _configurationService;
        this._gadgetLibraryService = _gadgetLibraryService;
        this._toastService = _toastService;
        this._menuEventService = _menuEventService;
        this.boardUpdateEvent = new EventEmitter();
        this.model = {};
        this.noGadgets = true;
        this.dropZone1 = null;
        this.dropZone2 = null;
        this.dropZone3 = null;
        this.gadgetLibrary = [];
        /** todo
         * Temporary objects for experimenting with AI
         * @type
         */
        this.gridInsertionPosition = {
            x: 0,
            y: 0
        };
        this.removeOldListeners();
        this.setupEventListeners();
        this.initializeBoard();
        this.getGadgetLibrary();
    }
    /**
     * todo - This is a temporary attempt to avoid emitting events from stale listeners.
     * Most severe symptom is when you drill down and then change the layout.
     * Multiple events are triggered per action due to the services not
     * getting destroyed when coming into the main board from a child route. The end result is multiple gadget instances
     * appearing. The following code improves the condition but there still are issues with multiple gadgets appearing
     * when changing the layout.
     *
     */
    removeOldListeners() {
        this._gadgetInstanceService.unSubscribeAll();
        this._menuEventService.unSubscribeAll();
    }
    setupEventListeners() {
        const gadgetRemoveEventSubscriber = this._gadgetInstanceService
            .listenForInstanceRemovedEventsFromGadgets().subscribe((message) => {
            this.saveBoard('Gadget Removed From Board: ' + message, false);
        });
        const menuEventSubscriber = this._menuEventService.listenForMenuEvents().subscribe((event) => {
            const edata = event['data'];
            switch (event['name']) {
                case 'boardChangeLayoutEvent':
                    this.updateBoardLayout(edata);
                    break;
                case 'boardSelectEvent':
                    this.loadBoard(edata);
                    break;
                case 'boardCreateEvent':
                    this.createBoard(edata);
                    break;
                case 'boardEditEvent':
                    this.editBoard(edata);
                    break;
                case 'boardDeleteEvent':
                    this.deleteBoard(edata);
                    break;
                case 'boardAddGadgetEvent':
                    this.addGadget(edata);
                    break;
                case 'boardAIAddGadgetEvent':
                    this.addGadgetUsingArtificialIntelligence(edata);
                    break;
            }
        });
        this._gadgetInstanceService.addSubscriber(gadgetRemoveEventSubscriber);
        this._menuEventService.addSubscriber(menuEventSubscriber);
    }
    /**
     *
     * This is experimental code that deals with AI
     */
    getGadgetLibrary() {
        this._gadgetLibraryService.getGadgetLibrary().subscribe(data => {
            this.gadgetLibrary.length = 0;
            const me = this;
            data.library.forEach(function (item) {
                me.gadgetLibrary.push(item);
            });
        });
    }
    getGadgetFromLibrary(gadgetType) {
        let gadgetObject = null;
        this.gadgetLibrary.forEach(gadget => {
            if (gadgetType.localeCompare(gadget['componentType']) === 0) {
                gadgetObject = gadget;
            }
        });
        return gadgetObject;
    }
    addGadgetUsingArtificialIntelligence(aiObject) {
        /** todo
         * make confidence code configurable
         */
        if (aiObject && aiObject.operation) {
            switch (aiObject.operation) {
                case 'get_storage':
                    this.addGadget(this.getGadgetFromLibrary('StorageObjectListComponent'));
                    break;
                case 'get_cpu':
                    this.addGadget(this.getGadgetFromLibrary('CPUGadgetComponent'));
                    break;
            }
        }
    }
    /**
     * This is the end of the experimental AI code.
     */
    updateGadgetPositionInBoard($event, columnNumber, rowNumber, type) {
        console.log($event['item']['data']);
        console.log(columnNumber + ' ' + rowNumber);
        let moveComplete = false;
        this.getModel().rows.forEach(row => {
            let colpos = 0;
            row.columns.forEach(column => {
                let gadgetpos = 0;
                if (column.gadgets) {
                    column.gadgets.forEach(_gadget => {
                        if (_gadget.instanceId === $event['item']['data'] && !moveComplete) {
                            const gadget = column.gadgets.splice(gadgetpos, 1);
                            if (!this.getModel().rows[rowNumber].columns[columnNumber].gadgets) {
                                this.getModel().rows[rowNumber].columns[columnNumber].gadgets = [];
                            }
                            this.getModel().rows[rowNumber].columns[columnNumber].gadgets.push(gadget[0]);
                            this.saveBoard('drag drop operation', false);
                            moveComplete = true;
                        }
                        gadgetpos++;
                    });
                    colpos++;
                }
            });
        });
    }
    createBoard(name) {
        this.loadNewBoard(name);
    }
    editBoard(name) {
    }
    deleteBoard(name) {
        this._configurationService.deleteBoard(name).subscribe(data => {
            this.initializeBoard();
        }, error => console.error('Deletion error', error), () => console.debug('Board Deletion: ' + name));
    }
    addGadget(gadget) {
        const _gadget = Object.assign({}, gadget);
        _gadget.instanceId = new Date().getTime();
        _gadget.config = new GadgetConfigModel(gadget.config);
        this.setGadgetInsertPosition();
        const x = this.gridInsertionPosition.x;
        const y = this.gridInsertionPosition.y;
        if (!this.getModel().rows[x].columns[y].gadgets) {
            this.getModel().rows[x].columns[y].gadgets = [];
        }
        this.getModel().rows[x].columns[y].gadgets.push(_gadget);
        this.saveBoard('Adding Gadget To The Board', false);
    }
    updateBoardLayout(structure) {
        console.log('IN UPDATE BOARD LAYOUT');
        // user selected the currently selected layout
        if (structure.id === this.getModel().id) {
            return;
        }
        // copy the current board's model
        const _model = Object.assign({}, this.getModel());
        // get just the columns that contain gadgets from all rows
        const originalColumns = this.readColumnsFromOriginalModel(_model);
        // reset the copied model's rows, which include columns
        _model.rows.length = 0;
        // copy the contents of the requested structure into the temporary model
        // we now have a board model we can populate with the original board's gadgets
        Object.assign(_model.rows, structure.rows);
        _model.structure = structure.structure;
        _model.id = structure.id;
        let originalColumnIndexToStartProcessingFrom = 0;
        /* For each column from the original board, copy its gadgets to the new structure.
         The requested layout may have more or less columns than defined by the original layout. So the fillGridStructure method
         will copy column content into the target. If there are more columns than the target,
         the fillGridStructure will return the count of remaining columns to be processed and then process those.
         */
        while (originalColumnIndexToStartProcessingFrom < originalColumns.length) {
            originalColumnIndexToStartProcessingFrom = this.fillGridStructure(_model, originalColumns, originalColumnIndexToStartProcessingFrom);
        }
        // This will copy the just processed model and present it to the board
        this.setModel(_model);
        // clear temporary object
        for (const member in _model) {
            delete _model[member];
        }
        // persist the board change
        this.saveBoard('Grid Layout Update', false);
    }
    enableConfigMode() {
        this._gadgetInstanceService.enableConfigureMode();
    }
    setModel(model) {
        this.model = Object.assign({}, model);
    }
    getModel() {
        return this.model;
    }
    onDrop(data) {
        console.log(data);
    }
    updateGridState() {
        let gadgetCount = 0;
        if (this.getModel().rows) {
            this.getModel().rows.forEach(function (row) {
                row.columns.forEach(function (column) {
                    if (column.gadgets) {
                        column.gadgets.forEach(function (gadget) {
                            gadgetCount++;
                        });
                    }
                });
            });
        }
        this.noGadgets = !gadgetCount;
        this.dashedStyle = {
            'border-style': this.noGadgets ? 'dashed' : 'none',
            'border-width': this.noGadgets ? '2px' : 'none',
            'border-color': this.noGadgets ? 'darkgray' : 'none',
            'padding': this.noGadgets ? '5px' : 'none'
        };
    }
    readColumnsFromOriginalModel(_model) {
        const columns = [];
        _model.rows.forEach(function (row) {
            row.columns.forEach(function (col) {
                columns.push(col);
            });
        });
        return columns;
    }
    fillGridStructure(destinationModelStructure, originalColumns, counter) {
        const me = this;
        destinationModelStructure.rows.forEach(function (row) {
            row.columns.forEach(function (destinationColumn) {
                if (!destinationColumn.gadgets) {
                    destinationColumn.gadgets = [];
                }
                if (originalColumns[counter]) {
                    me.copyGadgets(originalColumns[counter], destinationColumn);
                    counter++;
                }
            });
        });
        return counter;
    }
    copyGadgets(source, target) {
        if (source.gadgets && source.gadgets.length > 0) {
            let w = source.gadgets.shift();
            while (w) {
                target.gadgets.push(w);
                w = source.gadgets.shift();
            }
        }
    }
    initializeBoard() {
        this._configurationService.getBoards().subscribe(boards => {
            if (boards && boards instanceof Array && boards.length) {
                this.boards = boards.sort((a, b) => {
                    return a.boardInstanceId - b.boardInstanceId;
                });
                this.loadBoard(this.boards[0].title);
            }
            else {
                this.loadDefaultBoard();
            }
        });
    }
    loadBoard(boardTitle) {
        this.clearGridModelAndGadgetInstanceStructures();
        const selectedBoard = this.boards.find(board => board.title === boardTitle);
        this.setModel(selectedBoard);
        this.updateServicesAndGridWithModel();
        this.boardUpdateEvent.emit(boardTitle);
    }
    loadDefaultBoard() {
        this.clearGridModelAndGadgetInstanceStructures();
        this._configurationService.getDefaultBoard().subscribe((board) => {
            this.setModel(board);
            this.updateServicesAndGridWithModel();
            this.saveBoard('Initialization of a default board', true);
        });
    }
    loadNewBoard(name) {
        this.clearGridModelAndGadgetInstanceStructures();
        this._configurationService.getDefaultBoard().subscribe((res) => {
            this.setModel(res);
            this.getModel().title = name;
            this.getModel().boardInstanceId = new Date().getTime();
            this.updateServicesAndGridWithModel();
            this.saveBoard('Initialization of a new board', true);
        });
    }
    updateServicesAndGridWithModel() {
        this._gadgetInstanceService.setCurrentModel(this.getModel());
        this._configurationService.setCurrentModel(this.getModel());
        this.updateGridState();
    }
    saveBoard(operation, alertBoardListenerThatTheMenuShouldBeUpdated) {
        this.updateServicesAndGridWithModel();
        this._configurationService.saveBoard(this.getModel()).subscribe(result => {
            this._toastService.sendMessage(this.getModel().title + ' has been updated!', '');
            if (alertBoardListenerThatTheMenuShouldBeUpdated) {
                this._menuEventService.raiseGridEvent({ name: 'boardUpdateEvent', data: this.getModel().title });
            }
        }, error => console.error('Error' + error), () => console.debug('Saving configuration to store!'));
    }
    clearGridModelAndGadgetInstanceStructures() {
        // clear gadgetInstances
        this._gadgetInstanceService.clearAllInstances();
        // clear current model
        for (const prop in this.getModel()) {
            if (this.model.hasOwnProperty(prop)) {
                delete this.model[prop];
            }
        }
    }
    setGadgetInsertPosition() {
        for (let x = 0; x < this.getModel().rows.length; x++) {
            for (let y = 0; y < this.getModel().rows[x].columns.length; y++) {
                if (this.getModel().rows[x].columns[y].gadgets && this.getModel().rows[x].columns[y].gadgets.length === 0) {
                    this.gridInsertionPosition.x = x;
                    this.gridInsertionPosition.y = y;
                    return;
                }
            }
        }
        // we go here because the board is either empty or full
        // insert in the top left most cell
        this.gridInsertionPosition.y = 0;
        if (this.noGadgets) {
            // there are no gadgets so insert in top row
            this.gridInsertionPosition.x = 0;
        }
        else {
            // board is full so insert in the last row
            this.gridInsertionPosition.x = this.getModel().rows.length - 1;
        }
    }
}
GridComponent.decorators = [
    { type: Component, args: [{
                moduleId: module.id,
                selector: 'dashboard-grid',
                template: "<div class=\"ui centered stackable grid\" style=\"margin-left: 5px; margin-right: 5px\"\n>\n    <div *ngFor=\"let row of model.rows; let r = index\" [ngClass]=\"{'short-row': row.columns.length > 3, 'tall-row': row.columns.length <= 3 }\"\n         class=\"row\">\n\n        <ng-container cdkDropListGroup>\n            <div (cdkDropListDropped)=\"updateGadgetPositionInBoard($event, c , r, 'column')\" *ngFor=\"let col of row.columns; let c = index\" cdkDropList\n                 class=\"{{col.styleClass}} column\">\n\n                <div *ngIf=\"!col.gadgets || col.gadgets.length== 0\" [ngClass]=\"{'short-row': row.columns.length > 3, 'tall-row': row.columns.length <= 3 }\"\n                     class=\"ui center aligned massive info message\"\n                     style=\"text-align:center\">\n\n                    <span style=\"font-size: .75em\">Add Gadget!</span>\n\n                </div>\n\n                <dashboard-grid-cell *ngFor=\"let cellGadget of col.gadgets\" [cdkDragData]=\"cellGadget.instanceId\" [gadgetConfig]=\"cellGadget.config\"\n                                     [gadgetInstanceId]=\"cellGadget.instanceId\"\n                                     [gadgetTags]=\"cellGadget.tags\"\n                                     [gadgetType]=\"cellGadget.componentType\"\n                                     cdkDrag\n                                     class=\"target-box\"\n                                     style=\"display:inline-block; width:100%\"\n                >\n                    <div *cdkDragPlaceholder class=\"target-box-placeholder\"></div>\n                </dashboard-grid-cell>\n            </div>\n        </ng-container>\n    </div>\n</div>\n<dashboard-toast></dashboard-toast>\n",
                styles: [".ui.info.message{background-color:#c8cbce;color:#fff}.ui.attached.info.message,.ui.info.message{box-shadow:inset 0 0 0 0 #c8cbce,0 0 0 0 transparent}.short-row{min-height:150px}.tall-row{min-height:370px}.target-box-placeholder{background:transparent;border:none;min-height:400px;transition:transform .25s cubic-bezier(0,0,.2,1)}.target-box{align-items:center;box-sizing:border-box;color:rgba(0,0,0,.87);cursor:move;display:inline-block;flex-direction:row;font-size:14px;justify-content:center;padding:5px}"]
            },] }
];
GridComponent.ctorParameters = () => [
    { type: GadgetInstanceService },
    { type: ConfigurationService },
    { type: AddGadgetService },
    { type: ToastService },
    { type: MenuEventService }
];
GridComponent.propDecorators = {
    boardUpdateEvent: [{ type: Output }]
};

class GadgetFactory {
    static getComponentType(gadgetType) {
        return GadgetFactory.componentMap.get(gadgetType);
    }
    static setComponentType(gadgetType, componentRef) {
        GadgetFactory.componentMap.set(gadgetType, componentRef);
    }
}
GadgetFactory.componentMap = new Map();

/*
 this class handles the dynamic creation of components
 */
class CellComponent {
    constructor(cfr, gadgetInstanceService) {
        this.cfr = cfr;
        this.gadgetInstanceService = gadgetInstanceService;
    }
    ngOnInit() {
        /*
         create component instance dynamically
         */
        const component = GadgetFactory.getComponentType(this.gadgetType);
        if (component) {
            const compFactory = this.cfr.resolveComponentFactory(component);
            const gadgetRef = this.viewContainerRef.createComponent(compFactory);
            /*
             we need to pass the input parameters (instance id and config) back into the newly created component.
             */
            gadgetRef.instance.configureGadget(this.gadgetInstanceId, this.gadgetConfig, this.gadgetTags);
            /*
             add concrete component to service for tracking
             */
            this.gadgetInstanceService.addInstance(gadgetRef);
        }
    }
}
CellComponent.decorators = [
    { type: Component, args: [{
                selector: 'dashboard-grid-cell',
                template: '<ng-template #container></ng-template>'
            },] }
];
CellComponent.ctorParameters = () => [
    { type: ComponentFactoryResolver },
    { type: GadgetInstanceService }
];
CellComponent.propDecorators = {
    gadgetType: [{ type: Input }],
    gadgetConfig: [{ type: Input }],
    gadgetInstanceId: [{ type: Input }],
    gadgetTags: [{ type: Input }],
    viewContainerRef: [{ type: ViewChild, args: ['container', { static: true, read: ViewContainerRef },] }]
};

class GridModule {
    static withComponents(components) {
        return {
            ngModule: GridModule,
            providers: [
                { provide: ANALYZE_FOR_ENTRY_COMPONENTS, useValue: components, multi: true }
            ]
        };
    }
}
GridModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    FormsModule,
                    ToastModule,
                    HttpClientModule,
                    DragDropModule
                ],
                declarations: [
                    GridComponent,
                    CellComponent
                ],
                exports: [
                    GridComponent
                ],
                providers: [
                    GadgetInstanceService,
                    ConfigurationService,
                    AddGadgetService
                ]
            },] }
];

class BoardModule {
}
BoardModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    GridModule,
                    MenuModule,
                ],
                providers: [],
                declarations: [
                    BoardComponent
                ],
                exports: [
                    BoardComponent
                ]
            },] }
];

/**
 * Created by jayhamilton on 2/28/17.
 */
/**
 * Created by jayhamilton on 2/26/17.
 */
class GadgetHeaderComponent {
    constructor() {
        this.removeEvent = new EventEmitter();
        this.toggleConfigModeEvent = new EventEmitter();
        this.runEvent = new EventEmitter();
        this.stopEvent = new EventEmitter();
        this.helpEvent = new EventEmitter();
    }
    remove() {
        this.removeEvent.emit();
    }
    toggleConfigMode() {
        this.toggleConfigModeEvent.emit();
    }
    run() {
        this.runEvent.emit();
    }
    stop() {
        this.stopEvent.emit();
    }
    help() {
        this.helpEvent.emit();
    }
}
GadgetHeaderComponent.decorators = [
    { type: Component, args: [{
                moduleId: module.id,
                selector: 'dashboard-gadget-header',
                template: "<div class=\"ui top attached label\">\n\n    <button (click)=\"remove()\"\n            *ngIf=\"showControls || globalOptions.enableHover\" class=\"compact ui button right floated\">\n        <i class=\"trash icon\" style=\"margin-right:0 !important\"></i>\n    </button>\n    <button (click)=\"toggleConfigMode()\"\n            *ngIf=\"showControls || globalOptions.enableHover && showConfigurationControl\" class=\"compact ui button right floated\">\n        <i class=\"setting  icon\" style=\"margin-right:0 !important\"></i>\n    </button>\n    <button (click)=\"help()\"\n            *ngIf=\"showControls || globalOptions.enableHover\" class=\"compact ui button right floated\">\n        <i class=\"info icon\" style=\"margin-right:0 !important\"></i>\n    </button>\n\n    <dashboard-gadget-operation-control (runEvent)=\"run()\"\n                                        (stopEvent)=\"stop()\"\n                                        *ngIf=\"showControls || globalOptions.enableHover\"\n                                        [actionInitiated]=\"actionInitiated\"\n                                        [gadgetHasOperationControls]=\"gadgetHasOperationControls\"\n                                        [inConfig]=\"inConfig\"\n                                        [inRun]=\"inRun\"\n                                        [showOperationControls]=\"showOperationControls\">\n\n    </dashboard-gadget-operation-control>\n\n    <div class=\"ui top left floated label ct-title\">{{title}}</div>\n</div>\n<div class=\"spacer\"></div>\n",
                styles: [".gadget{color:grey;font-family:Helvetica Neue,Helvetica,Open Sans,Arial,Lucida Grande,sans-serif;margin-bottom:15px!important;min-height:inherit!important}:host /deep/ div:not(.proppages){display:block!important;margin-left:auto;margin-right:auto}.ui.form .field>label{color:grey;font-family:Helvetica Neue,Helvetica,Open Sans,Arial,Lucida Grande,sans-serif;font-size:1.3em!important;font-weight:300}#bt{text-align:center!important}input{border:none!important;font-size:1.3em!important;font-weight:300!important;outline:none!important;padding:5px;width:75%!important}.ct-title,input{font-family:Helvetica Neue,Helvetica,Open Sans,Arial,Lucida Grande,sans-serif}.ct-title{color:#585858!important;font-size:1.5em!important;font-weight:500!important;text-align:left!important}.ui.top.attached.label,.ui.top.left.floated.label{background-color:#fff}hr{border-style:solid;color:#000;font-weight:300!important;height:1px}.ui[class*=\"top attached\"].label:before{border-bottom:0 solid #d0d3d6;bottom:5px;content:\"\";height:1px;left:2%;position:absolute;width:95%}.spacer{height:2em}.ui.items>.item>.content>.description{color:#a9a9a9!important}.suggestions>li{background-color:#fff!important}td{color:grey}td,th{font-family:Helvetica Neue,Helvetica,Open Sans,Arial,Lucida Grande,sans-serif}th{color:#369}.example-full-width{width:100%}h4{font-weight:400}.ui.segment{border:none;box-shadow:none}"]
            },] }
];
GadgetHeaderComponent.propDecorators = {
    title: [{ type: Input }],
    showControls: [{ type: Input }],
    inRun: [{ type: Input }],
    inConfig: [{ type: Input }],
    actionInitiated: [{ type: Input }],
    showOperationControls: [{ type: Input }],
    showConfigurationControl: [{ type: Input }],
    gadgetHasOperationControls: [{ type: Input }],
    globalOptions: [{ type: Input }],
    removeEvent: [{ type: Output }],
    toggleConfigModeEvent: [{ type: Output }],
    runEvent: [{ type: Output }],
    stopEvent: [{ type: Output }],
    helpEvent: [{ type: Output }]
};

/**
 * Created by jayhamilton on 6/29/17.
 */
class GadgetOperationComponent {
    constructor() {
        this.runEvent = new EventEmitter();
        this.stopEvent = new EventEmitter();
    }
    run() {
        this.runEvent.emit();
    }
    stop() {
        this.stopEvent.emit();
    }
}
GadgetOperationComponent.decorators = [
    { type: Component, args: [{
                moduleId: module.id,
                selector: 'dashboard-gadget-operation-control',
                template: `

        <button class="compact ui button right floated"
                *ngIf="!inRun && !actionInitiated && showOperationControls && gadgetHasOperationControls"
                (click)="run()"><i class="green play icon" style="margin-right:0 !important"></i>
        </button>

        <button class="compact ui button right floated"
                *ngIf="!inRun &&
        actionInitiated &&
        showOperationControls &&
        gadgetHasOperationControls">
            <i class="black spinner loading icon" style="margin-right:0 !important"></i>
        </button>

        <button class="compact ui button right floated"
                *ngIf="inRun && !actionInitiated && showOperationControls && gadgetHasOperationControls"
                (click)="stop()"><i class="red stop icon" style="margin-right:0 !important"></i>
        </button>
    `
            },] }
];
GadgetOperationComponent.propDecorators = {
    runEvent: [{ type: Output }],
    stopEvent: [{ type: Output }],
    inRun: [{ type: Input }],
    actionInitiated: [{ type: Input }],
    inConfig: [{ type: Input }],
    showOperationControls: [{ type: Input }],
    gadgetHasOperationControls: [{ type: Input }]
};

/**
 * Created by jayhamilton on 1/24/17.
 */
/**
 * Message Modal - clasable modal with message
 *
 * Selector message-modal
 *
 * Methods
 *      popMessageModal - display a message modal for a sepcified duration
 *      showMessageModal - show the message modal
 *      hideMessageModal - hide the message modal
 */
class HelpModalComponent {
    constructor() {
    }
    showMessageModal(icon, header, message) {
        this.modalicon = icon;
        this.modalheader = header;
        this.modalconfig = message;
        this.configModal.modal('show');
    }
    hideMessageModal() {
        this.modalicon = '';
        this.modalheader = '';
        this.modalconfig = '';
        this.configModal.modal('hide');
    }
    ngAfterViewInit() {
        this.configModal = jQuery(this.helpmodalaRef.nativeElement);
        this.configModal.modal('hide');
    }
    showHelp() {
        this.showMessageModal(null, 'Help', 'Get me out of here!');
    }
}
HelpModalComponent.decorators = [
    { type: Component, args: [{
                selector: 'dashboard-help-modal',
                moduleId: module.id,
                template: "<div #helpmodal_tag class=\"ui long modal\">\n    <div class=\"header\">\n        Help\n    </div>\n\n    <div class=\"ui basic segment\">\n        <br>\n        <br>\n        <p></p>\n        <br><br>\n    </div>\n    <div class=\"actions\">\n        <div class=\"ui approve button\">Close</div>\n    </div>\n</div>\n"
            },] }
];
HelpModalComponent.ctorParameters = () => [];
HelpModalComponent.propDecorators = {
    topic: [{ type: Input }],
    helpmodalaRef: [{ type: ViewChild, args: ['helpmodal_tag', { static: true },] }]
};

/**
 * Created by jayhamilton on 1/24/17.
 */
/**
 * Message Modal - clasable modal with message
 *
 * Selector message-modal
 *
 * Methods
 *      popMessageModal - display a message modal for a sepcified duration
 *      showMessageModal - show the message modal
 *      hideMessageModal - hide the message modal
 */
class VisDrillDownComponent {
    constructor() {
        this.objectList = [];
        this.objectTitleList = [];
        this.placeHolderText = 'Begin typing vm name';
        this.layoutColumnOneWidth = 'six';
        this.layoutColumnTwoWidth = 'ten';
        this.facetTags = [];
    }
    showMessageModal(icon, header, message) {
        this.modalicon = icon;
        this.modalheader = header;
        this.modalconfig = message;
        this.configModal.modal('show');
    }
    hideMessageModal() {
        this.modalicon = '';
        this.modalheader = '';
        this.modalconfig = '';
        this.configModal.modal('hide');
    }
    ngAfterViewInit() {
        this.configModal = jQuery(this.vismodalaRef.nativeElement);
        this.configModal.modal('hide');
    }
    showDrillDownDetail($event) {
        const data = JSON.stringify($event, null, 4);
        this.showMessageModal(null, 'Detail', data);
    }
    showDetail($event) {
        const data = JSON.stringify($event, null, 4);
        this.showMessageModal(null, 'Detail', data);
    }
}
VisDrillDownComponent.decorators = [
    { type: Component, args: [{
                selector: 'dashboard-vis-drill-down-modal',
                moduleId: module.id,
                template: "<div #vismodal_tag class=\"ui long modal\">\n    <div class=\"header\">\n        <h2>{{modalheader}}</h2>\n    </div>\n\n\n    <div class=\"content\">\n\n        {{modalconfig}}\n\n    </div>\n\n\n    <div class=\"actions\">\n        <div class=\"ui approve button\">Close</div>\n    </div>\n</div>\n",
                animations: [
                    trigger('contentSwitch', [
                        state('inactive', style({
                            opacity: 0
                        })),
                        state('active', style({
                            opacity: 1
                        })),
                        transition('inactive => active', animate('750ms ease-in')),
                        transition('active => inactive', animate('750ms ease-out'))
                    ]),
                    trigger('tabSwitch', [
                        state('inactive', style({
                            opacity: .75
                        })),
                        state('active', style({
                            opacity: 1
                        })),
                        transition('inactive => active', animate('750ms ease-in')),
                        transition('active => inactive', animate('750ms ease-out'))
                    ])
                ]
            },] }
];
VisDrillDownComponent.ctorParameters = () => [];
VisDrillDownComponent.propDecorators = {
    vismodalaRef: [{ type: ViewChild, args: ['vismodal_tag', { static: true },] }]
};

class GadgetSharedModule {
}
GadgetSharedModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    MatProgressBarModule,
                ],
                declarations: [
                    GadgetHeaderComponent,
                    GadgetOperationComponent,
                    HelpModalComponent,
                    VisDrillDownComponent
                ],
                exports: [
                    GadgetHeaderComponent,
                    GadgetOperationComponent,
                    HelpModalComponent,
                    VisDrillDownComponent
                ]
            },] }
];

class DetailService {
    constructor(_http) {
        this._http = _http;
        this.testURL = 'http://localhost:8080';
        this.detailURL = '/detail';
        this.configure();
    }
    configure() {
        if (!environment.production) {
            this.detailURL = this.testURL + this.detailURL;
        }
    }
    getDetailByChartSeriesSelected(chartType, chartSeries, chartMetric, endPointName) {
        /**
         * Todo - review these parameters and make them more meaningful. The goal is to send to the service
         * a pameterized URL /endpointURL?detailParameter={value or id}
         */
        let p = new HttpParams();
        p = p.append('detailParam', chartSeries);
        p = p.append('detailMetric', chartMetric);
        return this._http.get(this.detailURL, { params: p })
            .pipe(catchError(RuntimeService.handleError));
    }
    getDetail(url) {
        return this._http.get(url)
            .pipe(catchError(RuntimeService.handleError));
    }
    getRecord(url) {
        return this._http.get(url)
            .pipe(catchError(RuntimeService.handleError));
    }
}
DetailService.decorators = [
    { type: Injectable }
];
DetailService.ctorParameters = () => [
    { type: HttpClient }
];

/**a
 * Detail component
 *
 */
class DetailComponent {
    constructor(_router, _route, _detailService) {
        this._router = _router;
        this._route = _route;
        this._detailService = _detailService;
        this.chartType = '';
        this.chartSeries = '';
        this.chartMetric = '';
        this.endPointName = '';
        this.data = [];
        this.searchText = '';
        this.navRoutes = [];
        this.objectAsArray = [];
        this.navigationSubscription = this._router.events.subscribe((e) => {
            // If it is a NavigationEnd event re-initalise the component
            if (e instanceof NavigationEnd) {
                this.getObjectsByMetric(true);
            }
        });
    }
    ngOnInit() {
        this.chartType = this._route.snapshot.queryParams['chartType'];
        this.chartSeries = this._route.snapshot.queryParams['chartSeries'];
        this.chartMetric = this._route.snapshot.queryParams['chartMetric'];
        this.endPointName = this._route.snapshot.queryParams['endPointName'];
        this.getObjectsByMetric(false);
    }
    getObject(record) {
        this.clearDetailDisplay();
        Object.keys(record).forEach(key => {
            if (key.indexOf('link') < 0) {
                this.objectAsArray.push({ 'key': key, 'value': record[key] });
            }
        });
    }
    getObjectsByHateoasLink(detail) {
        this.clearDetailDisplay();
        let href = '';
        detail.links.forEach((link) => {
            if (link.rel === 'self') {
                href = link.href;
            }
        });
        let navArray = href.split('/');
        //trying to get data for the current record so do nothing. Avoid this altogether by removing the link from the table.
        if (navArray[navArray.length - 1] == this.navRoutes[this.navRoutes.length - 1]) {
            return;
        }
        this.navRoutes.push(navArray[navArray.length - 1]);
        this._detailService.getDetail(href).subscribe((data) => {
            this.data = data.slice();
        });
    }
    getObjectsByMetric(isReload) {
        this.clearDetailDisplay();
        this._detailService.getDetailByChartSeriesSelected(this.chartType, this.chartSeries, this.chartMetric, this.endPointName).subscribe(data => {
            this.data = data.slice();
        });
        if (!isReload) {
            this.navRoutes.push(this.chartMetric);
        }
        else {
            /**
             * todo - this won't work for many routes.
             */
            while (this.navRoutes.length > 1) {
                this.navRoutes.splice(this.navRoutes.length - 1, 1);
            }
        }
    }
    goHome() {
        this._router.navigate(['/main-board']);
    }
    gotToRoute(nav, disabled) {
        if (!disabled) {
            this._router.navigate(['/detail'], {
                queryParams: {
                    chartType: this.chartType,
                    chartSeries: this.chartSeries,
                    chartMetric: this.chartMetric,
                    endPointName: this.endPointName
                }
            });
        }
    }
    clearDetailDisplay() {
        this.objectAsArray = [];
    }
}
DetailComponent.decorators = [
    { type: Component, args: [{
                moduleId: module.id,
                template: "<div class=\"ui centered stackable grid\" style=\"margin-left: 5px; margin-right: 5px\">\n    <div class=\"row\">\n        <div class=\"three wide column\"></div>\n        <div class=\"ten wide column left aligned\">\n\n            <div class=\"ui breadcrumb\">\n                <a (click)=\"goHome()\" class=\"section\">Board</a>\n\n                <span *ngFor=\"let nav of navRoutes,let i = index\">\n                    <span class=\"divider\">/</span>\n                    <a (click)=\"gotToRoute(nav, navRoutes.length -1 == i)\" [ngClass]=\"{'active': navRoutes.length -1 == i }\"\n                       class=\"section\">{{nav}}</a>\n                </span>\n\n            </div>\n\n            <div class=\"ui segment\">\n                <div class=\"ui top right attached label\"><a href=\"http://localhost:8080/detail/downloadFile\"\n                                                            target=\"_self\"><i class=\"big grey download icon\"></i></a>\n                </div>\n\n                <div class=\"ui  basic segment\">\n                     <span class=\"detail-left-header\">\n                        Type: {{chartSeries}}\n                    </span>\n                    <span class=\"detail-right-header\">\n                       Count: {{data.length}} Tasks\n                    </span>\n\n                </div>\n                <div class=\"ui fluid icon input\">\n                    <i class=\"search icon\"></i>\n                    <input [(ngModel)]=\"searchText\" placeholder=\"search text goes here\">\n                </div>\n                <table class=\"ui celled compact fixed table\">\n                    <thead>\n                    <tr>\n                        <th></th>\n                        <th style=\"text-align: center\">Status</th>\n                        <th>Master</th>\n                        <th>Job Name</th>\n                        <th>Sub Type</th>\n                        <th>Job Id</th>\n                        <th>Task Id</th>\n                        <th>Node</th>\n                        <th>Disk</th>\n                        <th>Task Start</th>\n                        <th>Task End</th>\n                    </tr>\n                    </thead>\n                    <tbody>\n                    <tr *ngFor=\"let task of data | filter : searchText; let i = index \">\n                        <td [ngClass]=\"{'negative': task.status != 'Completed', 'positive': task.status == 'Completed' }\">\n                            {{i + 1}}\n                        </td>\n                        <td style=\"text-align: center\"><i [ngClass]=\"{\n                        'red ban': task.status != 'Completed',\n                        'green check': task.status == 'Completed'\n                        }\" class=\"icon large \"></i></td>\n                        <td>{{task.master}}</td>\n                        <td>{{task.jobName}}</td>\n                        <td>{{task.subType}}</td>\n                        <td><a (click)=\"getObjectsByHateoasLink(task)\">{{task.jobId}}</a></td>\n                        <td><a (click)=\"getObject(task)\">{{task.taskId}}</a></td>\n                        <td>{{task.node}}</td>\n                        <td>{{task.disk}}</td>\n                        <td>{{task.taskStart}}</td>\n                        <td>{{task.taskEnd}}</td>\n                    </tr>\n                    </tbody>\n                </table>\n            </div>\n        </div>\n\n        <div class=\"three wide column\">\n            <div class=\"ui breadcrumb\">\n                &nbsp;\n            </div>\n            <div *ngIf=\"objectAsArray.length\" class=\"ui  segment\">\n                <div class=\"ui top right attached label\"><i class=\"big download icon\"></i></div>\n\n                <div class=\"ui  basic segment\">\n                    <span class=\"detail-left-header\">Detail</span>\n                </div>\n\n                <table class=\"ui celled compact fixed table\">\n                    <tr *ngFor=\"let property of objectAsArray; \">\n                        <td class=\"field\">{{property.key}}</td>\n                        <td>{{property.value}}</td>\n                    </tr>\n\n                </table>\n            </div>\n        </div>\n    </div>\n</div>\n\n",
                styles: ["a,a label{cursor:pointer}.field{background:#4a95c3;color:#fff}.ui .breadcrumb{font-size:1.28571429rem}.detail-right-header{float:right}.detail-left-header,.detail-right-header{color:#616161;font-size:1.2rem}.label{background:transparent}.active{color:grey!important;font-weight:400!important}"]
            },] }
];
DetailComponent.ctorParameters = () => [
    { type: Router },
    { type: ActivatedRoute },
    { type: DetailService }
];

class FilterPipe {
    transform(items, searchText) {
        if (!items) {
            return [];
        }
        if (!searchText) {
            return items;
        }
        searchText = searchText.toLowerCase();
        return items.filter(it => {
            return it.node.toLowerCase().includes(searchText)
                || it.master.toLowerCase().includes(searchText)
                || it.jobName.toLowerCase().includes(searchText)
                || it.status.toLowerCase().includes(searchText);
        });
    }
}
FilterPipe.decorators = [
    { type: Pipe, args: [{
                name: 'filter'
            },] }
];

class DetailModule {
}
DetailModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    FormsModule
                ],
                declarations: [
                    DetailComponent,
                    FilterPipe
                ],
                providers: [
                    DetailService
                ],
                exports: [
                    DetailComponent
                ]
            },] }
];

/**
 * Created by jayhamilton on 2/5/17.
 */
class DynamicFormPropertyComponent {
    constructor() {
        this.endPoints = [];
        this.updateEndPointList();
    }
    get isValid() {
        return this.form.controls[this.property.key].valid;
    }
    updateEndPointList() {
        /*
        this.endPointService.getEndPoints().subscribe(data => {

            this.endPoints = data['endPoint'].slice();

        });
         */
    }
    ngAfterViewInit() {
        //filter endpoints based on the gadgets tags
        let me = this;
        let eligibleEndpoints = [];
        this.endPoints.forEach(function (point, index, object) {
            let found = false;
            point['tags'].forEach(tag => {
                me.gadgetTags.forEach(_gt => {
                    if (_gt.name.trim().toLowerCase() === tag.name.trim().toLowerCase()) {
                        found = true;
                    }
                });
            });
            if (found) {
                eligibleEndpoints.push(point);
            }
        });
        this.endPoints = eligibleEndpoints.slice();
    }
}
DynamicFormPropertyComponent.decorators = [
    { type: Component, args: [{
                moduleId: module.id,
                selector: 'dashboard-df-property',
                template: "<div [formGroup]=\"form\">\n\n    <div class=\"field\">\n        <label [attr.for]=\"property.key\">{{property.label}}</label>\n\n        <div [ngSwitch]=\"property.controlType\">\n\n            <input *ngSwitchCase=\"'textbox'\"\n                   [formControlName]=\"property.key\"\n                   [id]=\"property.key\"\n                   [type]=\"property.type\">\n\n            <select *ngSwitchCase=\"'dropdown'\"\n                    [formControlName]=\"property.key\"\n                    [id]=\"property.key\">\n\n                <option *ngFor=\"let opt of property.options\"\n                        [value]=\"opt.value\">{{opt.key}}\n                </option>\n            </select>\n\n            <!--todo - fix checkbox type. The value is not properly set by the data so using hard coded type -->\n            <input (change)=\"property.value = $event.target.checked\"\n                   *ngSwitchCase=\"'checkbox'\"\n                   [(ngModel)]=\"property.value\"\n                   [attr.checked]=\"property.value ? true : null\"\n                   [formControlName]=\"property.key\"\n                   [id]=\"property.key\"\n                   [type]=\"'checkbox'\">\n\n            <input *ngSwitchCase=\"'number'\"\n                   [formControlName]=\"property.key\"\n                   [id]=\"property.key\"\n                   [type]=\"'number'\">\n\n\n            <input *ngSwitchCase=\"'hidden'\"\n                   [formControlName]=\"property.key\"\n                   [id]=\"property.key\"\n                   [type]=\"property.type\">\n\n            <select (change)=\"property.value = $event.target.toString()\"\n                    *ngSwitchCase=\"'dynamicdropdown'\"\n                    [formControlName]=\"property.key\"\n                    [id]=\"property.key\">\n\n                <option></option>\n                <option *ngFor=\"let opt of endPoints\"\n                        [value]=\"opt.name\">{{opt.name}}\n                </option>\n\n            </select>\n        </div>\n    </div>\n\n    <div *ngIf=\"!isValid\" [@showHideAnimation] class=\"ui error message\">{{property.label}} is required</div>\n\n\n</div>\n",
                animations: [
                    trigger('showHideAnimation', [
                        transition(':enter', [
                            style({ opacity: 0 }),
                            animate(750, style({ opacity: 1 }))
                        ]),
                        transition(':leave', [
                            animate(750, style({ opacity: 0 }))
                        ])
                    ])
                ],
                styles: [".field>label{color:#767676!important;font-size:1.1em!important;font-weight:300}.ui.tabular.menu .active.item{border-top-color:#2185d0!important;border-top-width:3px!important}.ui.segment{border:none}select{color:rgba(97,97,97,.68)!important;height:3em!important}input{border:thin solid #d3d3d3;border-radius:5px;line-height:2em;padding:5px;width:100%}"]
            },] }
];
DynamicFormPropertyComponent.ctorParameters = () => [];
DynamicFormPropertyComponent.propDecorators = {
    property: [{ type: Input }],
    form: [{ type: Input }],
    gadgetTags: [{ type: Input }]
};

/**
 * Created by jayhamilton on 2/3/17.
 */
class PropertyControlService {
    constructor() {
    }
    toFormGroupFromPP(propertyPages) {
        const group = {};
        propertyPages.forEach(propertyPage => {
            propertyPage.properties.forEach(property => {
                group[property.key] = property.required ? new FormControl(property.value
                    || '', Validators.required) : new FormControl(property.value
                    || '');
            });
        });
        return new FormGroup(group);
    }
}
PropertyControlService.decorators = [
    { type: Injectable }
];
PropertyControlService.ctorParameters = () => [];

/**
 * Created by jayhamilton on 2/5/17.
 */
class DynamicFormComponent {
    constructor(pcs, configService, changeDetectionRef) {
        this.pcs = pcs;
        this.configService = configService;
        this.changeDetectionRef = changeDetectionRef;
        this.updatePropertiesEvent = new EventEmitter(true);
        this.currentTab = 'run';
        this.state = 'inactive';
        this.lastActiveTab = {};
        this.form = new FormGroup({});
        this.payLoad = '';
        this.showMessage = false;
    }
    get isPropertyPageValid() {
        return this.form.valid;
    }
    /* better solution that solves error: Expression has changed after it was checked exception resolution*/
    ngAfterViewInit() {
        this.changeDetectionRef.detectChanges();
    }
    ngOnInit() {
        this.form = this.pcs.toFormGroupFromPP(this.propertyPages);
    }
    onSubmit() {
        this.payLoad = JSON.stringify(this.form.value);
        console.debug('Saving Form!');
        this.updatePropertiesEvent.emit(this.payLoad);
        console.debug('Sending configuration to the config service!');
        this.configService.notifyGadgetOnPropertyChange(this.payLoad, this.instanceId);
        if (this.payLoad) {
            this.showMessage = true;
            setTimeout(function () {
                this.showMessage = false;
            }.bind(this), 2000);
        }
    }
    setCurrentTab(tab) {
        this.currentTab = tab.groupId;
    }
}
DynamicFormComponent.decorators = [
    { type: Component, args: [{
                /* solves error: Expression has changed after it was checked exception resolution - https://www.youtube.com/watch?v=K_BRcal-JfI*/
                // changeDetection: ChangeDetectionStrategy.OnPush,
                moduleId: module.id,
                selector: 'dashboard-dynamic-form',
                template: "<div class=\"ui form\" style=\"text-align: left !important\">\n    <form (ngSubmit)=\"onSubmit()\" [formGroup]=\"form\">\n\n        <div #tabComponentTag class=\"proppages ui top attached tabular menu\">\n            <a #tabComponentTag (click)=\"setCurrentTab(tab)\" *ngFor=\"let tab of propertyPages; let i = index\"\n               [ngClass]=\"{'active': tab.groupId == currentTab }\"\n               attr.data-tab=\"{{tab.groupId}}\" class=\"item\">{{tab.displayName}} </a>\n        </div>\n\n        <div *ngFor=\"let page of propertyPages; let i = index\"\n             [@contentSwitch]=\"page.groupId == currentTab ? 'active':'inactive'\" [ngClass]=\"{'active': page.groupId == currentTab }\"\n             attr.data-tab=\"{{page.groupId}}\"\n             class=\"proppages ui bottom attached tab segment\">\n\n\n            <div *ngFor=\"let property of page.properties\"\n                 class=\"form-row\">\n                <dashboard-df-property\n                    [form]=\"form\"\n                    [gadgetTags]=\"gadgetTags\"\n                    [property]=\"property\"\n                >\n\n                </dashboard-df-property>\n                <br>\n            </div>\n\n        </div>\n        <div class=\"form-row field\">\n            <button [disabled]=\"!form.valid || !form.dirty\" class=\"ui green submit button\" type=\"submit\"> Save</button>\n        </div>\n    </form>\n\n    <div *ngIf=\"showMessage\" [@showHideAnimation] class=\"form-row\">\n        <br>\n        <div class=\"ui success message\">Saved!</div>\n    </div>\n\n</div>\n",
                animations: [
                    trigger('contentSwitch', [
                        state('inactive', style({
                            opacity: 0
                        })),
                        state('active', style({
                            opacity: 1
                        })),
                        transition('inactive => active', animate('750ms ease-in')),
                        transition('active => inactive', animate('750ms ease-out'))
                    ]),
                    trigger('showHideAnimation', [
                        transition(':enter', [
                            style({ opacity: 0 }),
                            animate(750, style({ opacity: 1 }))
                        ]),
                        transition(':leave', [
                            animate(750, style({ opacity: 0 }))
                        ])
                    ])
                ],
                providers: [PropertyControlService],
                styles: [".field>label{color:#767676!important;font-size:1.1em!important;font-weight:300}.ui.tabular.menu .active.item{border-top-color:#2185d0!important;border-top-width:3px!important}.ui.segment{border:none}select{color:rgba(97,97,97,.68)!important;height:3em!important}input{border:thin solid #d3d3d3;border-radius:5px;line-height:2em;padding:5px;width:100%}"]
            },] }
];
DynamicFormComponent.ctorParameters = () => [
    { type: PropertyControlService },
    { type: ConfigurationService },
    { type: ChangeDetectorRef }
];
DynamicFormComponent.propDecorators = {
    gadgetTags: [{ type: Input }],
    propertyPages: [{ type: Input }],
    instanceId: [{ type: Input }],
    updatePropertiesEvent: [{ type: Output }]
};

class DynamicFormModule {
}
DynamicFormModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    FormsModule,
                    ReactiveFormsModule
                ],
                declarations: [
                    DynamicFormComponent,
                    DynamicFormPropertyComponent
                ],
                providers: [PropertyControlService],
                exports: [
                    DynamicFormComponent,
                    DynamicFormPropertyComponent
                ]
            },] }
];

/**
 * Created by jayhamilton on 6/22/17.
 */
class GadgetBase {
    constructor(_gadgetInstanceService, _propertyService, changeDetectionRef, _optionsService) {
        this._gadgetInstanceService = _gadgetInstanceService;
        this._propertyService = _propertyService;
        this.changeDetectionRef = changeDetectionRef;
        this._optionsService = _optionsService;
        /**
         * Used to determine when to show the controls that appear in the gadgets
         * heading area. This is set by the mouseover/mouseout events.
         * @type {boolean}
         */
        this.showControls = false;
        /**
         * determines whether to show the gadgets property page
         * @type {boolean}
         */
        this.inConfig = false;
        /**
         * Determines if a gadget is runnning or not
         * @type {boolean}
         */
        this.inRun = false;
        /**
         * When a gadget is manually put into run mode this property will be used to
         * display a spinning icon and will be enabled between intiating an operation (run or stop)
         * to the operation is enabled
         * @type {boolean}
         */
        this.actionInitiated = false;
        /**
         * Gadgets that are of type realtime have a run/stop set of controls.
         * Those gadgets should set this property to true. This property's visibility
         * will also be controlled by whether the gadget's configuration form is valid.
         * @type {boolean}
         */
        this.showOperationControls = false;
        /**
         * This property is used to simply allow the gadget to not show any run/stop controls.
         * This is needed because the showOperationControls does something similar but not exactly the same.
         * The showOperationControls property allows the gadgetBase to determine if the run control, if the gadget
         * uses it, to be displayed when the gadget has a valid configuration.
         *
         * Default: true - Gadgets without a need for run/stop control should override this value.
         * @type {boolean}
         */
        this.gadgetHasOperationControls = true;
        /**
         * Most gadgets need configuration so gadgets that don't can override this property
         * @type {boolean}
         */
        this.showConfigurationControl = true;
        // internally controls dynamic form properties
        this.propertyPages = [];
        this.errorExists = false;
        this._optionsService.listenForGlobalOptionsChanges().subscribe(options => {
            /**
             * This is called when there is a change to the options tab within the configuration modal.
             * The following method needs to also be called when the gadget is initially instantiated.
             * When the gadget is instantiated the option values need to come from the persistent store.
             */
            this.updateGadgetWithGlobalOptions(options);
        });
        this.updateGadgetWithGlobalOptions(this._optionsService.getBoardOptions());
    }
    ngOnInit() {
        this.toggleConfigMode();
        this.changeDetectionRef.detectChanges();
    }
    ngAfterViewInit() {
        if (this.propertyPageForm) {
            if (this.propertyPageForm.isPropertyPageValid) {
                this.toggleConfigMode();
                this.changeDetectionRef.detectChanges();
                this.showOperationControls = true;
                this.changeDetectionRef.detectChanges();
            }
            else {
                this.showOperationControls = false;
                this.changeDetectionRef.detectChanges();
            }
        }
        this.preRun();
    }
    initializeState() {
    }
    toggleConfigMode() {
        if (!this.inConfig) {
            this.initializeProperties();
        }
        this.inConfig = !this.inConfig;
    }
    initializeProperties() {
        if (this.propertyPages.length === 0 && this.config.propertyPages) {
            this._propertyService.setPropertyPagesAndProperties(this.config.propertyPages, this.propertyPages);
        }
    }
    handleError(error) {
        this.inRun = false;
        this.actionInitiated = false;
        this.errorExists = true;
        this.errorObject = error;
    }
    initializeRunState(forceRunState) {
        this.errorExists = false;
        this.actionInitiated = true;
        this.inConfig = false;
        if (forceRunState) {
            this.setInRunState();
        }
    }
    setInRunState() {
        this.inRun = true;
        this.actionInitiated = false;
    }
    setStopState(longRunningStopAction) {
        /**
         *  If the gadget indicates longRunningStopAction then the gadget has to set this value to
         *  false once the operation is complete
         */
        this.actionInitiated = longRunningStopAction;
        this.inRun = false;
    }
    remove() {
        this._gadgetInstanceService.removeInstance(this.instanceId);
    }
    showGadgetControls(enable) {
        this.showControls = enable;
    }
    /**
     * called from cell.component after the gadget is created during runtime
     * intanceId, config, title and endpoint are common to all gadgets. Once the gadgets are configured
     * we give them an opportunity to perform an action during the preRun() method. For example,
     * the statistic gadget uses preRun() to make a single call to the endpoint to update its display.
     * */
    configureGadget(instanceId, config, tags) {
        this.instanceId = instanceId;
        this.config = config;
        this.gadgetTags = tags.slice();
        this.setTitle(this.getPropFromPropertyPages('title'));
        this.setEndPoint(this.getPropFromPropertyPages('endpoint'));
        /**
         *  Todo - remove this prerun call and refactor remaining code. Prerun was called twice and had an impact on the barchart api calls.
         *  API calls continued after route changes which is undesirable. See ngAfterViewInit where it is also called from.
         */
        //this.preRun();
    }
    ngOnDestroy() {
    }
    updateGadgetWithGlobalOptions(options) {
        this.globalOptions = Object.assign({}, options);
    }
    setEndPoint(endpoint) {
        /*this._endPointService.getEndPoints().subscribe(data => {

            if (data['endPoint']) {
                data['endPoint'].forEach(item => {
                    if (item.name === endpoint) {
                        this.endpointObject = item;
                    }
                });
            }
        });*/
    }
    getEndPoint() {
        return this.endpointObject;
    }
    setTitle(title) {
        this.title = title;
    }
    getPropFromPropertyPages(prop) {
        for (let x = 0; x < this.config.propertyPages.length; x++) {
            for (let i = 0; i < this.config.propertyPages[x].properties.length; i++) {
                if (this.config.propertyPages[x].properties[i].key === prop) {
                    return this.config.propertyPages[x].properties[i].value;
                }
            }
        }
        return 'Unknown';
    }
}
GadgetBase.propDecorators = {
    propertyPageForm: [{ type: ViewChild, args: [DynamicFormComponent, { static: true },] }]
};

class ErrorHandlerComponent {
    constructor() {
    }
    closeMessage() {
        this.errorExists = false;
    }
}
ErrorHandlerComponent.decorators = [
    { type: Component, args: [{
                moduleId: module.id,
                selector: 'dashboard-error-handler',
                template: "<div *ngIf=\"errorExists\" [@error]=\"errorExists\" class=\"ui bottom attached negative message\">\n    <i (click)=\"closeMessage()\" class=\"close icon\"></i>\n    <div class=\"header\">\n        {{errorObject.summary}}\n    </div>\n    <p>\n        {{errorObject.detail}}\n    </p>\n\n    <p>\n        resource: {{errorObject.resource}}\n    </p>\n\n    <span class=\"error-heading\"> Potential solutions</span>\n    <hr style=\"color:rgba(194,197,200,0.37)\">\n    <div *ngFor=\"let solution of errorObject.solutions\" style=\"text-align: left !important\">\n        <br><span class=\"error-heading\">article:&nbsp;</span> {{solution.link}}\n        <br><span class=\"error-heading\">summary:&nbsp;</span> {{solution.summary}}\n    </div>\n</div>\n",
                animations: [
                    trigger('error', [
                        transition(':enter', [
                            style({ opacity: 0 }),
                            animate('1000ms', style({ opacity: 1 }))
                        ]),
                        transition(':leave', [
                            style({ opacity: 1 }),
                            animate('1000ms', style({ opacity: 0 }))
                        ])
                    ])
                ],
                styles: [".error-heading{color:grey!important;text-align:left}"]
            },] }
];
ErrorHandlerComponent.ctorParameters = () => [];
ErrorHandlerComponent.propDecorators = {
    errorObject: [{ type: Input }],
    errorExists: [{ type: Input }]
};

class ErrorHandlerModule {
}
ErrorHandlerModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ],
                declarations: [ErrorHandlerComponent],
                exports: [ErrorHandlerComponent]
            },] }
];

/*
 * Public API Surface of my-lib
 */

/**
 * Generated bundle index. Do not edit.
 */

export { AboutComponent, AboutModule, AboutService, AddGadgetComponent, AddGadgetModule, AddGadgetService, BoardComponent, BoardLayoutManagerComponent, BoardModule, BoardsConfigurationTabComponent, CapitalizeFirstPipe, CellComponent, ConfigurationComponent, ConfigurationModule, DetailComponent, DetailModule, DetailService, DynamicFormComponent, DynamicFormModule, DynamicFormPropertyComponent, ErrorHandlerComponent, ErrorHandlerModule, FacetComponent, FacetModule, FacetTagProcessor, FilterListComponent, FilterPipe, FilterTagComponent, GadgetBase, GadgetConfigModel, GadgetFactory, GadgetHeaderComponent, GadgetInstanceService, GadgetOperationComponent, GadgetPropertyService, GadgetSharedModule, GridComponent, GridModule, HelpModalComponent, LayoutModule, MenuComponent, MenuEventService, MenuModule, NotificationComponent, NotificationDetailComponent, NotificationService, OptionsConfigurationTabComponent, OptionsService, RuntimeService, ToastComponent, ToastModule, TypeAheadInputComponent, TypeAheadInputModule, VisDrillDownComponent, ToastService as ɵa, ConfigurationService as ɵb, NotificationModule as ɵc, DataListModule as ɵd, DataListComponent as ɵe, ObservableWebSocketService as ɵf, ReversePipe as ɵg, PropertyControlService as ɵh };
//# sourceMappingURL=ngx-dynamic-dashboard.js.map
